      SUBROUTINE FITWCS(Npts,dra,ddec,dx,dy,derror,ctype,
     &   Imat,Ibad,dpar,Nfree,Itoss,Xchi)
      INTEGER   Npts, Ibad(*), Imat, Nfree, Itoss
      DOUBLE PRECISION dra(*), ddec(*), dx(*), dy(*)
      DOUBLE PRECISION derror(*), dpar(8)
      CHARACTER ctype*(*)
      REAL      Xchi
C---
C Least squares fitting to solve for FITS WCS.
C---
C Npts    I     The number of sources
C Dra     I     The "true" Dec of the sources
C Ddec    I     The "true" RA of the sources
C Dx      I     X location of sources
C Dy      I     Y location of sources
C Derror  I     Uncertainty (in pixels) typically PSF_SIG/SQRT(counts)
C Ctype   I     Coordinate transmation (typically -TAN)
C Imat    I     =1 if using WCS matrix, =0 for old WCS
C Ibad    I/O   Flag <>0 for points not included in the fit
C Dpar    I/O   The WCS parameters
C Nfree     O   Nunber of free paraameters
C Itoss     O   Number of points tossed out.
C Xchi      O   Final chi^2
C---
C 2004-Apr-19 - Extracted from FNDWCS [AFT]
C----
      INTEGER   MXFAC
      PARAMETER (MXFAC=7)
C
      DOUBLE PRECISION dstep(8), dsum(-1:1)
      DOUBLE PRECISION dchi, dsav, dsum0, dxpix, dypix
      REAL      factor(MXFAC)
      REAL      emax
      INTEGER   i, inew, iclast, icnt, imax
      INTEGER   ipar, iscale, istat, itry, npar, ntot
C
      DATA factor/1.0, 0.1, 0.01, 1.0e-3, 1.0e-4, 1.0e-5, 1.0e-6/
      DATA dstep/ 10., 0.0, 10., 0.0, 10., 1.0d-3, 1.0d-3, 1.0d-3/
C---
      IF ( imat.NE.0 ) THEN
         dstep(5) = 1.0d-3
         npar = 8
         nfree = 6
      ELSE
C This is rotation angle.
         dstep(5) = 10.0
         npar = 7
         nfree = 5
      END IF
      IF ( npts.LE.3 ) THEN
         dstep(5)=0.0
         dstep(6)=0.0
         dstep(7)=0.0
         dstep(8)=0.0
         nfree = 2
      ELSE
         dstep(5)=1.0d-3
         dstep(6)=1.0d-3
         dstep(7)=1.0d-3
         dstep(8)=1.0d-3
      END IF
      Itoss = 0
  340 CONTINUE
      WRITE(*,*) 'Fitting...'
      iclast = 0
      DO iscale=1,MXFAC
         icnt = 0
  350    CONTINUE
         inew = 0
         DO 390 ipar=1,npar
C Either RA, Dec or Xref, Yref should be kept constant.
            IF ( dstep(ipar).LE.0 ) GOTO 390
            dsav = dpar(ipar)
            DO itry=-1,1
               dpar(ipar) = dsav + itry*dstep(ipar)*factor(iscale)
               dsum(itry) = 0.0d0
               DO i=1,Npts
                  istat = 0
                  IF ( Ibad(i).EQ.0 ) THEN
                     IF ( Imat.NE.0 ) THEN
            CALL afxypx(Dra(i),Ddec(i),dpar(2),dpar(4),dpar(1),dpar(3),
     &     dpar(5),dpar(6),dpar(7),dpar(8),ctype,dxpix,dypix,istat)
                     ELSE
                        CALL ftxypx(dra(i),ddec(i),dpar(2),dpar(4),
     &  dpar(1),dpar(3),dpar(6),dpar(7),dpar(5),ctype,dxpix,dypix,istat)
                     END IF
                     dsum(itry)=dsum(itry)+
     &              ((dx(i)-dxpix)**2+(dy(i)-dypix)**2)/derror(i)**2
                  END IF
               END DO
            END DO
            IF ( dsum(-1).LT.dsum(0) ) THEN
               dpar(ipar) = dsav - dstep(ipar)*factor(iscale)
               inew = ipar
            ELSE IF ( dsum(0).GT.dsum(+1) ) THEN
               dpar(ipar) = dsav + dstep(ipar)*factor(iscale)
               inew = ipar
            ELSE
               dpar(ipar) = dsav
            END IF
  390    CONTINUE
         IF ( inew.NE.0 ) THEN
            IF ( inew.EQ.iclast ) THEN
               icnt = icnt + 1
               IF ( icnt.GT.20 ) GOTO 410
            ELSE
               icnt = 0
            END IF
            iclast = inew
            GOTO 350
         END IF
      END DO
C
      WRITE(*,*)
      inew = 0
  410 CONTINUE
      emax = 0.0
      imax = 0
      dsum0 = 0.0d0
      ntot = 0
      DO i=1,Npts
         IF ( Ibad(i).EQ.0 ) THEN
            istat = 0
            IF ( imat.NE.0 ) THEN
           CALL afxypx(Dra(i),Ddec(i),dpar(2),dpar(4),dpar(1),dpar(3),
     &     dpar(5),dpar(6),dpar(7),dpar(8),ctype,dxpix,dypix,istat)
            ELSE
               CALL ftxypx(dra(i),ddec(i),dpar(2),dpar(4),dpar(1),
     &        dpar(3),dpar(6),dpar(7),dpar(5),ctype,dxpix,dypix,istat)
            END IF
            dchi=((dx(i)-dxpix)**2+(dy(i)-dypix)**2)/derror(i)**2
C      write(*,*) i,dchi,dx(i)-dxpix,dy(i)-dypix
            ntot = ntot + 2
            dsum0 = dsum0 + dchi
            IF ( dchi.GT.emax ) THEN
               emax = dchi
               imax = i
            END IF
         END IF
      END DO
C If we have a very good fit, we can stop
      IF ( dsum0.LT.0.01 ) GOTO 900
C
C Apply the F-test on the point with the largest error.  If omitting
C the point decreases F enough then toss the point and recalculate.
      IF ( (ntot-7)*emax/(dsum0-emax).GT.7.0 ) THEN
         WRITE(*,*) 'Toss point ',imax,', F=',(ntot-7)*emax/(dsum0-emax)
         Itoss = Itoss + 1
         Ibad(imax) = 1
         inew = 1
         GOTO 410
      END IF
C
C If any points tossed then recalculate from the beginning.
      IF ( inew.NE.0 ) GOTO 340
C      WRITE(*,*) dsum0, emax
C
  900 CONTINUE
      Xchi = dsum0
      RETURN
      END
