C Program FNDWCS, solves for FITS WCS.
C The input file should be ASCII text and contain 4 columns:
C Xpixel, Ypixel, RA(degs), Dec(degs)
C
C If the first file is called name.txt, then the program searches for
C name.xco, and tries to read it.  name.xco should contain the initial
C guess for the WCS coordinates, expressed in a lextrct indirect command
C file that starts with "SET WCS" and should contain 8 lines setting all
C the initial values (order is not important).
C
C A simple grid search is made for the best fitting parameters.
C---
C 2001-06-25 - [AFT]
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INTEGER   MXFAC, MXROW
      PARAMETER (MXFAC=7, MXROW=100000)
      DOUBLE PRECISION DPNUM
      INTEGER   LENACT
C
      CHARACTER cpar(7)*16
      CHARACTER cbuf*256, cfile*256, cout*256
      CHARACTER ctmp*64
      CHARACTER ccol*16
      CHARACTER carg*8, ctype*8
      DOUBLE PRECISION dra(MXROW), ddec(MXROW), dx(MXROW), dy(MXROW)
      DOUBLE PRECISION derror(MXROW)
      DOUBLE PRECISION dpar(7)
      DOUBLE PRECISION dnul
      REAL      ang1, ang2, rtd, xchi, xt, yt
      INTEGER   ibad(MXROW)
      INTEGER   naxes(99)
      INTEGER   i, ibits, iblk, idcol, ier, iext
      INTEGER   ifits, igcnt, ipar, ipcnt, ircol
      INTEGER   istat, itoss, ixcol, iycol
      INTEGER   kp, larg, lbuf, lfile, lout, ltmp, luni, luno
      INTEGER   naxis, nfree, npts, nrows, ntot
      LOGICAL   qbad, qsimp, qxtend
      DATA cpar/'XPix0','XRef','YPix0','YRef','ROta','XDel','YDel'/
      DATA      rtd/57.2957795/
C---
   11 FORMAT(A)
C---
      CALL GETLUN(luni)
  100 CONTINUE
      CALL GTBUF('Position file, [outfile]:',ier)
      IF ( ier.NE.0 ) GOTO 900
      CALL GTCHAR(cfile, lfile)
      IF ( lfile.LE.0 ) GOTO 900
C
      istat = 0
      CALL FTOPEN(luni, cfile, 0, iblk, istat)
      CALL FTGHPR(luni,99,qsimp,ibits,naxis,naxes,ipcnt,igcnt,qxtend,
     &            istat)
      IF ( istat.EQ.0 ) THEN
         ifits = 1
C FITS file.  Go to 2nd HDU.
         istat = 0
         CALL FTMAHD(luni, 2, iext, istat)
         CALL FTGKYJ(luni, 'NAXIS2', nrows, ctmp, istat)
         nrows = MIN(nrows, MXROW)
         istat = 0
         CALL FTGCNN(luni, .FALSE., 'RA',  ccol, ircol, istat)
         CALL FTGCNN(luni, .FALSE., 'DEC', ccol, idcol, istat)
C
         IF ( istat.NE.0 ) THEN
            istat = 0
            CALL FTGCNN(luni, .FALSE., 'RA(ICRS)', ccol, ircol, istat)
            CALL FTGCNN(luni, .FALSE., 'DE(ICRS)', ccol, idcol, istat)
         END IF
C
         IF ( istat.NE.0 ) THEN
            istat = 0
            CALL FTGCNN(luni, .FALSE., 'RAJ2000', ccol, ircol, istat)
            CALL FTGCNN(luni, .FALSE., 'DEJ2000', ccol, idcol, istat)
         END IF
C
         CALL FTGCNN(luni, .FALSE., 'X', ccol, ixcol, istat)
         CALL FTGCNN(luni, .FALSE., 'Y', ccol, iycol, istat)
         IF ( istat.NE.0 ) THEN
            WRITE(*,*) 'Failed to find all needed columns.'
            GOTO 100
         END IF
      ELSE
         istat = 0
         CALL FTCLOS(luni, istat)
         ifits = 0
         CALL OPENWR(luni,cfile,'OLD',' ',' ',0,1,ier)
         IF ( ier.NE.0 ) THEN
            WRITE(*,*) 'Unable to open ',cfile(:lfile)
            WRITE (*,*) 'IOSTAT from OPEN is', ier
            CALL FRELUN(luni)
            GOTO 100
         END IF
         nrows = MXROW
      END IF
C
      WRITE(*,*)
      istat = 0
      dnul = NO
      ntot = 0
      npts = 0
      DO 190 i=1, nrows
         IF ( ifits.EQ.0 ) THEN
            READ(luni, 11, IOSTAT=istat) cbuf
            IF ( istat.NE.0 ) GOTO 200
            kp = 0
            CALL ALFSKS(cbuf, lbuf, kp)
            IF ( cbuf(kp+1:kp+1).EQ.'!' ) GOTO 190
            lbuf = LENACT(cbuf)
            npts = npts + 1
            CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
            dx(npts) = DPNUM(ctmp, ltmp, ier)
            CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
            dy(npts) = DPNUM(ctmp, ltmp, ier)
            CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
            dra(npts) = DPNUM(ctmp, ltmp, ier)
            CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
            ddec(npts) = DPNUM(ctmp, ltmp, ier)
         ELSE
            CALL FTGCVD(luni,ircol,i,1,1,dnul,dra(i), qbad,istat)
            CALL FTGCVD(luni,idcol,i,1,1,dnul,ddec(i),qbad,istat)
            CALL FTGCVD(luni,ixcol,i,1,1,dnul,dx(i),  qbad,istat)
            CALL FTGCVD(luni,iycol,i,1,1,dnul,dy(i),  qbad,istat)
            IF ( istat.NE.0 ) GOTO 200
            npts = npts + 1
         END IF
         derror(i) = 1.0d0
         ibad(i) = 0
         xt = dx(i)
         yt = dy(i)
         IF ( dra(i).LT.0.0 .OR. xt.EQ.NO .OR. yt.EQ.NO ) THEN
            ibad(i) = 1
         ELSE
            ntot = ntot + 1
         END IF
  190 CONTINUE
C
  200 CONTINUE
      IF ( ifits.EQ.0 ) THEN
         CLOSE(unit=luni)
      ELSE
         CALL FTCLOS(luni, istat)
      END IF
      CALL FRELUN(luni)
      WRITE(*,*) 'npts=',npts,', ntot=',ntot
C---
C Default parameters.
      ctype = '-TAN'
      dpar(1) = dx(npts/2)
      dpar(2) = dra(npts/2)
      dpar(6) = -0.00017
      dpar(3) =  dy(npts/2)
      dpar(4) = ddec(npts/2)
      dpar(7) = -dpar(6)
      ang1 = atan2(ddec(npts)-dpar(3),dra(npts)-dpar(1))
      ang2 = atan2(dx(npts)-dpar(4),dy(npts)-dpar(2))
      dpar(5) =  ang1*rtd-ang2*rtd
      IF ( dpar(5).LT.0.0 ) dpar(5) = dpar(5) + 360.0d0
C
C Try to open an lextrct *.xco file that contains the current guess
      CALL XTEND(cfile,'.xco')
      CALL OPENWR(luni,cfile,'OLD',' ',' ',0,1,ier)
      IF ( ier.EQ.0 ) THEN
         DO i=1,10
            READ(luni, 11, IOSTAT=ier) cbuf
            IF ( ier.NE.0 ) GOTO 330
            lbuf = LENACT(cbuf)
C Skip first two arguments which should be "SET WCS"
            kp = 0
            CALL ALFSKS(cbuf, lbuf, kp)
            CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
            CALL UPC(ctmp(:ltmp))
            IF ( ctmp(1:3).NE.'WCS' ) THEN
               CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
            END IF
            CALL ALF(cbuf, lbuf, kp, carg, larg)
            CALL UPC(carg(:larg))
            IF ( carg(1:larg).EQ.'TYPE' ) THEN
               CALL ALF(cbuf, lbuf, kp, carg, larg)
               ctype = carg
            ELSE
               DO ipar=1,7
                  ctmp = cpar(ipar)
                  ltmp = LENACT(ctmp)
                  CALL UPC(ctmp(:ltmp))
                  IF ( ctmp(1:ltmp).EQ.carg(:ltmp) ) THEN
                     CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
                     IF ( ltmp.LT.LEN(ctmp) ) ctmp(ltmp+1:) = ','
                     dpar(ipar) = DPNUM(ctmp, ltmp, ier)
                  END IF
               END DO
            END IF
         END DO
  330    CONTINUE
         CLOSE(unit=luni)
      END IF
C
      WRITE(*,*)
      WRITE(*,*) 'Starting with:'
      DO ipar=1,7
         WRITE(*,*) dpar(ipar)
      END DO
C
      CALL FITWCS(npts,dra,ddec,dx,dy,derror,ctype,0,ibad,
     &   dpar,nfree,itoss,xchi)
C
      CALL GTCHAR(cout, lout)
      IF ( lout.GT.0 ) THEN
         CALL GETLUN(luno)
         CALL OPENWR(luno,cout,'NEW',' ',' ',0,0,ier)
         IF ( ier.NE.0 ) lout=0
      END IF
      WRITE(*,*)
      WRITE(*,*) 'Best fit:'
      WRITE(*,701) 'SET WCS  TYpe   ',ctype
  701 FORMAT(A,A)
      WRITE(*,711) cpar(5)(:5),dpar(5)
  711 FORMAT('SET WCS  ',A5,2X,F9.3)
      WRITE(*,721) cpar(1)(:5),dpar(1)
  721 FORMAT('SET WCS  ',A5,2X,F9.2)
      WRITE(*,731) cpar(2)(:5),dpar(2)
  731 FORMAT('SET WCS  ',A5,2X,F10.6)
      WRITE(*,741) cpar(6)(:5),dpar(6)
  741 FORMAT('SET WCS  ',A5,2X,F13.9)
      WRITE(*,721) cpar(3)(:5),dpar(3)
      WRITE(*,731) cpar(4)(:5),dpar(4)
      WRITE(*,741) cpar(7)(:5),dpar(7)
      WRITE(*,*)
      WRITE(*,791) xchi,ntot-itoss
  791 FORMAT('! W-VAR=',F10.3,',  nfit=',I10)
C
      IF ( lout.GT.0 ) THEN
         WRITE(luno,701) 'SET WCS  TYpe   ',ctype
         WRITE(luno,711) cpar(5)(:5),dpar(5)
         WRITE(luno,721) cpar(1)(:5),dpar(1)
         WRITE(luno,731) cpar(2)(:5),dpar(2)
         WRITE(luno,741) cpar(6)(:5),dpar(6)
         WRITE(luno,721) cpar(3)(:5),dpar(3)
         WRITE(luno,731) cpar(4)(:5),dpar(4)
         WRITE(luno,741) cpar(7)(:5),dpar(7)
         WRITE(luno,791) SQRT(xchi/(ntot-1.)),ntot
         CLOSE(UNIT=luno)
         CALL FRELUN(luno)
      END IF
C---
  900 CONTINUE
      CALL EDICOM('off',3)
      END
