C Program near2.  Takes two FITS files as input (the Object list and
C Catalog files).  For each object in the object list, find the closest
C object in the Catalog and output distance.
C-
C The probability of the nearest source being at distance r is given
C by the probability of seeing one and one source within that radius.
C P(x;m) = (m^x/x!) exp(-m)
C P(1;m) = m*exp(-m)
C If the surface density of sources is uniform s, then m = pi r^2 s.
C---
C 2004-03-23 - From nearest [AFT]
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INTEGER   MXCOL, MXCAT, MXMAT, MXNAX
      PARAMETER (MXCOL=30, MXCAT=100000, MXMAT=100000, MXNAX=99)
C
      CHARACTER ctypes(MXCOL)*16, cform(MXCOL)*16
      CHARACTER cmd(10)*64
      CHARACTER ccat*256, cobj*256, cvalue*64, ctmp*64
      CHARACTER ccol*16, ckey*16, ctype*16
      DOUBLE PRECISION dde(MXCAT), dra(MXCAT)
      DOUBLE PRECISION dd(MXMAT), dr(MXMAT), dx(MXMAT), dy(MXMAT)
      DOUBLE PRECISION derr(MXMAT), dpar(8)
      DOUBLE PRECISION dcd11, dcd12, dcd21, dcd22
      DOUBLE PRECISION ddobj, dnul, drobj, drota, dxpos, dypos
      DOUBLE PRECISION dxdel, dxpix0, dxref, dydel, dypix0, dyref
      DOUBLE PRECISION dxpix, dypix, dsum1, dsum2
      REAL      yplt(MXMAT,2)
      REAL      fnul, psf, rnear, r2cur, r2min, rcnts, rcos
      REAL      rnul, rmax, rtd, scale, sigsys
      REAL      tlmax, xchi, xpos, ypos
      INTEGER   ibad(MXMAT), ic(MXMAT), io(MXMAT)
      INTEGER   naxes(MXNAX), iery(10)
      INTEGER   i, ians, ibest, ibits, iblk
      INTEGER   icr0, icd0, iccnts, icdcat, icdobj, icpsf, icrcat
      INTEGER   icrobj, icxobj, icyobj
      INTEGER   idoxy, ier, iext, igcnt, icnear, ihsav
      INTEGER   imat, ipcnt, irow, istat, itoss, iwcs
      INTEGER   jrow, lcat, lobj, ltmp, lunc, luno
      INTEGER   naxis, ncats, ncmd, nfield, nfree, nmat, nobjs, nplt
      LOGICAL   qbad, qsimp, qxtend
      DATA  rtd/57.2957795/
C---
   11 FORMAT(A)
C---
      rmax = 2.0
      sigsys = 0.1
      luno = 1
      lunc = 3
  100 CONTINUE
      CALL GTBUF('Object list, Catalog:', ier)
      IF ( ier.LT.0 ) GOTO 900
      CALL GTCHAR(cobj, lobj)
      IF ( lobj.LE.0 ) GOTO 900
      CALL GTCHAR(ccat, lcat)
      IF ( lcat.LE.0 ) GOTO 900
C
      istat = 0
      CALL FTOPEN(luno, cobj(:lobj), 0, iblk, istat)
      IF ( istat.NE.0 ) THEN
         WRITE(*,*) 'Unable to open ',cobj(:lobj),', istat=',istat
         GOTO 100
      END IF
      CALL FTOPEN(lunc, ccat(:lcat), 0, iblk, istat)
      IF ( istat.NE.0 ) THEN
         WRITE(*,*) 'Unable to open ',ccat(:lcat),', istat=',istat
         GOTO 100
      END IF
C---
C Read keywords from primary header
C
      istat = 0
      CALL FTGHPR(luno,MXNAX,qsimp,ibits,naxis,naxes,ipcnt,igcnt,qxtend,
     &            istat)
      IF ( naxis.NE.0 .OR. .NOT.qxtend ) THEN
         WRITE(*,*) 'Not a binary table FITS file.', naxis,qxtend
         GOTO 100
      END IF
C
      CALL FTGHPR(lunc,MXNAX,qsimp,ibits,naxis,naxes,ipcnt,igcnt,qxtend,
     &            istat)
      IF ( naxis.NE.0 .OR. .NOT.qxtend ) THEN
         WRITE(*,*) 'Not a binary table FITS file.', naxis,qxtend
         GOTO 100
      END IF
C
C Assume data is in first extension.  Note, reading NAXIS2 works for
C both FITS binary and ASCII tables.
C
      istat = 0
      CALL FTMAHD(luno, 2, iext, istat)
      CALL FTGKYJ(luno,'NAXIS2',nobjs,ctmp,istat)
      CALL FTGKYJ(luno,'TFIELDS',nfield,ctmp,istat)
      CALL FTMAHD(lunc, 2, iext, istat)
      CALL FTGKYJ(lunc,'NAXIS2',ncats,ctmp,istat)
C
C We are now at the EVENTS extension.  Scan TTYPE keywords until
C we find the columns we need.
      istat = 0
      icpsf = 0
      CALL FTGCNN(luno, .FALSE., 'PSF_SIG',  ccol, icpsf, istat)
      iccnts = 0
      CALL FTGCNN(luno, .FALSE., 'COUNTS',  ccol, iccnts, istat)
C
      icrobj = 0
      icdobj = 0
      icxobj = 0
      icyobj = 0
      DO i=1,99
C Construct keyword name
         istat = 0
         CALL FTKEYN('TTYPE',i,ckey,istat)
C Search for keyword value
         CALL FTGKYS(luno,ckey,cvalue,ctmp,istat)
         IF (istat .GT. 0) GOTO 130
C
         CALL UPC(cvalue)
C Prevent matching original RA/DEC which could be called RA0/DEC0
         IF ( cvalue(1:2).EQ.'RA' .AND. cvalue(1:3).NE.'RA0') THEN
            icrobj = i
         ELSE IF( cvalue(1:2).EQ.'DE' .AND. cvalue(1:4).NE.'DEC0') THEN
            icdobj = i
         ELSE IF ( cvalue.EQ.'X' ) THEN
            icxobj = i
         ELSE IF ( cvalue.EQ.'Y' ) THEN
            icyobj = i
         END IF
      END DO
C
C Try to read the WCS system.  This is required if file only contains X,Y.
  130 CONTINUE
      imat=0
      istat = 0
      ctype = '-TAN'
      CALL FTKEYN('TCTYP',icxobj,ckey,istat)
      ckey(1:5) = 'TCRVL'
      CALL FTGKYD(luno, ckey, dxref, ctmp,istat)
      ckey(1:5) = 'TCRPX'
      CALL FTGKYD(luno,ckey, dxpix0, ctmp,istat)
C
      CALL FTKEYN('TCTYP',icyobj,ckey,istat)
      ckey(1:5) = 'TCRVL'
      CALL FTGKYD(luno, ckey, dyref, ctmp,istat)
      ckey(1:5) = 'TCRPX'
      CALL FTGKYD(luno,ckey, dypix0, ctmp,istat)
C
      IF ( istat.EQ.0 ) THEN
         CALL FTKEYN('TCDLT',icxobj,ckey,istat)
         CALL FTGKYD(luno,ckey, dxdel, ctmp,istat)
         ckey(1:5) = 'TCROT'
         CALL FTGKYD(luno,ckey, drota, ctmp,istat)

         CALL FTKEYN('TCDLT',icyobj,ckey,istat)
         CALL FTGKYD(luno,ckey, dydel, ctmp,istat)
         IF ( istat.NE.0 ) THEN
            istat = 0
            CALL FTKEYN('CD1_1',icxobj,ckey,istat)
            CALL FTGKYD(luno,ckey, dcd11, ctmp,istat)
            ckey(1:5) = 'CD1_2'
            CALL FTGKYD(luno,ckey, dcd12, ctmp,istat)
C
            CALL FTKEYN('CD2_1',icyobj,ckey,istat)
            CALL FTGKYD(luno,ckey, dcd21, ctmp,istat)
            ckey(1:5) = 'CD2_2'
            CALL FTGKYD(luno,ckey, dcd22, ctmp,istat)
            IF ( istat.EQ.0 ) imat=1
         END IF
      END IF
      IF ( istat.NE.0 ) THEN
         WRITE(*,*) 'Failed to read required WCS keywords.'
         iwcs = 0
         scale = 1.0
      ELSE
         iwcs = 1
C Scale is arcsec/pixel
         IF ( imat.NE.0 ) THEN
            scale = 3600.*SQRT(ABS(dcd11*dcd11+dcd21*dcd21))
         ELSE
            scale = 3600.*SQRT(ABS(dxdel*dydel))
         END IF
      END IF
      IF ( iwcs.EQ.0 .AND. (icrobj.EQ.0 .OR. icdobj.EQ.0) ) THEN
         GOTO 900
      END IF
      dpar(1) = dxpix0
      dpar(2) = dxref
      dpar(3) = dypix0
      dpar(4) = dyref
      IF ( imat.NE.0 ) THEN
         dpar(5) = dcd11
         dpar(6) = dcd12
         dpar(7) = dcd21
         dpar(8) = dcd22
      ELSE
         dpar(5) = drota
         dpar(6) = dxdel
         dpar(7) = dydel
      END IF
C
      istat = 0
      icrcat = 0
      icdcat = 0
      DO i=1,99
C Construct keyword name
         CALL FTKEYN('TTYPE',i,ckey,istat)
C Search for keyword value
         CALL FTGKYS(lunc,ckey,cvalue,ctmp,istat)
         IF (istat .GT. 0) GOTO 160
C
         CALL UPC(cvalue)
         IF ( cvalue(1:2).EQ.'RA' ) THEN
            icrcat = i
         ELSE IF ( cvalue(1:2).EQ.'DE' ) THEN
            icdcat = i
         END IF
      END DO
C
  160 CONTINUE
      WRITE(*,*) 'Columns:'
      IF ( iwcs.NE.0 .AND. icxobj.NE.0 .AND. icyobj.NE.0 ) THEN
         WRITE(*,*) 'Object :   X   =',icxobj,',   Y  =',icyobj
         idoxy = 1
      ELSE
         WRITE(*,*) 'Object :  RA   =',icrobj,', DEC  =',icdobj
         idoxy = 0
      END IF
      WRITE(*,*) 'Catalog:  RA   =',icrcat,', DEC  =',icdcat
C
C Now create the new (empty) column:
      ians=-1
C
C Now read selected data for each event and process.
      WRITE(*,*) '! ',nobjs,' in ',cobj(:lobj)
      WRITE(*,*) '! ',ncats,' in ',ccat(:lcat)
      IF ( iwcs.EQ.0 ) THEN
         WRITE(*,*) 'NOTE--PSF_SIG is in original pixels.'
      END IF
      WRITE(ctmp, 161) rmax,sigsys
  161 FORMAT('Maximum radius (',F10.1,
     &   '), systematic error(',F10.2,'):')
      CALL GTBUF(ctmp, ier)
      CALL GTREAL(rmax, ier)
      CALL GTREAL(sigsys, ier)
C
      istat = 0
      DO irow=1,ncats
         istat = 0
         CALL FTGCVD(lunc,icrcat,irow,1,1,dnul,dra(irow), qbad,istat)
         CALL FTGCVD(lunc,icdcat,irow,1,1,dnul,dde(irow), qbad,istat)
      END DO
      WRITE(*,*)
      nmat = 0
      tlmax = 0.0
      dsum1 = 0.0d0
      dsum2 = 0.0d0
      WRITE(*,*) '       Object                  Catalog             '//
     &           '    dist.    error'
      DO irow=1,nobjs
         istat = 0
         IF ( idoxy.EQ.0 ) THEN
            CALL FTGCVD(luno,icrobj,irow,1,1,dnul,drobj,qbad,istat)
            CALL FTGCVD(luno,icdobj,irow,1,1,dnul,ddobj,qbad,istat)
         ELSE
            CALL FTGCVE(luno,icxobj,irow,1,1,fnul,xpos, qbad,istat)
            CALL FTGCVE(luno,icyobj,irow,1,1,fnul,ypos, qbad,istat)
            dxpos = xpos
            dypos = ypos
            istat = 0
            IF ( Imat.NE.0 ) THEN
               CALL afwldp(dxpos,dypos,dxref,dyref,dxpix0,dypix0,
     &           dcd11,dcd12,dcd21,dcd22,ctype,drobj,ddobj,istat)
            ELSE
               CALL ftwldp(dxpos,dypos,dxref,dyref,dxpix0,dypix0,
     &            dxdel,dydel,drota,ctype,drobj,ddobj,istat)
            END IF
         END IF
         IF ( icpsf.NE.0 ) THEN
            CALL FTGCVE(luno,icpsf,irow,1,1,rnul,psf,qbad,istat)
         END IF
         IF ( iccnts.NE.0 ) THEN
            CALL FTGCVE(luno,iccnts,irow,1,1,rnul,rcnts,qbad,istat)
         END IF
         r2min = 1e37
         rcos = COS(ddobj/rtd)
         ibest = 0
         DO jrow=1,ncats
            r2cur=rcos*rcos*(drobj-dra(jrow))**2 + (ddobj-dde(jrow))**2
            IF ( r2cur.LT.r2min ) THEN
               ibest = jrow
               r2min = r2cur
            END IF
         END DO
         rnear = 3600.*SQRT(r2min)
         tlmax = MAX(tlmax, rnear)
         IF ( ians.GT.0 ) THEN
            istat = 0
            CALL FTPCLE(luno, icnear, irow, 1, 1, rnear, istat)
            IF ( icr0.NE.0 ) THEN
               CALL FTPCLD(luno, icr0, irow, 1, 1, dra(ibest), istat)
            END IF
            IF ( icd0.NE.0 ) THEN
               CALL FTPCLD(luno, icd0, irow, 1, 1, dde(ibest), istat)
            END IF
         END IF
         IF ( rnear.LE.rmax ) THEN
            nmat = nmat + 1
            dsum1 = dsum1 + rnear
            dsum2 = dsum2 + rnear*rnear
            IF ( nmat.GT.MXMAT ) THEN
               WRITE(*,*) 'MXMAT is too small.'
               GOTO 900
            END IF
            ibad(nmat) = 0
            dr(nmat) = dra(ibest)
            dd(nmat) = dde(ibest)
            dx(nmat) = xpos
            dy(nmat) = ypos
            io(nmat) = irow
            ic(nmat) = ibest
            IF ( icpsf.GT.0 .AND. iccnts.GT.0 ) THEN
               derr(nmat)=SQRT((sigsys/scale)**2+psf*psf/rcnts)
            ELSE
               derr(nmat) = 1.0d0
            END IF
            IF ( idoxy.EQ.0 ) THEN
              WRITE(*,181) irow,drobj,ddobj,ibest,dra(ibest),dde(ibest),
     &            rnear,derr(nmat)*scale
  181        FORMAT(I6,F10.5,F10.5,1X,I6,F10.5,F10.5,F10.3,'"',F9.2,'"')
            ELSE
               WRITE(*,191) irow,xpos,ypos,ibest,dra(ibest),dde(ibest),
     &            rnear,derr(nmat)*scale
  191          FORMAT(I6,2F9.2,1X,I6,F10.5,F10.5,F10.3,'"',F9.2,'"')
            END IF
         END IF
      END DO
C
      WRITE(*,201) tlmax
  201 FORMAT('! Maximum distance=',F10.3,' arcsec')
      WRITE(*,211) dsum1/nmat,SQRT(dsum2/nmat),nmat
  211 FORMAT('! <dist>=',F10.3,'", RMS(dist)=',F10.3,'", npts=',I5)
C      IF ( ians.GT.0 ) THEN
C         istat = 0
C         CALL FTKEYN('TLMAX',nfield+1,ckey,istat)
C         CALL FTIKYE(luno, ckey, tlmax, 5, 'Max NEAR', istat)
C      END IF
C
      WRITE(*,*) 'Found ',nmat,' candidate matches.'
      IF ( iwcs.LE.0 ) THEN
         WRITE(*,*) 'No WCS to initilize fit.'
         GOTO 900
      END IF
      CALL FITWCS(nmat,dr,dd,dx,dy,derr,ctype,imat,ibad,dpar,
     &   nfree,itoss,xchi)
C
      WRITE(*,*)
      WRITE(*,*) 'Best fit:'
      WRITE(*,221) 'SET WCS   TYpe   ',ctype
  221 FORMAT(A,A)
      drota=dpar(5)
      IF ( imat.EQ.0 ) WRITE(*,231) 'ROta  ',dpar(5)
  231 FORMAT('SET WCS   ',A5,2X,F9.3)
      dxpix0=dpar(1)
      WRITE(*,241) 'XPix0',dpar(1)
  241 FORMAT('SET WCS   ',A5,2X,F9.2)
      dxref=dpar(2)
      WRITE(*,251) 'XRef',dpar(2)
  251 FORMAT('SET WCS  ',A5,2X,F10.6)
      IF ( imat.EQ.0 ) THEN
         dxdel=dpar(6)
         WRITE(*,261) 'XDel',dpar(6)
  261    FORMAT('SET WCS  ',A5,2X,F13.9)
      END IF
      dypix0=dpar(3)
      WRITE(*,241) 'YPix0',dpar(3)
      dyref=dpar(4)
      WRITE(*,251) 'YRef',dpar(4)
      IF ( imat.EQ.0 ) THEN
         dydel = dpar(7)
         WRITE(*,261) 'YDel',dpar(7)
      END IF
      IF ( imat.NE.0 ) THEN
         dcd11=dpar(5)
         dcd12=dpar(6)
         dcd21=dpar(7)
         dcd22=dpar(8)
         WRITE(*,271) 'CD1_1',dpar(5)
         WRITE(*,271) 'CD1_2',dpar(6)
         WRITE(*,271) 'CD2_1',dpar(7)
         WRITE(*,271) 'CD2_2',dpar(8)
  271    FORMAT('SET WCS  ',A5,2X,1PG13.6)
      END IF
      WRITE(*,291) xchi,nmat-itoss,2*(nmat-itoss)-nfree
  291 FORMAT('! W-VAR=',F10.3,',  nfit=',I8,',  dof=',I8)
C---
      WRITE(*,*)
      WRITE(*,*) '       Object                  Catalog             '//
     &           '    dist.    error'
      nplt = 0
      dsum1 = 0.0d0
      dsum2 = 0.0d0
      DO i=1,nmat
         IF ( Ibad(i).EQ.0 ) THEN
            IF ( imat.NE.0 ) THEN
               CALL afxypx(dr(i),dd(i),dpar(2),dpar(4),dpar(1),dpar(3),
     &     dpar(5),dpar(6),dpar(7),dpar(8),ctype,dxpix,dypix,istat)
               CALL afwldp(dx(i),dy(i),dxref,dyref,dxpix0,dypix0,
     &           dcd11,dcd12,dcd21,dcd22,ctype,drobj,ddobj,istat)
            ELSE
               CALL ftxypx(dr(i),dd(i),dpar(2),dpar(4),dpar(1),
     &        dpar(3),dpar(6),dpar(7),dpar(5),ctype,dxpix,dypix,istat)
               CALL ftwldp(dx(i),dy(i),dxref,dyref,dxpix0,dypix0,
     &            dxdel,dydel,drota,ctype,drobj,ddobj,istat)
            ENDIF
            nplt = nplt + 1
            yplt(nplt,1) = (dx(i)-dxpix)/derr(i)
            yplt(nplt,2) = (dy(i)-dypix)/derr(i)
C      write(*,*) i,dx(i)-dxpix,dy(i)-dypix,derr(i)
C
            rcos = COS(ddobj/rtd)
            r2cur=rcos*rcos*(drobj-dr(i))**2 + (ddobj-dd(i))**2
            rnear = 3600.*SQRT(r2cur)
            dsum1 = dsum1 + rnear
            dsum2 = dsum2 + rnear*rnear
            IF ( idoxy.EQ.0 ) THEN
               WRITE(*,181)io(i),drobj,ddobj,ic(i),dr(i),dd(i),
     &            rnear,derr(i)*scale
            ELSE
               WRITE(*,191) io(i),dx(i),dy(i),ic(i),dr(i),dd(i),
     &            rnear,derr(i)*scale
            END IF
         END IF
      END DO
      WRITE(*,211) dsum1/nplt,SQRT(dsum2/nplt),nplt
C
C Now we add all the sources that were tossed out during the fit.
      nplt = nplt + 1
      yplt(nplt,1) = NO
      yplt(nplt,2) = NO
      DO i=1,nmat
         IF ( Ibad(i).NE.0 ) THEN
            IF ( imat.NE.0 ) THEN
               CALL afxypx(dr(i),dd(i),dpar(2),dpar(4),dpar(1),dpar(3),
     &     dpar(5),dpar(6),dpar(7),dpar(8),ctype,dxpix,dypix,istat)
            ELSE
               CALL ftxypx(dr(i),dd(i),dpar(2),dpar(4),dpar(1),
     &        dpar(3),dpar(6),dpar(7),dpar(5),ctype,dxpix,dypix,istat)
            END IF
            nplt = nplt + 1
            yplt(nplt,1) = (dx(i)-dxpix)/derr(i)
            yplt(nplt,2) = (dy(i)-dypix)/derr(i)
         END IF
      END DO
      iery(1) = 0
      iery(2) = 0
      cmd(1) = 'SKI SIN'
      cmd(2) = 'LAB X xerr/sig'
      cmd(3) = 'LAB Y yerr/sig'
      cmd(4) = 'MAR 2 ON 1'
      cmd(5) = 'MAR 3 ON 2'
      cmd(6) = 'MAR SIZ 2'
      cmd(7) = 'GRI ON'
      ncmd = 7
      CALL PLT(yplt,iery,MXMAT,nplt,2,cmd,ncmd,ier)
C
  300 CONTINUE
      CALL GTBUF('Redo or Quit:', ier)
      CALL GTCHAR(ctmp, ltmp)
      CALL UPC(ctmp(1:1))
      IF ( ctmp(1:1).EQ.'R' ) GOTO 160
      GOTO 900
C---
C Add near column (incomplete
C  350 CONTINUE
C      CALL YORN('Add near column to object list', idef, ians)
C Reopen file with R/W access
      CALL FTGHDN(luno, ihsav)
      istat = 0
      CALL FTCLOS(luno, istat)
      istat = 0
      CALL FTOPEN(luno, cobj(:lobj), 1, iblk, istat)
      IF ( istat.NE.0 ) THEN
         WRITE(*,*) 'Unable to open ',cobj(:lobj),', istat=',istat
         GOTO 300
      END IF
      CALL FTMAHD(luno, ihsav, iext, istat)
      icnear = 0
      istat = 0
      CALL FTGCNN(luno, .FALSE., 'NEAR', cvalue, icnear, istat)
      IF ( icnear.NE.0 ) THEN
         WRITE(*,*) 'NEAR already exists, overwriting...'
      ELSE
         WRITE(*,*) 'Creating space in FITS file for NEAR...'
         nfield = nfield + 1
         ctypes(1) = 'NEAR'
         cform(1) = '1E'
         icnear = nfield
         istat = 0
C I would like to add TLMIN after I've created the colume (FTICLS)
C but that screws up the FITS file.
         CALL FTKEYN('TLMIN',nfield,ckey,istat)
         CALL FTIKYJ(luno, ckey, 0, 'Min NEAR', istat)
         CALL FTICLS(luno, icnear, 1, ctypes, cform, istat)
         CALL FTKEYN('TUNIT',nfield,ckey,istat)
         CALL FTPKYS(luno, ckey, 'arcsec', ' ', istat)
         IF ( istat.NE.0 ) THEN
            WRITE(*,*) 'Problem adding to FITS file.  Istat=',istat
            ians = 0
         END IF
      END IF
      icr0 = 0
      istat = 0
      CALL FTGCNN(luno, .FALSE., 'RA0', cvalue, icr0, istat)
      IF ( icr0.NE.0 ) THEN
         WRITE(*,*) 'RA0 already exists, overwriting...'
      ELSE
         WRITE(*,*) 'Creating space in FITS file for RA0...'
         nfield = nfield + 1
         ctypes(1) = 'RA0'
         cform(1) = '1D'
         icr0 = nfield
         istat = 0
         CALL FTICLS(luno, icr0, 1, ctypes, cform, istat)
         CALL FTKEYN('TUNIT',nfield,ckey,istat)
         CALL FTPKYS(luno, ckey, 'deg', ' ', istat)
         IF ( istat.NE.0 ) THEN
            WRITE(*,*) 'Problem adding to FITS file.  Istat=',istat
            GOTO 300
         END IF
      END IF
      icd0 = 0
      istat = 0
      CALL FTGCNN(luno, .FALSE., 'DEC0', cvalue, icd0, istat)
      IF ( icd0.NE.0 ) THEN
         WRITE(*,*) 'DEC0 already exists, overwriting...'
      ELSE
         WRITE(*,*) 'Creating space in FITS file for DEC0...'
         nfield = nfield + 1
         ctypes(1) = 'DEC0'
         cform(1) = '1D'
         icd0 = nfield
         istat = 0
         CALL FTICLS(luno, icd0, 1, ctypes, cform, istat)
         CALL FTKEYN('TUNIT',nfield,ckey,istat)
         CALL FTPKYS(luno, ckey, 'deg', ' ', istat)
         IF ( istat.NE.0 ) THEN
            WRITE(*,*) 'Problem adding to FITS file.  Istat=',istat
         END IF
      END IF
      GOTO 300
C---
  900 CONTINUE
      istat = 0
      CALL FTCLOS(luno, istat)
      CALL FTCLOS(lunc, istat)
      CALL EDICOM('OFF',3)
      END
