C Program nearx.  Takes two FITS files as input (the Object list and
C Catalog files).  For each object in the object list, find the closest
C object in the Catalog and output distance.  No WCS, only used X and
C Y columns.
C---
C 2015-05-28 - From near2 [AFT]
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INTEGER   MXCOL, MXCAT, MXMAT, MXNAX
      PARAMETER (MXCOL=30, MXCAT=100000, MXMAT=100000, MXNAX=99)
C
      CHARACTER cmd(10)*64
      CHARACTER ccat*256, cobj*256, cvalue*64, ctmp*64
      CHARACTER ccol*16, ckey*16
      DOUBLE PRECISION dxcat(MXCAT), dycat(MXCAT)
      DOUBLE PRECISION dnul, dsum1, dsum2, dxobj, dyobj
      REAL      yplt(MXMAT,2)
      REAL      rccnts(MXCAT)
      REAL      rnear, r2cur, r2min, rocnts
      REAL      rnul, rmax, tlmax
      INTEGER   ib(MXMAT)
      INTEGER   naxes(MXNAX), iery(10)
      INTEGER   i, ibest, ibits, iblk
      INTEGER   icccat, iccobj, icxcat, icycat, icxobj, icyobj
      INTEGER   ier, iext, igcnt,ipcnt, irow, istat
      INTEGER   jrow, lcat, lobj, ltmp, lunc, luno
      INTEGER   naxis, ncats, ncmd, nfield, nmat, nobjs, nplt
      LOGICAL   qbad, qsimp, qxtend
C---
   11 FORMAT(A)
C---
      rmax = 2.0
      luno = 1
      lunc = 3
  100 CONTINUE
      CALL GTBUF('Object list, Catalog:', ier)
      IF ( ier.LT.0 ) GOTO 900
      CALL GTCHAR(cobj, lobj)
      IF ( lobj.LE.0 ) GOTO 900
      CALL GTCHAR(ccat, lcat)
      IF ( lcat.LE.0 ) GOTO 900
C
      istat = 0
      CALL FTOPEN(luno, cobj(:lobj), 0, iblk, istat)
      IF ( istat.NE.0 ) THEN
         WRITE(*,*) 'Unable to open ',cobj(:lobj),', istat=',istat
         GOTO 100
      END IF
      CALL FTOPEN(lunc, ccat(:lcat), 0, iblk, istat)
      IF ( istat.NE.0 ) THEN
         WRITE(*,*) 'Unable to open ',ccat(:lcat),', istat=',istat
         GOTO 100
      END IF
C---
C Read keywords from primary header
C
      istat = 0
      CALL FTGHPR(luno,MXNAX,qsimp,ibits,naxis,naxes,ipcnt,igcnt,qxtend,
     &            istat)
      IF ( naxis.NE.0 .OR. .NOT.qxtend ) THEN
         WRITE(*,*) 'Not a binary table FITS file.', naxis,qxtend
         GOTO 100
      END IF
C
      CALL FTGHPR(lunc,MXNAX,qsimp,ibits,naxis,naxes,ipcnt,igcnt,qxtend,
     &            istat)
      IF ( naxis.NE.0 .OR. .NOT.qxtend ) THEN
         WRITE(*,*) 'Not a binary table FITS file.', naxis,qxtend
         GOTO 100
      END IF
C
C Assume data is in first extension.  Note, reading NAXIS2 works for
C both FITS binary and ASCII tables.
C
      istat = 0
      CALL FTMAHD(luno, 2, iext, istat)
      CALL FTGKYJ(luno,'NAXIS2',nobjs,ctmp,istat)
      CALL FTGKYJ(luno,'TFIELDS',nfield,ctmp,istat)
      CALL FTMAHD(lunc, 2, iext, istat)
      CALL FTGKYJ(lunc,'NAXIS2',ncats,ctmp,istat)
C
C We are now at the EVENTS extension.  Scan TTYPE keywords until
C we find the columns we need.
      istat = 0
      iccobj = 0
      CALL FTGCNN(luno, .FALSE., 'COUNTS',  ccol, iccobj, istat)
      icxobj = 0
      icyobj = 0
      DO i=1,99
C Construct keyword name
         istat = 0
         CALL FTKEYN('TTYPE',i,ckey,istat)
C Search for keyword value
         CALL FTGKYS(luno,ckey,cvalue,ctmp,istat)
         IF (istat .GT. 0) GOTO 130
C
         CALL UPC(cvalue)
         IF ( cvalue.EQ.'X' ) THEN
            icxobj = i
         ELSE IF ( cvalue.EQ.'Y' ) THEN
            icyobj = i
         END IF
      END DO
C
  130 CONTINUE
      icccat = 0
      istat=0
      CALL FTGCNN(lunc, .FALSE., 'COUNTS',  ccol, icccat, istat)
      istat = 0
      DO i=1,99
C Construct keyword name
         CALL FTKEYN('TTYPE',i,ckey,istat)
C Search for keyword value
         CALL FTGKYS(lunc,ckey,cvalue,ctmp,istat)
         IF (istat .GT. 0) GOTO 160
         CALL UPC(cvalue)
         IF ( cvalue(1:2).EQ.'X' ) THEN
            icxcat = i
         ELSE IF ( cvalue(1:2).EQ.'Y' ) THEN
            icycat = i
         END IF
      END DO
C
  160 CONTINUE
      WRITE(*,*) 'Columns:'
      WRITE(*,*) 'Object :   X   =',icxobj,',   Y  =',icyobj
      WRITE(*,*) 'Catalog:   X   =',icxcat,',   Y  =',icycat
C
C Now read selected data for each event and process.
      WRITE(*,*) '! ',nobjs,' in ',cobj(:lobj)
      WRITE(*,*) '! ',ncats,' in ',ccat(:lcat)
      WRITE(ctmp, 161) rmax
  161 FORMAT('Maximum radius (',F10.1,'):')
      CALL GTBUF(ctmp, ier)
      CALL GTREAL(rmax, ier)
C
      istat = 0
      DO irow=1,ncats
         istat = 0
         CALL FTGCVD(lunc,icxcat,irow,1,1,dnul,dxcat(irow), qbad,istat)
         CALL FTGCVD(lunc,icycat,irow,1,1,dnul,dycat(irow), qbad,istat)
         IF ( icccat.NE.0 ) THEN
            CALL FTGCVE(lunc,icccat,irow,1,1,rnul,rccnts(irow),
     &         qbad,istat)
         END IF
      END DO
      WRITE(*,*)
      nmat = 0
      tlmax = 0.0
      dsum1 = 0.0d0
      dsum2 = 0.0d0
      WRITE(*,*) '       Object                  Catalog             '//
     &           '    dist.    error'
      DO irow=1,nobjs
         istat = 0
         CALL FTGCVD(luno,icxobj,irow,1,1,dnul,dxobj, qbad,istat)
         CALL FTGCVD(luno,icyobj,irow,1,1,dnul,dyobj, qbad,istat)
         IF ( iccobj.NE.0 ) THEN
            CALL FTGCVE(luno,iccobj,irow,1,1,rnul,rocnts,qbad,istat)
         END IF
         r2min = 1e37
         ibest = 0
         DO jrow=1,ncats
            r2cur=(dxobj-dxcat(jrow))**2 + (dyobj-dycat(jrow))**2
            IF ( r2cur.LT.r2min ) THEN
               ibest = jrow
               r2min = r2cur
            END IF
         END DO
         rnear = SQRT(r2min)
         tlmax = MAX(tlmax, rnear)
         IF ( rnear.LE.rmax ) THEN
            nmat = nmat + 1
            dsum1 = dsum1 + rnear
            dsum2 = dsum2 + rnear*rnear
            IF ( nmat.GT.MXMAT ) THEN
               WRITE(*,*) 'MXMAT is too small.'
               GOTO 900
            END IF
            ib(nmat)=ibest
C            WRITE(*,181) irow,dxobj,dyobj,ibest,
C     &         dxcat(ibest),dycat(ibest),rnear
C  181       FORMAT(I6,2F9.1,1X,I6,2F9.1,F9.2)
            WRITE(*,191) irow,ibest,rnear,rocnts,rccnts(ibest)
  191       FORMAT(I6,I6,F9.2,2F9.0)
         END IF
      END DO
C
      WRITE(*,201) tlmax
  201 FORMAT('! Maximum distance=',F10.3)
      WRITE(*,211) dsum1/nmat,SQRT(dsum2/nmat),nmat
  211 FORMAT('! <dist>=',F10.2,', RMS(dist)=',F10.2,', npts=',I6)
      WRITE(*,*) 'Found ',nmat,' candidate matches.'
C---
C Plot results
      nplt = 0
      yplt(nplt,1) = NO
      yplt(nplt,2) = NO
      DO i=1,nmat
         nplt = nplt + 1
         ibest=ib(i)
         yplt(nplt,1) = dxcat(ibest)
         yplt(nplt,2) = dycat(ibest)
      END DO
      iery(1) = 0
      iery(2) = 0
      cmd(1) = 'SKI SIN'
      cmd(2) = 'LAB X xerr/sig'
      cmd(3) = 'LAB Y yerr/sig'
      cmd(4) = 'MAR 2 ON 1'
      cmd(5) = 'MAR 3 ON 2'
      cmd(6) = 'MAR SIZ 2'
      cmd(7) = 'GRI ON'
      ncmd = 6
      CALL PLT(yplt,iery,MXMAT,nplt,2,cmd,ncmd,ier)
C
      CALL GTBUF('Redo or Quit:', ier)
      CALL GTCHAR(ctmp, ltmp)
      CALL UPC(ctmp(1:1))
      IF ( ctmp(1:1).EQ.'R' ) GOTO 160
C---
  900 CONTINUE
      istat = 0
      CALL FTCLOS(luno, istat)
      CALL FTCLOS(lunc, istat)
      CALL EDICOM('OFF',3)
      END
