C Program noray
C---
C Read in N FITS files images (N<=100) and remove cosmic rays.  A cosmic
C ray is defined as high DN values at a given location in only a few
C images.  The algorithm works by examining the N values, from the N
C files, for each pixel.  The data are divided into two groups, those
C with values below a threshold, and those with values above.  If at
C a given pixel location one or more DN value is below the threshold,
C then the output is the average of all the DN's that were below.  If
C one sets a global threshold, then it is possible for all the DN values
C to be above the threshold.  If this occurs, then the pixel is considered
C to be a "hot pixel", and the output is the average all all the DN
C that were above the threshold.
C
C There are two ways to set the threshold.  The first is a global
C threshold which is the same for every pixel in the image.  This requires
C that the user enter this threshold value.  The second method is to
C dynamically determine a value for every pixel.  This is done by reading
C the data files 3 times.  First time through, we compute the average
C and standard deivation (SD) at each pixel location.  At preliminary
C threshold is defined to be the average plus 1.0 times sigma.  Second
C time, we compute the average and SD again, but only for pixels below
C the preliminary threshold.  We now define the threshold for every pixel
C which is the new average plus 2.0 times the new sigma.  In the final
C pass we divide the data into DN values below the threshold, which
C is the new average plus 2.0 times the new sigma.
C---
C 1998-Jun-30 - [AFT]
C---
      INTEGER   MXBUF, MXCOL, MXFILE, MXNAX
      PARAMETER (MXBUF=2150, MXCOL=3200, MXFILE=400, MXNAX=99)
      INTEGER   LENACT
C
      CHARACTER cnam(MXFILE)*256
      CHARACTER chead(MXFILE+2)*80
      CHARACTER cbuf*256, cout*256
      CHARACTER ctmp*64
      REAL      rsum1(MXBUF, MXCOL), rsum2(MXBUF, MXCOL)
      REAL      thres(MXBUF, MXCOL)
      REAL      rrow(MXBUF)
      REAL      gthres, rnul, rsig1, rsig2
      INTEGER*2 icnt(MXBUF, MXCOL)
      INTEGER   naxes(MXNAX)
      INTEGER   i, ibegin, ibits, ier, idef, ifile, ifirst, iglob
      INTEGER   igcnt, igroup, imin, ipcnt, istat, itmp
      INTEGER   j, lbuf, lhead, lout, ltmp, lunin, lunout
      INTEGER   naxis, ncol, nfiles, nhead, nrow
      LOGICAL   qbad, qsimp, qxtend
C---
      nfiles = 0
      idef = -1
      gthres = 0.0
      iglob = 0
      imin = 0
C
  100 CONTINUE
      CALL GTBUF('Threshold, Global, Dynamic, Minimum:', ier)
      CALL GTCHAR(ctmp, ltmp)
      CALL UPC(ctmp)
      IF ( ctmp(1:1).EQ.'D' ) THEN
         iglob = -1
         rsig1 = 1.0
         rsig2 = 2.0
         WRITE(chead(1), 131) 1, rsig1
  131    FORMAT('NSIG',I1,'   =           ',F10.1,' /')
         WRITE(chead(2), 131) 2, rsig2
         nhead = 2
      ELSE IF ( ctmp(1:1).EQ.'G' ) THEN
         iglob = 1
         WRITE(ctmp, 141) gthres
  141    FORMAT(' Threshold value (',F10.2,'):')
         CALL GTBUF(ctmp, ier)
         IF ( ier.LT.0 ) GOTO 100
         CALL GTREAL(gthres, ier)
         WRITE(chead(1), 171) gthres
  171    FORMAT('Thres   =           ',F10.2,' /')
         nhead = 1
      ELSE IF ( ctmp(1:1).EQ.'M' ) THEN
         imin = 1
      ELSE
         GOTO 100
      END IF
C
      lhead = 100
      lunin = 3
      lunout = 4
      DO i=1, MXBUF
         DO j=1, MXBUF
            rsum1(j, i) = 0.0
            rsum2(j, i) = 0.0
            icnt (j, i) = 0
         END DO
      END DO
      ifirst = 1
C---
  200 CONTINUE
      CALL GTBUF('FITS image file: ', ier)
      CALL GTCHAR(ctmp, ltmp)
      IF ( ltmp.GT.0 ) THEN
         nfiles = nfiles + 1
         cnam(nfiles) = ctmp
C Add filename to history record
         IF ( ltmp+lhead.GE.80 ) THEN
            nhead = nhead + 1
            chead(nhead) = 'HISTORY   noray:'
            lhead = LENACT(chead(nhead))
         END IF
         chead(nhead) = chead(nhead)(:lhead)//'  '//ctmp
         lhead = LENACT(chead(nhead))
C
         istat = 0
         CALL FTOPEN(lunin,cnam(nfiles),0,itmp,istat)
         IF ( istat.NE.0 ) THEN
            nfiles = nfiles - 1
            GOTO 200
         END IF
         CALL FTGHPR(lunin,MXNAX,qsimp,ibits,naxis,naxes,ipcnt,igcnt,
     &      qxtend, istat)
         CALL FTGKYJ(lunin,'NAXIS1',naxes(1),ctmp,istat)
         CALL FTGKYJ(lunin,'NAXIS2',naxes(2),ctmp,istat)
         naxes(3) = 0
         CALL FTGKYJ(lunin,'NAXIS3',naxes(3),ctmp,istat)
         IF ( naxes(1).GT.MXBUF ) THEN
            WRITE(*,*) 'Error:  FITS image dimension is larger',
     :                 ' than the internal buffer'
            WRITE(*,*) '        size of',MXBUF
            GOTO 990
         END IF
         IF ( naxes(3).GT.0 ) naxes(2) = naxes(2)*naxes(3)
         IF ( ifirst.NE.0 ) THEN
            nrow = naxes(1)
            ncol = naxes(2)
         ELSE
            IF ( nrow.NE.naxes(1) .OR. ncol.NE.naxes(2) ) THEN
               WRITE(*,*) 'number of rows and columns do not match!'
               GOTO 990
            END IF
         END IF
      write(*,*) 'nrow, ncol=',nrow,ncol
C Read one row at a time into buffer, then add to rsum1 and rsum2
C arrays.
         ibegin = 1
         DO i=1, ncol
            istat = 0
            CALL FTGPVE(lunin,igroup,ibegin,nrow,rnul,
     &         rrow,qbad,istat)
            IF ( imin.GT.0 ) THEN
               IF ( ifirst.GT.0 ) THEN
                  DO j=1, nrow
                     rsum1(j, i) = rrow(j)
                  END DO
               ELSE
                  DO j=1, nrow
                     rsum1(j, i) = MIN(rsum1(j, i), rrow(j))
                  END DO
               END IF
            ELSE IF ( iglob.LT.0 ) THEN
C Compute mean and var
               DO j=1, nrow
                  rsum1(j, i) = rsum1(j, i) + rrow(j)
                  rsum2(j, i) = rsum2(j, i) + rrow(j)*rrow(j)
               END DO
            ELSE
               DO j=1, nrow
                  IF ( rrow(j).LT.gthres ) THEN
                     rsum1(j, i) = rsum1(j, i) + rrow(j)
                     icnt(j, i) = icnt(j, i) + 1
                  ELSE
                     rsum2(j, i) = rsum2(j, i) + rrow(j)
                  END IF
               END DO
            END IF
            ibegin = ibegin + nrow
         END DO
         CALL FTCLOS(lunin, istat)
         ifirst = 0
         GOTO 200
      END IF
C---
C Blank filename, so finish up.
      IF ( iglob.LT.0 ) THEN
C Compute mean and sigma
         DO i=1, ncol
            DO j=1, nrow
C Replace rsum1 and rsum2 with the mean and sigma
               rsum1(j, i) = rsum1(j, i)/nfiles
               rsum2(j, i) = rsum2(j, i)-rsum1(j,i)*rsum1(j, i)*nfiles
               rsum2(j, i) = SQRT(rsum2(j, i)/(nfiles-1.0))
C Now replace rsum2 with the upper limit (mean + rsig1*sigma)
               thres(j, i) = rsum1(j, i) + rsig1*rsum2(j, i)
               rsum1(j, i) = 0.0
               rsum2(j, i) = 0.0
               icnt (j, i) = 0
            END DO
         END DO
C---
C Now reread the files
         WRITE(*,*) 'Number of files=',nfiles
         WRITE(*,*) 'Computing a better threshold.'
C---
C Read file again to compute a better average and sigma.
         DO ifile = 1, nfiles
            istat = 0
            CALL FTOPEN(lunin,cnam(ifile),0,itmp,istat)
            CALL FTGHPR(lunin,MXNAX,qsimp,ibits,naxis,naxes,ipcnt,igcnt,
     &      qxtend, istat)
            ibegin = 1
            DO i=1, ncol
               CALL FTGPVE(lunin,igroup,ibegin,nrow,rnul,
     &         rrow,qbad,istat)
               DO j=1, nrow
                  IF ( rrow(j).LE.thres(j, i) ) THEN
                     rsum1(j, i) = rsum1(j, i) + rrow(j)
                     rsum2(j, i) = rsum2(j, i) + rrow(j)*rrow(j)
                     icnt (j, i) = icnt(j, i) + 1.0
                  END IF
               END DO
               ibegin = ibegin + nrow
            END DO
            IF ( istat.NE.0 ) write(*,*) 'istat=',istat
            CALL FTCLOS(lunin, istat)
         END DO
C---
C Compute better mean and sigma for each pixel
         DO i=1, ncol
            DO j=1, nrow
C Replace rsum1 and rsum2 with the mean and sigma
               IF ( icnt(j,i).GT.0 ) THEN
                  rsum1(j, i) = rsum1(j, i)/icnt(j, i)
                  rsum2(j, i) = rsum2(j, i) -
     &                        rsum1(j,i)*rsum1(j,i)*icnt(j,i)
                  rsum2(j, i) = SQRT(rsum2(j, i)/(icnt(j,i)-1.0))
C Now replace rsum2 with the upper limit (mean + rsig2*sigma)
                  thres(j, i) = rsum1(j, i) + rsig2*rsum2(j, i)
                  rsum1(j, i) = 0.0
                  icnt (j, i) = 0
               END IF
            END DO
         END DO
C---
         WRITE(*,*) 'Computing final average.'
         DO ifile = 1, nfiles
            istat = 0
            CALL FTOPEN(lunin,cnam(ifile),0,itmp,istat)
            CALL FTGHPR(lunin,MXNAX,qsimp,ibits,naxis,naxes,ipcnt,igcnt,
     &      qxtend, istat)
            ibegin = 1
            DO i=1, ncol
               CALL FTGPVE(lunin,igroup,ibegin,nrow,rnul,
     &         rrow,qbad,istat)
               DO j=1, nrow
                  IF ( rrow(j).LE.thres(j, i) ) THEN
                     rsum1(j, i) = rsum1(j, i) + rrow(j)
                     icnt (j, i) = icnt(j, i) + 1.0
                  ELSE
                     rsum2(j, i) = rsum2(j, i) + rrow(j)
                  END IF
               END DO
               ibegin = ibegin + nrow
            END DO
            IF ( istat.NE.0 ) write(*,*) 'istat=',istat
            CALL FTCLOS(lunin, istat)
         END DO
      END IF
C---
C Compute average.
      IF ( imin.EQ.0 ) THEN
         DO i=1, ncol
            DO j=1, nrow
C If there were one or more pixels at this location that were below
C the threshold then
               IF ( icnt(j, i).GT.0 ) THEN
C Average all the pixels that were *below* the threshold.
                  rsum1(j, i) = rsum1(j, i)/icnt(j, i)
               ELSE
C Otherwise average all the that were *above* the threshold (stored in rsum2).
                  rsum1(j, i) = rsum2(j, i)/(nfiles - icnt(j, i))
                  WRITE(*,311) j,i,rsum1(j,i)
 311              FORMAT(' Hot pixel at',I6,I6,F11.2)
               END IF
            END DO
         END DO
      END IF
C---
C and write out the result.
      CALL GTBUF('Output file name?', ier)
      CALL GTCHAR(cout, lout)
      IF ( lout.GT.0 ) THEN
         CALL XTEND(cout, 'FIT')
         cbuf = 'cp -p '//cnam(1)(:LENACT(cnam(1)))//' '//cout(:lout)
         lbuf = LENACT(cbuf)
         CALL SPAWN(cbuf, lbuf, ier)
         istat = 0
         lunout = 4
         CALL FTOPEN(lunout,cout(:lout),1,itmp,istat)
         ibegin = 1
         igroup = 1
         DO i=1, ncol
            DO j=1, nrow
               rrow(j) = rsum1(j, i)
            END DO
            CALL FTPPRE(lunout,igroup,ibegin,nrow,rrow,istat)
            ibegin = ibegin + nrow
         END DO
         CALL FTCLOS(lunout, istat)
      END IF
C---
C      CALL WR4MAP(rsum1,MXBUF,nrow,ncol, chead,nhead, ier)
C      WRITE(*,*) 'To write icnt array, input name now:'
C      CALL WI2MAP(icnt,MXBUF,nrow,ncol, chead,nhead, ier)
C      WRITE(*,*) 'To write thres array, input name now:'
C      CALL WR4MAP(thres,MXBUF,nrow,ncol, chead,nhead, ier)
C---
  990 CONTINUE
      CALL EDICOM('OFF', 3)
      END
