      SUBROUTINE APROF(Rmap, Tdata, Back, Tback, Mxrow, Nrow, Ncol,
     &         Cmd, Mxcmd)
      INTEGER   Mxrow, Nrow, Ncol
      REAL      Rmap(Mxrow,*), Back(Mxrow,*)
      REAL      Tdata, Tback
      INTEGER   Mxcmd
      CHARACTER Cmd(Mxcmd)*(*)
C---
C LEXTRCT subroutine to plot the radially-summed azmuthal profile of map
C data.
C---
C Rmap    I    The data array
C Tdata   I    Integration time for data map,
C Back    I    Background map
C Tback   I    Integration time for background map.
C Mxrow   I    Maximum number of rows in the arrays.
C Nrow    I    Actual number of rows in map arrays.
C Ncol    I    Actual number of columns in map arrays.
C Cmd       O  PLT command array
C Mxcmd   I    Maximum size of command array
C
C Global:
C  Xdet, Ydet     position in detector coordinates
C---
C 1997-Jan-13 - Adapted from RPROF [AFT]
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INCLUDE 'posdef.inc'
      INTEGER   MXRING
      PARAMETER (MXRING=1440)
      REAL      GETVIG
      INTEGER   LENACT
C
      CHARACTER cprom*72, ctmp*72
      CHARACTER cans*10, cdef
      SAVE      cans
      DOUBLE PRECISION dalla(MXRING), dsuma(MXRING)
      DOUBLE PRECISION dsump(MXRING), dsumv(MXRING), dsumx(MXRING)
      DOUBLE PRECISION dtota, dfpow, dtmp, dtotv
      REAL      yplt(MXRING,5)
      REAL      angle, bfac, bcnts, dist
      REAL      renorm, rmax, rmin, tmp, weight
      SAVE      rmax, rmin
      REAL      xoff, xpos, yoff, ypos
      INTEGER   iery(5)
      INTEGER   i, ibada, ibadv, ibin, ier
      INTEGER   imxs, imxe, imys, imye, iscat
      INTEGER   j, lans, ltmp, ncmd, nocnt
      INTEGER   nrings
      SAVE      nrings
      LOGICAL   qfail, qnew, qplot
C
      DATA cans/'N'/
      DATA rmin,rmax/1.0,100.0/
      DATA nrings/90/
C
C Find center position for azim
C
      CALL PDSEL('IM',qnew,qplot)
 130  CALL ASKPOS(qplot,qfail)
      IF ( qfail ) GOTO 999
C
C Write centre of azim to terminal
C
      WRITE (ctmp,131) Xdet, Ydet
 131  FORMAT (' Center = (',F8.1,',',F8.1,')')
      ncmd = 1
      Cmd(ncmd) = 'LAB 1 Jus Rig VPOS .85 .86 "'//ctmp
      ncmd = 2
      Cmd(ncmd)='LAB 2 Jus Rig VPos .85 .82 "Ifac = '
      ltmp = LENACT(Cmd(ncmd))
      CALL CRAMIF(Ifac, 5, Cmd(ncmd), ltmp)
C
C Request width of rings to be used in azimuthal summing
C
  150 CONTINUE
      WRITE (cprom,151) rmin, rmax
  151 FORMAT ('Rmin (',1PG9.3,'), Rmax (',G9.3,'):')
      CALL GTBUF(cprom,ier)
      IF ( ier.NE.0 ) GOTO 130
      CALL GTREAL(rmin, ier)
      CALL GTREAL(rmax, ier)
      IF ( rmin.LE.0.0 .OR. rmax.LE.0.0) THEN
         CALL PTBUF('** Illegal radius **',-1)
         GOTO 150
      END IF
C
C Request number of rings to be used in azimuthal summing
C
 200  WRITE (cprom,201) nrings
 201  FORMAT ('Number of angular bins (',I6,'):')
      CALL GTBUF(cprom,ier)
      IF ( ier.NE.0 ) GOTO 150
      CALL GTINT(nrings,ier)
      IF ( nrings.LE.0 ) THEN
         CALL PTBUF('** Illegal number of rings **',-1)
         GOTO 200
      ELSE IF ( nrings.GT.MXRING ) THEN
         WRITE (*,*) 'Number of bins reset to', MXRING
         nrings = MXRING
      END IF
C---
      CALL GTPEEK(ctmp,ltmp)
      IF ( ltmp.LE.0 ) THEN
         cdef = cans
         cprom = 'Should errors be based on scatter (' // cdef // ')?'
         CALL GTBUF(cprom,ier)
         IF ( ier.NE.0 ) GOTO 200
      END IF
      CALL GTCHAR(cans,lans)
      CALL UPC(cans)
      IF ( lans.LE.0 ) cans = cdef
      IF ( cans.EQ.'Y' ) THEN
         iscat = 1
      ELSE
         iscat = 0
      END IF
C---
C
C Work out the limits on bin numbers to loop through (to save time)
C
      CALL POS_DT_IM(Xdet-rmax,Ydet-rmax,imxs,imys)
      imxs = MIN(MAX(1,imxs),Nrow)
      imys = MIN(MAX(1,imys),Ncol)
      CALL POS_DT_IM(Xdet+rmax,Ydet+rmax,imxe,imye)
      imxe = MIN(MAX(1,imxe),Nrow)
      imye = MIN(MAX(1,imye),Ncol)
C
C Initialize arrays
C
      DO i = 1, nrings
         dsump(i) = 0.0D0
         dsumv(i) = 0.0D0
         dsuma(i) = 0.0D0
         dalla(i) = 0.0D0
         dsumx(i) = 0.0D0
      END DO
C
C Initialize some counters
C
      nocnt = 0
      ibadv = 0
      ibada = 0
      dtota  = 0.0D0
      IF ( Tback.NE.0.0 ) THEN
         bfac = Tdata/Tback
      ELSE
         bfac = 0.0
      END IF
      bcnts = 0.0
C
C Do azimuthal summing - note use of vignetting corrections
C and the ignoring of data set to the no-data flag.
C
      DO j = imys, imye
         DO 650 i = imxs, imxe
C Ignore bins containing bad data.
            IF ( Rmap(i,j).EQ.NO ) THEN
               nocnt = nocnt + 1
               GOTO 650
            END IF
C
            CALL POS_MAP_DET(i, j, xpos, ypos)
C Compute distance to each corner of the current pixel and average.
            xoff = xpos - Xdet
            yoff = ypos - Ydet
            dist = (SQRT((xoff-0.5)**2+(yoff-0.5)**2) +
     :              SQRT((xoff-0.5)**2+(yoff+0.5)**2) +
     :              SQRT((xoff+0.5)**2+(yoff-0.5)**2) +
     :              SQRT((xoff+0.5)**2+(yoff+0.5)**2) )/4
            IF ( dist.LT.rmin .OR. rmax.LT.dist ) THEN
               GOTO 650
            END IF
C Now compute angle
            CALL GETANG(xpos,ypos,Xdet,Ydet,angle)
            ibin = 1 + nrings*(angle/360.)
            IF ( ibin.LT.1 .OR. ibin.GT.nrings ) THEN
               write(*,*) 'err.  i,ang,nrings=',i,angle,nrings
               ibin = MIN(MAX(1,ibin),nrings)
            END IF
C Count number of pixels that are suppose to included in this ring.
            dalla(ibin) = dalla(ibin) + 1.0
            tmp = GETVIG(xpos,ypos)
            IF ( tmp.GT.0.0 ) THEN
               weight = 1./tmp
               IF ( Tback.NE.0.0 ) THEN
                  IF ( Back(i,j).NE.NO ) THEN
                     bcnts = Back(i,j)
                  ELSE
                     WRITE (*, 631) xpos,ypos
 631                 FORMAT(' Bad background at ',I4,1X,I4)
                     GOTO 650
                  END IF
               END IF
               dtmp = weight*(Rmap(i,j) - bfac*bcnts)
               dsump(ibin) = dsump(ibin) + dtmp
               IF ( iscat.NE.0 ) THEN
                  dsumv(ibin) = dsumv(ibin) + dtmp*dtmp
               ELSE
C Assume Photon statistics (Needs to be checked!)
                  dsumv(ibin) = dsumv(ibin) + weight*weight*
     &               (Rmap(i,j) + bfac*bfac*bcnts)
               END IF
               dsuma(ibin) = dsuma(ibin) + 1.0
               dsumx(ibin) = dsumx(ibin) + dist
               dtota = dtota + 1.0
            ELSE
               WRITE (*, 641) xpos,ypos,tmp
 641           FORMAT(' Bad vignetting at ',I4,1X,I4,1PG10.4)
               ibadv = ibadv + 1
            END IF
 650     CONTINUE
      END DO
C
C Trap case of no pixels in region.
C
      IF ( dtota.LE.0.0D0 ) THEN
         CALL PTBUF('**  No pixels **',-1)
         GOTO 999
      END IF
C
C Report failures in vignetting correction and number of no-data
C flags encountered
C
      IF ( nocnt.NE.0 ) THEN
         WRITE (*,*)
     &          'Number of map bins with no-data flag set:', nocnt
      END IF
      IF ( ibada.NE.0 ) THEN
         WRITE (*,*)
     &          'Number of map bins with excluded angles:', ibada
      END IF
      IF ( ibadv.NE.0 ) THEN
         WRITE (*,*)
     &          'Number of map bins with invalid vignetting correction:'
     &         , ibadv
      END IF
C---
      WRITE (*,*) 'Number of rings used is:', nrings
      IF ( iscat.EQ.0 ) THEN
         CALL PTBUF('Errors based on photon counting statistics.',-1)
      ELSE
         CALL PTBUF('Errors based on scatter.',-1)
      END IF
      WRITE(*,*) 'Total pixels used =', dtota
C
C Area is a count of the number of Map pixels include.  Renorm will convert
C this to either detector pixels.
C
      renorm = 1.0/(Ifac*Ifac)
      Cmd(ncmd+1) = 'LAB G1 Angle (degrees)'
      Cmd(ncmd+2) = 'LAB G2 Counts/dpixel\u2'
      ncmd = ncmd + 3
      Cmd(ncmd)='LAB 3 Jus Rig VPos .85 .78 "Radii ='
      ltmp = LENACT(Cmd(ncmd))
      CALL CRAMFF(rmin, 5, 1, Cmd(ncmd), ltmp)
      ltmp = ltmp + 1
      CALL CRAMFF(rmax, 5, 1, Cmd(ncmd), ltmp)
C
C Calculate counts/pixel and error bounds.
C
      dfpow = 0.0
      dtotv = 0.0
      DO i = 1, nrings
         yplt(i,1) = (i-0.5)*360./nrings
         IF ( dsuma(i).LE.0.0 ) THEN
            yplt(i,2) = NO
            yplt(i,3) = NO
         ELSE
C
C Convert from counts/bin to counts/square arcsec
C
            yplt(i,2) = renorm*dsump(i)/dsuma(i)
            yplt(i,3) = renorm*SQRT(ABS(dsumv(i)))/dsuma(i)
         END IF
      END DO
C
C Plot
C
      CALL PDSEL('LI',qnew,qplot)
      IF ( qplot ) THEN
         iery(1) = 0
         iery(2) = 1
         iery(3) = 1
         Cmd(ncmd+1) = 'LAB G3 Encircled energy'
         Cmd(ncmd+2) = 'LINE Step'
         Cmd(ncmd+3) = 'Plot Vert'
         ncmd = ncmd + 3
         CALL GIFLBL(ctmp)
         Cmd(ncmd+1) = 'LAB F  '//ctmp
         ncmd = ncmd + 1
         CALL PLT(yplt,iery,MXRING,nrings,2,Cmd,ncmd,ier)
      END IF
C
C Tidy up before exiting
C
 999  CONTINUE
C
      END
