      SUBROUTINE CDTECT(Rmap,Mxrow,Nrow,Ncol,Qpoint,
     &      Mxray,Mxpsf,Brad,Srad,Sfparm,Box,Sorc,Sdev,Bgpix,Excess,
     &      Nmx,Nmy,Xpect,Npsf)
      INTEGER   Mxrow, Nrow, Ncol, Mxray, Mxpsf
      INTEGER   Nmx, Nmy, Npsf
      REAL      Rmap(Mxrow, *), Brad, Srad, Sfparm(2)
      LOGICAL   Qpoint
      REAL      Sorc(Mxray,Mxray), Sdev(Mxray,Mxray)
      REAL      Bgpix(Mxray,Mxray), Excess(Mxray,Mxray)
      REAL      Xpect(-Mxpsf:Mxpsf,-Mxpsf:Mxpsf)
      REAL      Box(4)
C---
C Find sources using the Cell Detect method.
C
C Assume
C   C = alpha*S + B
C   T = beta*S + arat*B
C where
C   alpha = psumi, beta=psumi+psumo
C   arat = (b/d)**2 = (cnti+cnto)/cnti   ! area ratio
C---
C Rmap    I
C Mxrow   I
C Nrow    I
C Ncol    I
C Qpoint  I    Is one point being interrogated?
C Mxray   I    Maximum size of Sorc, Sdev, and Bgpix
C Mxpsf   I    Maximum size of Xpect
C Brad    I    Radius of background region (in units of PSF_sig)
C Srad    I    Radius of source region (in units of PSF_sig)
C Sfparm  I    Source finding parameters (PFrac, PThres)
C Box       O  Maximum size of region searched
C Sorc      O  output array of source estimators
C Sdev      O  output array of sd of above
C Bgpix     O  output array of bg estimators
C Nmx,Mny   O  Actual size of Sorc, Sdev and Bgpix
C Xpect     O  The last PSF computed
C Npsf      O  Actual size of Xpect
C---
C 2000-07-31 - [AFT]
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INCLUDE 'posdef.inc'
C
      DOUBLE PRECISION dra, ddec
      REAL      arat, back, best, bvar, csumi, csumo, den
      REAL      psumi, psumo
      REAL      sest, rsig, srat, svar
      REAL      vari, varo, xpos, xsum, xtmp, ypos, ysum, ytmp
      INTEGER   i1, ib2, icnti, icnto, id2, imap
      INTEGER   imxh, imxl, imyh, imyl, is2, israd
      INTEGER   jmap, j1, l, l1, lhi, llo, m, m1, mhi, mlo
C
C Find corners of analysis box in map coordinates
      CALL POS_DT_IM(Box(1),Box(2),imxl,imyl)
      imxl = MIN(MAX(1,imxl),Nrow)
      imyl = MIN(MAX(1,imyl),Ncol)
      CALL POS_DT_IM(Box(3),Box(4),imxh,imyh)
      imxh = MIN(MAX(1,imxh),Nrow)
      imyh = MIN(MAX(1,imyh),Ncol)
C
C Start loop over search box.
      DO imap=imxl, imxh
         DO 400 jmap=imyl, imyh
            IF ( Rmap(imap,jmap).EQ.NO ) THEN
               IF ( .NOT.Qpoint ) THEN
                  i1 = imap - imxl + 1
                  j1 = jmap - imyl + 1
                  Sorc(i1,j1) = NO
                  Sdev(i1,j1) = NO
                  Bgpix(i1,j1) = NO
                  Excess(i1,j1) = NO
               ELSE
                  CALL PTBUF('Sorry, no valid data at center.',-1)
               END IF
               GOTO 400
            END IF
C
C Find position of i,j in terms of detector coordinates
            CALL POS_MAP_DET(imap,jmap,xpos,ypos)
C
C Find local size of detection cell (Npsf is in map units).
            CALL PSFSIG(xpos, ypos, rsig)
            israd = 1 + INT(Srad*rsig/Ifac)
            is2 = MAX(1,NINT((srad*rsig)**2/(Ifac*Ifac)))
            Npsf = MAX(israd+1,MAX(1, NINT(brad*rsig/Ifac)))
            ib2 = MAX(2,NINT((brad*rsig)**2/(Ifac*Ifac)))
C
C Set up array giving the weightings due to the PSF.
            CALL PSFSET(Ifac, xpos, ypos, Mxpsf, Npsf, Xpect)
C
C Initialize sums
            psumi = 0.0
            psumo = 0.0
            csumi = 0.0
            csumo = 0.0
            icnti = 0
            icnto = 0
C
C Loop over detection cell, calculating sums.
            llo = -Npsf - (MIN(imap-Npsf,1)-1)
            lhi =  Npsf - (MAX(imap+Npsf,Nrow)-Nrow)
            DO l=llo, lhi
               l1 = imap + l
               mlo = -Npsf - (MIN(jmap-Npsf,1)-1)
               mhi =  Npsf - (MAX(jmap+Npsf,Ncol)-Ncol)
               DO m=mlo, mhi
                  m1 = jmap + m
                  id2 = l*l + m*m
                  IF ( Rmap(l1,m1).NE.NO .AND. id2.LE.ib2 ) THEN
                     IF ( id2.LE.is2 ) THEN
                        psumi = psumi + Xpect(l,m)
                        csumi = csumi + Rmap(l1,m1)
                        icnti = icnti + 1
                     ELSE
                        psumo = psumo + Xpect(l,m)
                        csumo = csumo + Rmap(l1,m1)
                        icnto = icnto + 1
                     END IF
                  END IF
               END DO
            END DO
C
C If less than the requested fraction of the bins were included then give up.
            IF ( icnto+icnti.LT.Sfparm(1)*(2*Npsf+1)*(2*Npsf+1) ) THEN
               IF ( .NOT.Qpoint ) THEN
                  i1 = imap - imxl + 1
                  j1 = jmap - imyl + 1
                  Sorc(i1,j1) = NO
                  Sdev(i1,j1) = NO
                  Bgpix(i1,j1) = NO
                  Excess(i1,j1) = NO
               ELSE
                  CALL PTBUF('Sorry, not enough data points in map.',-1)
               END IF
               GOTO 400
            END IF
C
            arat = (icnti+icnto)/float(icnti)
            den = psumi*arat - psumi - psumo
            best = (psumi*csumo - psumo*csumi)/(den*icnti*Ifac*Ifac)
            bvar = (csumi*psumo**2+psumi*csumo)/(den*den)
            sest = (csumi*(arat-1.)-csumo)/den
C Use Gehrel's formula (1986, ApJ, 303, p336).
            vari = (1. + SQRT(csumi+0.75))**2
            varo = (1. + SQRT(csumo+0.75))**2
            svar = (vari*(arat-1.)**2+varo)/(den*den)
C
            IF ( Qpoint ) THEN
C
               xsum = 0.0
               ysum = 0.0
               llo = -israd - (MIN(imap-israd,1)-1)
               lhi =  israd - (MAX(imap+israd,Nrow)-Nrow)
               DO l=llo, lhi
                  l1 = imap + l
                  mlo = -israd - (MIN(jmap-israd,1)-1)
                  mhi =  israd - (MAX(jmap+israd,Ncol)-Ncol)
                  DO m=mlo, mhi
                     m1 = jmap + m
                     id2 = l*l+m*m
                     IF ( Rmap(l1,m1).NE.NO .AND. id2.LE.is2 ) THEN
                        xsum = xsum + l*Rmap(l1,m1)
                        ysum = ysum + m*Rmap(l1,m1)
                     END IF
                  END DO
               END DO
               xtmp = xpos + Ifac*xsum/csumi
               ytmp = ypos + Ifac*ysum/csumi
               srat = (csumi - best*icnti*Ifac*Ifac)/psumi
C
C If only one point asked for calculate upper limit and dump out all data
               CALL PTBUF(' ',-1)
               CALL POS_DT_RA(xtmp,ytmp,dra,ddec)
               WRITE(*,231) xpos, ypos, sest/SQRT(svar)
 231           FORMAT(' Center  : ',F8.1,1X,F8.1,'; SNR = ',F7.2)
               WRITE(*,241) xtmp, ytmp, srat, dra, ddec
 241           FORMAT(' Position: ',F8.1,1X,F8.1,'; counts=',F10.1,
     &            '; World=',F10.5,',',F9.5)
               WRITE(*,251) sest, SQRT(svar), best, SQRT(bvar)
 251           FORMAT(' Sest= ',F10.1,', sig(S)=',F8.1,
     &            ', bgdp=',F9.3,', sig(B)=',F9.3)
               WRITE(*,261) rsig, israd*Ifac, Npsf*Ifac, psumi
 261           FORMAT(' PSF_sig = ',F7.2,', Srad=',I5,
     &            ' dpix, Brad=',I5,' dpix, psumi=',F7.3)
               CALL PTBUF(' ',-1)
            ELSE
C
C Place data in arrays
               i1 = imap - imxl + 1
               j1 = jmap - imyl + 1
C Sigma based on source
               Sorc(i1,j1) = sest
               Sdev(i1,j1) = SQRT(svar)
               Bgpix(i1,j1) = best
               back = icnti*csumo/icnto
               Excess(i1,j1) = (csumi-back)/SQRT(MAX(1.0,back))
            END IF
C
  400    CONTINUE
      END DO
C
      RETURN
      END
