C---
      SUBROUTINE UINFO(IPAR, CNAME, NPAR)
      INTEGER   IPAR, NPAR
      CHARACTER CNAME*(*)
C---
C Uses 2D aray as model.
C---
C IPAR    I    The parameter number.
C CNAME     O  The name of the paramter IPAR.  Note if IPAR=0, then
C              -return the name of the model.
C NPAR      O  The number of parameters in your user model.
C---
C Number of parameters
      NPAR=5
      IF ( IPAR.EQ.0 ) THEN
C Component name
         CNAME='MOFFAT'
      ELSE IF ( IPAR.EQ.1 ) THEN
         CNAME='X'
      ELSE IF ( IPAR.EQ.2 ) THEN
         CNAME='Y'
      ELSE IF ( IPAR.EQ.3 ) THEN
         CNAME='SIG'
      ELSE IF ( IPAR.EQ.4 ) THEN
         CNAME='beta'
      ELSE IF ( IPAR.EQ.5 ) THEN
         CNAME='Norm'
      END IF
      RETURN
      END
C*********
      SUBROUTINE ULIMIT(Pval, PLIM, NT, NPAR)
      REAL      Pval(*), PLIM(3,*)
      INTEGER   NT, NPAR
C---
C ULIMIT is always called after any parameter values have been changed
C and before UFNY is called.
C---
C Pval(*)   I/O  The current parameter values
C PLIM(1,*) I    If <0 then the corresponding parameter is frozen
C NT        I    Pointer to first parameter value in array Pval(*)
C NPAR      I    Number of parameters
C---
C index must be positive
      pval(nt+3) = ABS(pval(nt+3))
      RETURN
      END
C*********
      REAL FUNCTION UFNY(Xt, Pval, Nt, Npar)
      REAL      Xt(*), Pval(*)
      INTEGER   Nt, Npar
C---
C UFNY the function that actually calculates user component at
C location X with parameter values given by Pval.
C---
C X       I    The current X value
C Pval    I    The current parameter values
C NT      I    Pointer to first parameter value in array Pval(*)
C NPAR    I    Number of parameters
C---
      REAL    r2
C
      r2 = (xt(1)-pval(nt))**2+(xt(2)-pval(nt+1))**2
      UFNY = pval(nt+4)/(1.+r2/(4*pval(nt+2)**2))**pval(nt+3)
      RETURN
      END
C*********
      SUBROUTINE UDERIV(Xt, Pval, PLIM, DERIV, NT, NPAR)
      REAL      Xt(*), Pval(*), Plim(3,*), Deriv(*)
      INTEGER   Nt, Npar
C---
C UDERIV should calculate the derivative of the UFNY function with respect
C to each parameter.
C---
C X       I    The current X value
C Pval    I    The current parameter values
C PLIM    I    The constraints array
C DERIV     O  The calculated derivative
C NT      I    Pointer to first parameter value in array Pval(*)
C NPAR    I    Number of parameters
C---
      REAL      UFNY
C
      REAL      dx, ptmp, tmp, ufny0
      INTEGER   i
C---
C
      UFNY0=UFNY(Xt,Pval,NT,NPAR)
      DO 190 I=NT,NT+4
         IF(PLIM(1,I).GE.0.) THEN
            PTMP=Pval(I)
            TMP=ABS(PTMP)
            IF(TMP.GT.0.01 .AND. TMP.LT.65000.) THEN
C- For scaled parameters, use a fixed step size.  This is more
C- numerically stable.
               DX=.01
            ELSE IF(TMP.LT.1.E-15) THEN
C- Near zero, avoid divide by zero problems.
               DX=.01
            ELSE
               DX=.001*TMP
            END IF
            Pval(I)=Pval(I)+DX
            CALL ULIMIT(Pval,PLIM,NT,NPAR)
            DERIV(I)=(UFNY(Xt,Pval,NT,NPAR)-UFNY0)/DX
            Pval(I)=PTMP
         END IF
  190 CONTINUE
      RETURN
      END
