      SUBROUTINE GAM(A, X, ACC, G, GSTAR, IFLG, IFLGST)
      REAL      A, X, Acc, G, Gstar
      INTEGER   Iflg, Iflgst
C---
C     LET  GAMMA(A)  DENOTE THE GAMMA FUNCTION AND  GAM(A,X)  THE
C (COMPLEMENTARY) INCOMPLETE GAMMA FUNCTION,
C
C    GAM(A,X)=INTEGRAL FROM T=X TO T=INFINITY OF EXP(-T)*T**(A-1).
C
C LET  GAMSTAR(A,X)  DENOTE TRICOMI:S FORM OF THE INCOMPLETE GAMMA
C FUNCTION, WHICH FOR A.GT.0. IS DEFINED BY
C
C  GAMSTAR(A,X)=(X**(-A)/GAMMA(A))*INTEGRAL FROM T=0 TO T=X OF
C                EXP(-T)*T**(A-1),
C
C AND FOR A.LE.0. BY ANALYTIC CONTINUATION. FOR THE PURPOSE OF
C THIS SUBROUTINE, THESE FUNCTIONS ARE NORMALIZED AS FOLLOWS&
C
C             GAM(A,X)/GAMMA(A),  IF A.GT.0.,
C
C     G(A,X)=
C
C             EXP(X)*X**(-A)*GAM(A,X),  IF A.LE.0.,
C
C     GSTAR(A,X)=(X**A)*GAMSTAR(A,X).
C
C THE PROGRAM BELOW ATTEMPTS TO EVALUATE  G(A,X)  AND  GSTAR(A,X),
C BOTH TO AN ACCURACY OF ACC SIGNIFICANT DECIMAL DIGITS, FOR ARBI-
C TRARY REAL VALUES OF  A  AND NONNEGATIVE VALUES OF  X. THE SUB-
C ROUTINE AUTOMATICALLY CHECKS FOR UNDERFLOW AND OVERFLOW CONDI-
C TIONS AND RETURNS APPROPRIATE WARNINGS THROUGH THE OUTPUT PARA-
C METERS  IFLG, IFLGST. A RESULT THAT UNDERFLOWS IS RETURNED WITH
C THE VALUE 0., ONE THAT OVERFLOWS WITH THE VALUE OF THE LARGEST
C MACHINE-REPRESENTABLE NUMBER.
C
C     NEAR LINES IN THE (A,X)-PLANE, A.LT.0., ALONG WHICH  GSTAR
C VANISHES, THE ACCURACY SPECIFIED WILL BE ATTAINED ONLY IN TERMS
C OF ABSOLUTE ERROR, NOT RELATIVE ERROR. THERE ARE OTHER (RARE)
C INSTANCES IN WHICH THE ACCURACY ATTAINED IS SOMEWHAT LESS THAN
C THE ACCURACY SPECIFIED. THE DISCREPANCY, HOWEVER, SHOULD NEVER
C EXCEED ONE OR TWO (DECIMAL) ORDERS OF ACCURACY# NO INDICATION
C OF THIS IS GIVEN THROUGH ERROR FLAGS.
C
C     PARAMETER LIST&
C
C        A - - THE FIRST ARGUMENT OF G AND GSTAR. TYPE REAL.
C        X - - THE SECOND ARGUMENT OF G AND GSTAR. TYPE REAL.
C      ACC - - THE NUMBER OF CORRECT SIGNIFICANT DECIMAL DIGITS
C              DESIRED IN THE RESULTS. TYPE REAL.
C        G - - AN OUTPUT VARIABLE RETURNING THE VALUE OF G(A,X).
C              TYPE REAL.
C    GSTAR - - AN OUTPUT VARIABLE RETURNING THE VALUE OF
C              GSTAR(A,X). TYPE REAL.
C     IFLG - - AN ERROR FLAG INDICATING A NUMBER OF ERROR CONDI-
C              TIONS IN G UPON EXECUTION. TYPE INTEGER.
C   IFLGST - - AN ERROR FLAG INDICATING A NUMBER OF ERROR CONDI-
C              TIONS IN GSTAR UPON EXECUTION. TYPE INTEGER.
C              THE VALUES OF IFLG AND IFLGST HAVE THE FOLLOWING
C              MEANINGS&
C              0 - NO ERROR CONDITION.
C              1 - ILLEGAL NEGATIVE ARGUMENT X. THE ROUTINE EXITS
C                  WITH THE VALUES ZERO FOR G AND GSTAR.
C              2 - INFINITELY LARGE RESULT AT X=0. THE ROUTINE
C                  RETURNS THE LARGEST MACHINE-REPRESENTABLE NUMBER.
C              3 - (ONLY FOR IFLGST) GSTAR IS INDETERMINATE AT
C                  A=0. AND X=0. THE ROUTINE RETURNS THE VALUE 1.,
C                  WHICH IS THE LIMIT VALUE AS X TENDS TO ZERO FOR
C                  FIXED A=0.
C              4 - THE RESULT UNDERFLOWS. IT IS SET EQUAL TO 0.
C              5 - THE RESULT OVERFLOWS. IT IS SET EQUAL TO THE
C                  LARGEST MACHINE-REPRESENTABLE NUMBER, WITH
C                  PROPER SIGN.
C              6 - CONVERGENCE FAILS WITHIN 600 ITERATIONS, EITHER
C                  IN TAYLOR:S SERIES OR IN LEGENDRE:S CONTINUED
C                  FRACTION. REASON UNKNOWN. THE COMPUTATION IS
C                  ABORTED AND THE ROUTINE RETURNS WITH ZERO
C                  VALUES FOR G AND GSTAR.
C
C     ALL MACHINE-DEPENDENT PARAMETERS ARE COLLECTED IN THE FIRST
C DATA DECLARATION. THEY ARE AS FOLLOWS&
C
C     PREC - - THE SINGLE PRECISION ACCURACY, TO BE SET APPROXI-
C              MATELY EQUAL TO BETA*ALOG(2.)/ALOG(10.), WHERE BETA
C              IS THE NUMBER OF BINARY DIGITS AVAILABLE IN THE MAN-
C              TISSA OF THE SINGLE PRECISION FLOATING-POINT WORD.
C              TYPE REAL.
C   TOPEXP - - APPROXIMATELY THE LARGEST POSITIVE NUMBER T SUCH
C              THAT 10.**T IS STILL REPRESENTABLE ON THE COMPUTER
C              IN SINGLE PRECISION FLOATING-POINT ARITHMETIC.
C              TYPE REAL.
C   BOTEXP - - APPROXIMATELY THE SMALLEST NEGATIVE NUMBER T SUCH
C              THAT 10.**T IS STILL REPRESENTABLE ON THE COMPUTER
C              IN SINGLE PRECISION FLOATING-POINT ARITHMETIC.
C              TYPE REAL.
C
C IN THE PROGRAM BELOW THESE PARAMETERS ARE SET TO CORRESPOND TO
C THE MACHINE CHARACTERISTICS OF THE CDC 6500 COMPUTER.
C
C     THE SECOND DATA DECLARATION CONTAINS THE SINGLE PRECISION
C VALUE OF ALOG(10.). THE NEXT DATA DECLARATION CONTAINS THE SUCCES-
C SIVE COEFFICIENTS IN THE MACLAURIN EXPANSION OF (1/GAMMA(A+1))-1.
C THEY ARE GIVEN TO AS MANY DECIMAL PLACES AS IS NECESSARY TO ACHIEVE
C MACHINE PRECISION (ON THE CDC 6500 COMPUTER) IN THE RANGE
C ABS(A).LE..5. MORE ACCURATE VALUES OF THESE COEFFICIENTS (TO
C 31 DECIMAL PLACES) CAN BE FOUND IN TABLE 5 OF J.W.WRENCH,JR.,
C CONCERNING TWO SERIES FOR THE GAMMA FUNCTION, MATH. COMPUT.
C 22, 1968, 617-626.
C
C     THE SUBROUTINE CALLS ON A FUNCTION SUBROUTINE, NAMED  ALGA,
C WHICH IS TO PROVIDE SINGLE PRECISION VALUES OF THE LOGARITHM OF
C THE GAMMA FUNCTION FOR ARGUMENTS LARGER THAN OR EQUAL TO .5.
C A POSSIBLE VERSION OF SUCH A FUNCTION SUBROUTINE IS APPENDED
C TO THE PRESENT SUBROUTINE. IT IS TAYLORED TO THE ACCURACY RE-
C QUIREMENTS OF THE CDC 6500 COMPUTER, AND USES RATIONAL APPROXI-
C MATIONS DUE TO CODY AND HILLSTROM (MATH. COMPUT. 21, 1967, 198-
C 203).
C
C     REFERENCE - W. GAUTSCHI, ::A COMPUTATIONAL PROCEDURE FOR
C INCOMPLETE GAMMA FUNCTIONS, ACM TRANS. MATH. SOFTWARE.
C---
      REAL      ALGA
C
      REAL      c(18)
      REAL      aa, aaeps, ae, aep1, aeps, alg, algs, algep1, algp1
      REAL      al10, alpha, alprec, alx, ainf, ap1, bot, botexp
      REAL      eps, eps1, fma, ga, p, prec, q, r, rho
      REAL      s, sga, sgae, sgga, sgt, sum
      REAL      term, t, top, topexp, u, v, xma, xpa, y
      INTEGER   i, k, k1, ma
C
CD      DATA PREC, TOPEXP, BOTEXP /1.445E1,3.22E2,-2.93E2/
      DATA PREC, TOPEXP, BOTEXP /7.2E0,3.85E1,-4.51E1/
      DATA AL10 /2.30258509299405E0/
      DATA C /.577215664901533E0,-.655878071520254E0,
     * -4.2002635034095E-2,.16653861138229E0,-4.219773455554E-2,
     * -9.62197152788E-3,7.21894324666E-3,-1.1651675919E-3,
     * -2.152416741E-4,1.28050282E-4,-2.0134855E-5,-1.25049E-6,
     * 1.13303E-6,-2.0563E-7,6.12E-9,5.00E-9,-1.2E-9,1.E-10/
      G = 0.
      GSTAR = 0.
      IF (X.LT.0.) GO TO 290
C
C INITIALIZATION
C
      IFLG = 0
      IFLGST = 0
      I = 0
      IF (X.GT.0.) ALX = ALOG(X)
      ALPHA = X + .25
      IF (X.LT..25 .AND. X.GT.0.) ALPHA = ALOG(.5)/ALX
      ALPREC = AL10*PREC
      TOP = AL10*TOPEXP
      BOT = AL10*BOTEXP
      AINF = 10.**TOPEXP
      EPS = .5*10.**(-ACC)
      EPS1 = EPS/100.
      SGA = 1.
      IF (A.LT.0.) SGA = -SGA
      AE = A
      AA = ABS(A)
      AP1 = A + 1.
      AEP1 = AP1
      MA = .5 - A
      FMA = FLOAT(MA)
      AEPS = A + FMA
      SGAE = 1.
      IF (AEPS.LT.0.) SGAE = -SGAE
      AAEPS = ABS(AEPS)
      ALGP1 = 0.
C
C EVALUATION OF THE LOGARITHM OF THE ABSOLUTE VALUE OF
C GAMMA(A+1.) AND DETERMINATION OF THE SIGN OF GAMMA(A+1.)
C
      SGGA = 1.
      IF (MA.LE.0) GO TO 10
      IF (AEPS.EQ.0.) GO TO 20
      SGGA = SGAE
      IF (MA.EQ.2*(MA/2)) SGGA = -SGGA
      ALGP1 = ALGA(AEPS+1.) - ALOG(AAEPS)
      IF (MA.EQ.1) GO TO 20
      ALGP1 = ALGP1 + ALGA(1.-AEPS) - ALGA(FMA-AEPS)
      GO TO 20
   10 ALGP1 = ALGA(AP1)
   20 ALGEP1 = ALGP1
      IF (X.GT.0.) GO TO 60
C
C EVALUATION OF GSTAR(A,0.) AND G(A,0.)
C
C      IF (A) 30, 40, 50
      IF (A.GT.0.0) GOTO 50
      IF (A.EQ.0.0) GOTO 40
   30 IFLGST = 2
      GSTAR = AINF
      G = 1./AA
      RETURN
   40 IFLGST = 3
      GSTAR = 1.
      IFLG = 2
      G = AINF
      RETURN
   50 G = 1.
      RETURN
   60 IF (A.GT.ALPHA) GO TO 220
      IF (X.GT.1.5) GO TO 240
      IF (A.LT.-.5) GO TO 170
C
C DIRECT EVALUATION OF G(A,X) AND GSTAR(A,X) FOR X.LE.1.5
C AND -.5.LE.A.LE.ALPHA(X)
C
      GSTAR = 1.
      IF (A.GE..5) GO TO 110
   70 SUM = C(18)
      DO 80 K=1,17
        K1 = 18 - K
        SUM = AE*SUM + C(K1)
   80 CONTINUE
      GA = -SUM/(1.+AE*SUM)
      Y = AE*ALX
      IF (ABS(Y).GE.1.) GO TO 100
      SUM = 1.
      TERM = 1.
      K = 1
   90 K = K + 1
      IF (K.GT.600) GO TO 330
      TERM = Y*TERM/FLOAT(K)
      SUM = SUM + TERM
      IF (ABS(TERM).GT.EPS1*SUM) GO TO 90
      U = GA - SUM*ALX
      GO TO 120
  100 U = GA - (EXP(Y)-1.)/AE
      GO TO 120
  110 U = EXP(ALGA(A)) - (X**A)/A
  120 P = AE*X
      Q = AEP1
      R = AE + 3.
      TERM = 1.
      SUM = 1.
      K = 1
  130 K = K + 1
      IF (K.GT.600) GO TO 330
      P = P + X
      Q = Q + R
      R = R + 2.
      TERM = -P*TERM/Q
      SUM = SUM + TERM
      IF (ABS(TERM).GT.EPS1*SUM) GO TO 130
      V = (X**AEP1)*SUM/AEP1
      G = U + V
      IF (I.EQ.1) GO TO 180
C      IF (A) 140, 150, 160
      IF (A.GT.0.0) GOTO 160
      IF (A.EQ.0.0) GOTO 150
  140 T = EXP(X)*X**(-A)
      G = T*G
      GSTAR = 1. - A*G*EXP(-ALGP1)/T
      RETURN
  150 G = EXP(X)*G
      RETURN
  160 G = A*G*EXP(-ALGP1)
      GSTAR = 1. - G
      RETURN
C
C RECURSIVE EVALUATION OF G(A,X) FOR X.LE.1.5 AND A.LT.-.5
C
  170 I = 1
      AE = AEPS
      AEP1 = AEPS + 1.
      IF (X.LT..25 .AND. AE.GT.ALPHA) GO TO 210
      GO TO 70
  180 G = G*EXP(X)*X**(-AE)
      DO 190 K=1,MA
        G = (1.-X*G)/(FLOAT(K)-AE)
  190 CONTINUE
      ALG = ALOG(G)
C
C EVALUATION OF GSTAR(A,X) IN TERMS OF G(A,X)
C
  200 GSTAR = 1.
      IF (MA.GE.0 .AND. AEPS.EQ.0.) RETURN
      SGT = SGA*SGGA
      T = ALOG(AA) - X + A*ALX + ALG - ALGP1
      IF (T.LT.-ALPREC) RETURN
      IF (T.GE.TOP) GO TO 320
      GSTAR = 1. - SGT*EXP(T)
      RETURN
  210 I = 2
      ALGEP1 = ALGA(AEP1)
C
C EVALUATION OF GSTAR(A,X) FOR A.GT.ALPHA(X) BY TAYLOR
C EXPANSION
C
  220 G = 1.
      TERM = 1.
      SUM = 1.
      K = 0
  230 K = K + 1
      IF (K.GT.600) GO TO 340
      TERM = X*TERM/(AE+FLOAT(K))
      SUM = SUM + TERM
      IF (ABS(TERM).GT.EPS*SUM) GO TO 230
      ALGS = AE*ALX - X + ALOG(SUM) - ALGEP1
      IF (ALGS.LE.BOT) GO TO 310
      GSTAR = EXP(ALGS)
      G = 1. - GSTAR
      IF (I.NE.2) RETURN
      G = G*EXP(ALGEP1)/AE
      GO TO 180
C
C EVALUATION OF G(A,X) FOR X.GT.1.5 AND A.LE.ALPHA(X) BY
C MEANS OF THE LEGENDRE CONTINUED FRACTION
C
  240 GSTAR = 1.
      XPA = X + 1. - A
      XMA = X - 1. - A
      P = 0.
      Q = XPA*XMA
      R = 4.*XPA
      S = -A + 1.
      TERM = 1.
      SUM = 1.
      RHO = 0.
      K = 1
  250 K = K + 1
      IF (K.GT.600) GO TO 330
      P = P + S
      Q = Q + R
      R = R + 8.
      S = S + 2.
      T = P*(1.+RHO)
      RHO = T/(Q-T)
      TERM = RHO*TERM
      SUM = SUM + TERM
      IF (ABS(TERM).GT.EPS*SUM) GO TO 250
C      IF (A) 260, 270, 280
      IF (A.GT.0.0) GOTO 280
      IF (A.EQ.0.0) GOTO 270
  260 G = SUM/XPA
      ALG = ALOG(G)
      GO TO 200
  270 G = SUM/XPA
      RETURN
  280 ALG = A*ALX - X + ALOG(A*SUM/XPA) - ALGP1
      IF (ALG.LE.BOT) GO TO 300
      G = EXP(ALG)
      GSTAR = 1. - G
      RETURN
  290 IFLG = 1
      IFLGST = 1
      RETURN
  300 IFLG = 4
      RETURN
  310 IFLGST = 4
      RETURN
  320 IFLGST = 5
      GSTAR = -SGT*AINF
      RETURN
  330 IFLG = 6
      RETURN
  340 IFLGST = 6
      RETURN
      END
