C The Good Time Interval (GTI) object.  All code that uses GTI's should
C in this file.
C---
      SUBROUTINE GTINEW(Lun, Ievent, Itcol, Cicol)
      INTEGER   Lun, Ievent, Itcol
      CHARACTER Cicol*(*)
C Entry GTIBEG
      INTEGER   Ievent1, Itcol1
      DOUBLE PRECISION Dtobeg
C Entry GTIEND
      DOUBLE PRECISION Dtoend
C ENTRY GTIEFF
      INTEGER   Neff
      REAL      Effic(Neff)
C ENTRY GTIINU
      DOUBLE PRECISION Dtime1
      INTEGER   Igood
C ENTRY GTIQDO/GTISDO
      LOGICAL   Qdogti
C ENTRY GTIQUT/GTISUT
      LOGICAL   Quser
C ENTRY GTISUT
      DOUBLE PRECISION Duts1, Dute1
C ENTRY GTITOT
      INTEGER   Lun1, igtccd(*)
      REAL      Tinteg
C ENTRY GTIVAL
      INTEGER   Iccd
C---
C GTINEW must be called before any other GTI routine in this file
C as GTINEW initilizes the data structures.
C---
C Lun     I    Unit number of open FITS events file
C Ievent  I    FITS header number containing EVENTS data
C Itcol   I    Time column number in EVENTS extension
C Cicol     O  Index column name in GTI extension (blank for no GTI)
C---
C 1997-Aug-27 - Implement indexed GTIs (for ACIS) [AFT]
C 1997-Aug-22 - Always read GTIs from file [AFT]
C 1997-Apr-18 - [AFT]
C---
      INTEGER   IPHASE
C MXGTI is the maximum number of FTIS extensions that contain GTIs.
C MXFILE is maximum number of open FITS files containing GTIs.
      INTEGER   MXGTI, MXFILE
      PARAMETER (MXGTI=20, MXFILE=3)
      REAL      NO
      PARAMETER (NO=-1.2E-34)
C
      INCLUDE 'time.inc'
C GTIEFF local variables:
      DOUBLE PRECISION dge, dgs
      INTEGER   igti, inext
C
      CHARACTER ctmp*64, cvalue*64
      CHARACTER cindex(MXFILE)*8
      SAVE      cindex
      CHARACTER ccol*16, ckey*16
      DOUBLE PRECISION dstart(MXGTI,MXFILE), dstop(MXGTI,MXFILE)
      SAVE             dstart,        dstop
      DOUBLE PRECISION dpstop(MXGTI,MXFILE)
      SAVE             dpstop
      DOUBLE PRECISION dtmp, dtime, dgtie, dgtis, dtcor
      REAL      rcor
      INTEGER   lunsav(MXFILE)
      SAVE      lunsav
      INTEGER   i, iadd, ibin, iext, ihsav, ifile
      INTEGER   istat, itmp, iv1, ivalue
      INTEGER   j, nccd, nps, nrws, ntims
      LOGICAL   qbad
C Local (hidden) data
      DOUBLE PRECISION dtbeg(MXFILE), dtend(MXFILE)
      SAVE             dtbeg,         dtend
      DOUBLE PRECISION dnul, dute, duts
      SAVE             dnul, dute, duts
      INTEGER   igtiex(MXGTI,MXFILE), ihdu(MXGTI,MXFILE)
      SAVE      igtiex,               ihdu
      INTEGER   irow(MXGTI,MXFILE), nrow(MXGTI,MXFILE)
      SAVE      irow,        nrow
      INTEGER   ichip(MXGTI,MXFILE)
      SAVE      ichip
      INTEGER   itdat(MXFILE), ngtiex(MXFILE), ntot(MXFILE)
      SAVE      itdat,         ngtiex,         ntot
      INTEGER   igscol(MXFILE), igecol(MXFILE)
      SAVE      igscol,         igecol
C idgti =1 use GTI, =0 ignore GTI.
      INTEGER   idgti, iutim, nfile
      SAVE      idgti, iutim, nfile
      DATA      idgti/1/, iutim/0/, nfile/0/
C
C---
C nfile is number of files for which we are remembering GTI extentions.
      IF ( nfile.LE.0 ) THEN
         nfile=1
         ifile=nfile
         lunsav(ifile)=Lun
      ELSE
C If a file is closed and reopened with same unit number, we can reuse.
         DO i=1,nfile
            IF ( lunsav(i).EQ.Lun ) THEN
               ifile=i
               lunsav(ifile)=Lun
               GOTO 120
            END IF
         END DO
         IF ( nfile.LT.MXFILE ) THEN
            nfile=nfile+1
         ELSE
            CALL PTBUF('GTINEW--Too many files.',-1)
         END IF
         ifile=nfile
         lunsav(ifile)=Lun
      END IF
      dtbeg(ifile) = -1.0
C
C GTI only apply to a column named "TIME"
  120 CONTINUE
      ntot(ifile) = 0
      istat = 0
      CALL FTKEYN('TTYPE',itcol,ckey,istat)
      CALL FTGKYS(Lun,ckey,cvalue,ctmp,istat)
      CALL UPC(cvalue)
      itdat(ifile) = 0
      IF ( cvalue(1:4).EQ.'TIME' ) THEN
         itdat(ifile) = 1
C Special kludge for CXC ACIS data.  Assume ccd_id is always the index,
C so if that column exists, then it is the index column.
         itmp = 0
         istat = 0
         CALL FTGCNO(Lun, .FALSE., 'ccd_id', itmp, istat)
         IF ( itmp.GT.0 ) THEN
C Must be upper case...
            cindex(ifile) = 'CCD_ID'
         ELSE
C Not ACIS...
            istat=0
            CALL FTGCNO(Lun, .FALSE., 'CCDNR', itmp, istat)
            IF ( itmp.GT.0 ) THEN
C But is XMM
               cindex(ifile) = 'CCDNR'
            ELSE
               cindex(ifile) = ' '
            END IF
         END IF
C Zero out any previous values.
         DO i=1,MXGTI
            irow(i,ifile) = 0
            nrow(i,ifile) = 0
         END DO
C
C Save the current header.
         CALL FTGHDN(Lun, ihsav)
C Scan the current FITS file for all GTI data extensions.
         ngtiex(ifile) = 0
         igscol(ifile) = 0
         igecol(ifile) = 0
         DO i=2,999
            istat = 0
            CALL FTMAHD(Lun, i, iext, istat)
            CALL FTGKYS(Lun, 'EXTNAME', cvalue, ctmp, istat)
            IF ( istat.NE.0 ) GOTO 140
            IF(cvalue(1:3).EQ.'GTI'.OR.cvalue(1:6).EQ.'STDGTI') THEN
               CALL FTGKYJ(Lun, 'NAXIS2', ntims, ctmp, istat)
               iv1 = 1
               IF ( cindex(ifile).NE.' ' ) THEN
                  IF ( cindex(ifile)(1:4).EQ.'CCDN' ) THEN
C XMM data, CCDNR value is encoded in EXTNAME.
                     IF ( cvalue(1:3).EQ.'GTI' ) THEN
C For GTIxxx02 (range is 0-11)
                        READ(cvalue, 111) ivalue
  111                   FORMAT(3X,I3)
                        iv1=ivalue+1
                     ELSE
C For STDGTIxx (range is 1-12)
                        READ(cvalue, 121) ivalue
  121                   FORMAT(6X,I2)
                        iv1=ivalue
                     END IF
                  ELSE
C ACIS, the index keyword is actually in the extension.
                     istat = 0
                   CALL FTGKYJ(Lun, cindex(ifile), ivalue, ctmp, istat)
                     IF ( istat.NE.0 ) THEN
                        WRITE(*,141) cindex(ifile)
  141                   FORMAT('GTI: Unable to find ',A,' (ignoring).')
                        cindex(ifile)=' '
                        ivalue=0
                     END IF
                     iv1=ivalue+1
                  END IF
               END IF
C Figure out if this CCD is new or if an entry already exists.
               iadd=0
               DO j=1,ngtiex(ifile)
                  IF ( ichip(j,ifile).EQ.iv1 ) THEN
                     IF ( cvalue(1:3).NE.'GTI' ) THEN
                        WRITE(*,*) 'GTI--Yikes! Using ',cvalue(1:6),
     &                  ' hope this is what you wanted.'
                     END IF
                     iadd=j
                  END IF
               END DO
               IF ( iadd.EQ.0 ) THEN
C Does not already exist, so add a new row.
                  IF ( ngtiex(ifile).GE.MXGTI ) THEN
                     CALL PTBUF(
     &      'GTI--Too many GTI extensions--Increase MXGTI.',-1)
                     STOP
                  END IF
                  ngtiex(ifile) = ngtiex(ifile) + 1
                  iadd = ngtiex(ifile)
               ELSE
C We are overwriting an existing row, subtract out what is about to be
C overwritten.
                  ntot(ifile) = ntot(ifile) - nrow(iv1,ifile)
               END IF
               igtiex(iadd,ifile)= i
               ichip(iadd,ifile) = iv1
               ihdu(iv1,ifile) = i
               nrow(iv1,ifile) = ntims
               ntot(ifile) = ntot(ifile) + ntims
               IF ( igscol(ifile).EQ.0 ) THEN
C Here we assume the columns are in the same order in all extensions
C so we only read once.
                  istat = 0
                  igscol(ifile) = 0
                  igecol(ifile) = 0
        CALL FTGCNN(Lun,.FALSE.,'START',ccol,igscol(ifile),istat)
        CALL FTGCNN(Lun,.FALSE.,'STOP', ccol,igecol(ifile),istat)
               END IF
               dnul = NO
        CALL FTGCVD(Lun,igscol(ifile),1,1,1,dnul,dtmp,qbad,istat)
               dstart(iv1,ifile)=dtmp
        CALL FTGCVD(Lun,igecol(ifile),1,1,1,dnul,dtmp,qbad,istat)
               dstop(iv1,ifile)=dtmp
C Don't need to set this but just in case...
               dpstop(iv1,ifile) = 1e38
               irow(iv1,ifile) = 1
            END IF
         END DO
      END IF
C---
  140 CONTINUE
      IF ( ntot(ifile).GT.0 ) THEN
         WRITE(*,*) 'File contains',ntot(ifile),' GTIs in ',
     &         ngtiex(ifile),' extensions.'
         Cicol = cindex(ifile)
      ELSE
         Cicol=' '
      END IF
C---
C Restore pointer to previous header location
      istat = 0
      CALL FTMAHD(Lun, ihsav, iext, istat)
C
      RETURN
C*********
      ENTRY GTIBEG(Lun1, Ievent1, Itcol1, Dtobeg)
C---
C Find the earliest GTI, i.e., the observation start time.
C---
C Lun1    I    Logical unit number of FITS events list
C Ievent1 I
C Itcol1  I
C Dtbeg     O  The observation start time
C---
      ifile=0
      DO i=1,nfile
         IF ( lunsav(i).EQ.Lun1 ) THEN
            ifile=i
         END IF
      END DO
C
      IF ( ntot(ifile).EQ.0 .OR.
     &    (idgti.EQ.0.AND.dtbeg(ifile).LT.0.0) ) THEN
         CALL PTBUF('File contains no good time intervals.',-1)
C Goto event data
         istat = 0
         CALL FTGHDN(Lun1, ihsav)
         CALL FTMAHD(Lun1, Ievent1, iext, istat)
C Read the column name for the time column.
         CALL FTKEYN('TTYPE',itcol1,ckey,istat)
         CALL FTGKYS(Lun1,ckey,cvalue,ctmp,istat)
C Check for a TIMEDEL value.  If Timedel exists, then the end of a
C time bin is dtime+timedel
         dtmp = 0.0D0
         IF ( itdat(ifile).GT.0 ) THEN
            istat = 0
            CALL FTGKYD(Lun1,'TIMEDEL', dtmp, ctmp, istat)
         END IF
C
         CALL PTBUF('Using first and last points.',-1)
         istat = 0
         CALL FTGKYJ(Lun1, 'NAXIS2', nrws, ctmp, istat)
         CALL FTGCVD(Lun1,itcol1,1,1,1,dnul,dtbeg(ifile),qbad,istat)
         dtend(ifile) = MAX( dtend(ifile), dtbeg(ifile)+dtmp)
         CALL FTGCVD(Lun1,itcol1,nrws,1,1,dnul,dtime,qbad,istat)
         dtbeg(ifile) = MIN( dtbeg(ifile), dtime)
         dtend(ifile) = MAX( dtend(ifile), dtime+dtmp)
C         CALL PTBUF('Calculating an approximation now.',-1)
C         DO i=1,nrws
C            CALL FTGCVD(Lun1,itcol1,i,1,1,dnul,dtime,qbad,istat)
C            dtbeg(ifile) = MIN( dtbeg(ifile), dtime)
C            dtend(ifile) = MAX( dtend(ifile), dtime+dtmp)
C         END DO
         CALL FTMAHD(Lun1, ihsav, iext, istat)
         IF ( iutim.NE.0 ) THEN
            dtbeg(ifile) = MAX( dtbeg(ifile), duts)
            dtend(ifile) = MIN( dtend(ifile), dute)
         END IF
C
         WRITE (*,*) 'Data start, data end:',dtbeg(ifile),dtend(ifile)
         ntot(ifile) = -1
      END IF
C
      IF ( ntot(ifile).GT.0 ) THEN
         Dtobeg = 1.0D37
         CALL FTGHDN(Lun1, ihsav)
         DO i=1,ngtiex(ifile)
            istat = 0
            CALL FTMAHD(Lun1, igtiex(i,ifile), iext, istat)
            CALL FTGKYJ(Lun1, 'NAXIS2', ntims, ctmp, istat)
            DO j=1, ntims
             CALL FTGCVD(Lun1,igscol(ifile),j,1,1,dnul,dgtis,qbad,istat)
               Dtobeg = MIN(Dtobeg, dgtis)
            END DO
         END DO
         CALL FTMAHD(Lun1, ihsav, iext, istat)
      ELSE
         Dtobeg = dtbeg(ifile)
      END IF
      IF ( iutim.NE.0 ) THEN
         Dtobeg = MAX(Dtobeg,duts)
      END IF
      RETURN
C*********
      ENTRY GTIDUR(Lun1, Tinteg)
C---
C Calculate the duration.  This should only be used if file contains
C no GTIs and no FITS keywords describing the exposure time.
C---
      ifile=0
      DO i=1,nfile
         IF ( lunsav(i).EQ.Lun1 ) THEN
            ifile=i
         END IF
      END DO
C---
      IF ( ntot(ifile).LE.0 ) THEN
C Guessed GTIs from data.
         Tinteg = dtend(ifile) - dtbeg(ifile)
      END IF
      RETURN
C*********
      ENTRY GTIEND(Lun1, Dtoend)
C---
C Find the last GTI, i.e., the observation end time.
C---
C Lun1    I    Logical unit number of FITS events list
C Dtoend    O  The observation end time
C---
      ifile=0
      DO i=1,nfile
         IF ( lunsav(i).EQ.Lun1 ) THEN
            ifile=i
         END IF
      END DO
C
      IF ( ntot(ifile).EQ.0 ) THEN
         CALL PTBUF('GTIEND--Bad code.',-1)
         STOP
      END IF
C
      IF ( ntot(ifile).GT.0 ) THEN
         Dtoend = -1.0D37
         CALL FTGHDN(Lun1, ihsav)
         DO i=1,ngtiex(ifile)
            istat = 0
            CALL FTMAHD(Lun1, igtiex(i,ifile), iext, istat)
            CALL FTGKYJ(Lun1, 'NAXIS2', ntims, ctmp, istat)
            DO j=1, ntims
             CALL FTGCVD(Lun1,igecol(ifile),j,1,1,dnul,dgtis,qbad,istat)
               Dtoend = MAX(Dtoend, dgtis)
            END DO
         END DO
         CALL FTMAHD(Lun1, ihsav, iext, istat)
      ELSE
         Dtoend = dtend(ifile)
      END IF
      IF ( iutim.NE.0 ) THEN
         Dtoend = MIN(Dtoend,dute)
      END IF
      RETURN
C*********
      ENTRY GTIEFF(Lun1, Igtccd, Neff, Effic)
C---
C When calculating a lightcurve, this routine initializes the efficiency
C array.  The efficiency array is the faction of the bin that is
C covered by a GTI.
C---
C Lun1    I    Logical unit number of FITS events list
C Igtccd  I    The CCD's included
C Neff    I    Number of bins in Effic to compute
C Effic     O  The efficiency array (percent of bin containing GTI.
C---
      ifile=0
      DO i=1,nfile
         IF ( lunsav(i).EQ.Lun1 ) THEN
            ifile=i
         END IF
      END DO
C
      IF ( ntot(ifile).LT.0 .OR. idgti.EQ.0 ) THEN
C Made up GTI is always efficient
         DO i = 1, Neff
            Effic(i) = 1.0
         END DO
      ELSE
C
C Initialize efficiency array to zero.
         DO i = 1, Neff
            Effic(i) = 0.0
         END DO
C
C Count number of CCDs included in the data.
         nccd = 0
         CALL FTGHDN(Lun1, ihsav)
         DO 290 i=1,ngtiex(ifile)
            istat = 0
            CALL FTMAHD(Lun1, igtiex(i,ifile), iext, istat)
C
            IF ( cindex(ifile).NE.' ' ) THEN
C For indexed GTI's, make sure we have actually read some data
C that matches the index.
               IF ( igtccd(ichip(i,ifile)).LE.0 ) GOTO 290
            END IF
            nccd = nccd + 1
            CALL FTGKYJ(Lun1, 'NAXIS2', ntims, ctmp, istat)
C
C Calculate efficiencies using good time intervals.  Note, this is the
C maximum active time of a bin.  dtime is the start time of a current
C data bin.
            igti = 0
C Force routine to find next GTI
            inext = 1
            DO 280 j=1, Neff
               dtime = Dts + (j-1)*Dtb
               IF ( Qphase ) THEN
                  ibin = IPHASE(dtime,Dtim0,Dnu,Dnudot,Neff)
               ELSE
                  ibin = j
               END IF
  230          CONTINUE
               IF ( inext.NE.0 ) THEN
                  IF ( ntot(ifile).LT.0 ) THEN
                     IF ( j.GT.1 ) GOTO 280
                     dgtis = dtbeg(ifile)
                     dgtie = dtend(ifile)
                  ELSE
  240                IF ( igti.LT.ntims ) THEN
                        igti = igti + 1
                        CALL FTGCVD(Lun1,igscol(ifile),igti,1,1,
     &                   dnul,dgtis,qbad,istat)
                        CALL FTGCVD(Lun1,igecol(ifile),igti,1,1,
     &                   dnul,dgtie,qbad,istat)
                     ELSE
                        GOTO 290
                     END IF
                  END IF
                  dgs = Dgtis - Doff
                  IF ( iutim.EQ.0 ) THEN
                     dge = Dgtie - Doff
                  ELSE
                     dge = MIN(Dgtie,dute) - Doff
                  END IF
                  IF ( dge.LT.dtime ) GOTO 230
               END IF
C---
C Below we account for the fact that the current data bin may contain
C anything from part of a GTI to several GTI's.  Due to possible round
C off error, we always compute the number of "points" (called nps) and
C compute efficiency based on that value.
 260           CONTINUE
               inext = 0
               IF ( dgs.LE.dtime ) THEN
C GTI starts prior to current bin.
                  IF ( dge.LE.dtime+Dtb ) THEN
C GTI ends in current bin
                     nps = NINT((dge-dtime)/dtp)
                     Effic(ibin) = Effic(ibin) + nps*dtp/Dtb
                     inext = 1
                  ELSE
C GTI covers entire bin.
                     Effic(ibin) = Effic(ibin) + 1.0
                  END IF
               ELSE IF ( dgs.LT.dtime+Dtb ) THEN
C GTI starts in bin
                  IF ( dge.LT.dtime+Dtb ) THEN
C GTI ends in bin
                     nps = NINT((dge-dgs)/dtp)
                     Effic(ibin) = Effic(ibin) + nps*dtp/Dtb
                     inext = 1
                  ELSE
C Rest of bin is in GTI.
                     nps = NINT((dtime+dtb-dgs)/dtp)
                     Effic(ibin) = Effic(ibin) + nps*dtp/Dtb
                  END IF
               END IF
C If we have not finished current bin, try to do so.
               IF ( inext.NE.0 ) GOTO 230
C---
  280       CONTINUE
  290    CONTINUE
         IF ( nccd.LE.0 ) THEN
            CALL PTBUF('GTIEFF--No CCDs were included.',-1)
            RETURN
         END IF
         rcor = 1./FLOAT(nccd)
         DO ibin= 1, Neff
            Effic(ibin) = rcor*Effic(ibin)
         END DO
C
         istat = 0
         CALL FTMAHD(Lun1, ihsav, iext, istat)
      END IF
C
      RETURN
C*********
      ENTRY GTIINU(Dtime1, Igood)
C---
C Checks to see if Dtime1 lies in user defined time limits. Note is
C is the fully corrected time including Doff.
C---
C Dtime1  I    The time being considered
C Igood     O  =0 outside the user time, =1 no user time or in limits.
C---
      Igood = 1
      IF ( iutim.NE.0 ) THEN
C If time outside user specificed bounds, no further checking needed.
         IF ( Dtime1.LT.duts .OR. dute.LT.Dtime1 ) THEN
            Igood = 0
         END IF
      END IF
      RETURN
C*********
      ENTRY GTIQDO(Qdogti)
C Return .TRUE. if GTI should be checked
      IF ( idgti.EQ.0 ) THEN
         Qdogti = .FALSE.
      ELSE
         Qdogti = .TRUE.
      END IF
      RETURN
C*********
      ENTRY GTIQUT(Quser, Duts1, Dute1)
C Return .TRUE. if user has selected time limits, .FALSE. otherwise.
      IF ( iutim.EQ.0 ) THEN
         Quser = .FALSE.
      ELSE
         Quser = .TRUE.
         Duts1 = Duts - Doff
         Dute1 = Dute - Doff
      END IF
      RETURN
C*********
      ENTRY GTISDO(Qdogti)
C---
C Specify if the GTIs should be checked
C---
      IF ( Qdogti ) THEN
         idgti = 1
      ELSE
         idgti = 0
      END IF
      RETURN
C*********
      ENTRY GTISUT(Duts1, Dute1)
C---
C Allow user to define a good time interval.
C---
      iutim = 1
      Duts = Duts1 + Doff
      Dute = Dute1 + Doff
      RETURN
C*********
      ENTRY GTITOT(Lun1, igtccd, Tinteg)
C---
C Sums total integration time using GTI intervals.
C---
      ifile=0
      DO i=1,nfile
         IF ( lunsav(i).EQ.Lun1 ) THEN
            ifile=i
         END IF
      END DO
C
      IF ( ifile.EQ.0 ) THEN
         Tinteg = -1.0
      ELSE IF ( ntot(ifile).LE.0 ) THEN
         Tinteg = -1.0
      ELSE
C Must read the FITS GTI extension.
         CALL FTGHDN(Lun1, ihsav)
         Tinteg = 0.0
         nccd = 0
         DO 490 i=1,ngtiex(ifile)
            istat = 0
            CALL FTMAHD(Lun1, igtiex(i,ifile), iext, istat)
C
            IF ( cindex(ifile).NE.' ' ) THEN
               iv1=ichip(i,ifile)
               IF ( iv1.LE.0 .OR. iv1.GT.MXGTI ) THEN
                  WRITE(*,*) 'GTI: bad value=',ivalue
                  GOTO 490
               END IF
               IF ( igtccd(iv1).LE.0 ) GOTO 490
            END IF
            nccd = nccd + 1
C
            CALL FTGKYJ(Lun1, 'NAXIS2', ntims, ctmp, istat)
            DO j=1, ntims
               dgtis = 0.0D0
               dgtie = 0.0D0
               istat = 0
             CALL FTGCVD(Lun1,igscol(ifile),j,1,1,dnul,dgtis,qbad,istat)
             CALL FTGCVD(Lun1,igecol(ifile),j,1,1,dnul,dgtie,qbad,istat)
               IF ( iutim.NE.0 ) THEN
                  dgtis = MAX(Dgtis,duts)
                  dgtie = MIN(Dgtie,dute)
               END IF
               IF ( dgtie.GT.dgtis ) THEN
                  Tinteg = Tinteg + (dgtie - dgtis)
               END IF
            END DO
  490    CONTINUE
         CALL FTMAHD(Lun1, ihsav, iext, istat)
         CALL FTGKYD(Lun1, 'DTCOR', dtcor, ctmp, istat)
         IF ( nccd.GT.1 ) Tinteg = Tinteg/nccd
         IF ( istat.EQ.0 ) THEN
            WRITE(*,*) 'Dead time correction :',dtcor
            Tinteg = Tinteg*dtcor
         END IF
      END IF
      RETURN
C*********
      ENTRY GTIVAL(Lun1, Dtime1, Iccd, Igood)
C---
C Checks to see if Dtime1 lies within a GTI.  Dtime1 is the time
C as read from the file.
C---
C Lun1    I    The LUN of the open FITS file
C Dtime1  I    The time being considered
C Iccd    I    The indexing value
C Igood     O  =0 outside the user time, =1 no user time or in limits.
C---
      ifile=0
      DO i=1,nfile
         IF ( lunsav(i).EQ.Lun1 ) THEN
            ifile=i
         END IF
      END DO
C
      iv1 = Iccd
      IF ( iv1.LT.1 .OR. MXGTI.LT.iv1 ) iv1 = 1
C The most common occurance is for the event to be in a valid GTI, so
C we do a quick check/return if this is the case.
      Igood = 1
C If no GTI then all data must be good
      IF ( ntot(ifile).LT.0 ) THEN
         IF ( iutim.EQ.0 ) RETURN
         IF ( dtbeg(ifile).LE.Dtime1 .AND.
     &        Dtime1.LE.dtend(ifile) ) RETURN
         Igood = 0
         RETURN
      END IF
C If time is in the previously located GTI then time is valid and
C we do a quick exit.
      IF( dstart(iv1,ifile).LE.Dtime1.AND.
     &     Dtime1.LE.dstop(iv1,ifile) ) RETURN
C
C Not in a valid range, try to find a new range that covers the time
C in question.
      Igood = 0
  500 CONTINUE
      IF ( Dtime1.LT.dstart(iv1,ifile) ) THEN
C If we are before the first GTI, then time is not valid
         IF ( irow(iv1,ifile).LE.1 ) RETURN
C If we are greater than the previous stop time, then we are in a gap.
         IF ( Dtime1.GT.dpstop(iv1,ifile) ) RETURN
C Read the previous range in the GTI extension
         CALL FTGHDN(Lun1, ihsav)
         istat = 0
         CALL FTMAHD(Lun1, ihdu(iv1,ifile), iext, istat)
         irow(iv1,ifile) = irow(iv1,ifile) - 1
         CALL FTGCVD(Lun1,igscol(ifile),irow(iv1,ifile),1,1,
     &      dnul,dtmp,qbad,istat)
         dstart(iv1,ifile) = dtmp
         CALL FTGCVD(Lun1,igecol(ifile),irow(iv1,ifile),1,1,
     &      dnul,dtmp,qbad,istat)
         dstop(iv1,ifile)  = dtmp
         CALL FTMAHD(Lun1, ihsav, iext, istat)
C If we are after the new stop time, then we must be in a gap
         IF ( dstop(iv1,ifile).LT.Dtime1 ) RETURN
         GOTO 500
      END IF
  550 CONTINUE
      IF ( dstop(iv1,ifile).LT.Dtime1 ) THEN
C If we are after the last GTI, then time is not valid
         IF ( irow(iv1,ifile).GE.nrow(iv1,ifile) ) RETURN
C Record current stop time, to prevent unnecessary jumping back and forth
C as we cross a GTI gap.
         dpstop(iv1,ifile) = dstop(iv1,ifile)
         CALL FTGHDN(Lun1, ihsav)
         istat = 0
         CALL FTMAHD(Lun1, ihdu(iv1,ifile), iext, istat)
         irow(iv1,ifile) = irow(iv1,ifile) + 1
         CALL FTGCVD(Lun1,igscol(ifile),irow(iv1,ifile),1,1,
     &      dnul,dtmp,qbad,istat)
         dstart(iv1,ifile) = dtmp
         CALL FTGCVD(Lun1,igecol(ifile),irow(iv1,ifile),1,1,
     &      dnul,dtmp,qbad,istat)
         dstop(iv1,ifile)  = dtmp
         CALL FTMAHD(Lun1, ihsav, iext, istat)
C If we are before the new start time, then we must be in a gap
         IF ( Dtime1.LT.dstart(iv1,ifile) ) RETURN
         GOTO 550
      END IF
      IF( dstart(iv1,ifile).LE.Dtime1.AND.
     &    Dtime1.LE.dstop(iv1,ifile) ) THEN
         Igood = 1
      ELSE
         Igood = 0
      END IF

      RETURN

      END
