      PROGRAM LEXTRCT
C ****************************************************************************
C *                                                                          *
C *                              LEXTRCT                                     *
C *                                                                          *
C *              A McGlynn/Arnaud/Johnstone/Tennant production               *
C *                              MCMLXXXIV                                   *
C ****************************************************************************
C
C This program was designed for analysis of EXOSAT CMA images.  It is
C slowly being turned into a general 2D X-ray analysis tool.
C
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INTEGER   MXCMD, MXHEAD
      PARAMETER (MXCMD=80, MXHEAD=200)
C Machine min and max
      REAL      SYSMIN, SYSMAX
      PARAMETER (SYSMIN=-1.E32,SYSMAX=1.E32)
      REAL      FPNUM
      INTEGER   ISNUM, LENACT
C Valid code? (function)
      LOGICAL   QVALID
C
      INCLUDE 'mapdef.inc'
      INCLUDE 'posdef.inc'
      INCLUDE 'scrat.inc'
      INTEGER*2 i2scrat(MXSR*MXSC)
      EQUIVALENCE (Rscrat, i2scrat)
C
C Version number
      CHARACTER cvers*20
C PLT command array
      CHARACTER cmd(MXCMD)*80
      INTEGER   ncmd
C Contour levels
      CHARACTER clevs*132
      INTEGER   llevs
C Input file logical unit number
      INTEGER   lun
C GTBUF strings
      CHARACTER cbuf*132
      CHARACTER cprom*72
      INTEGER   lprom
C GTBUF peek string
      CHARACTER cpeek*8
      INTEGER   lpeek
C Parameters of action code
      CHARACTER*72 ctok
C Used to get help for user.
      CHARACTER chlib*256, ctopic*64
C Temporary save area
      CHARACTER ctmp*256
      INTEGER   ltmp
C Plotting device type
      CHARACTER cimdev*256
      INTEGER   limdev
C Header records for FITS file
      CHARACTER chead(MXHEAD)*80
      INTEGER   nhead
C
C Needed for RA/DEC conversion of cursor command.
      DOUBLE PRECISION ddec, dra
      REAL      rdecs, rras
      INTEGER   idecm, irah, iram
C Select time intervals
      DOUBLE PRECISION duts, dute
C
C Image sum-signal median.
C Min/Max of the data
      REAL      rdmin, rdmax
      REAL      rbmin, rbmax
C Min/Max used for scaling
      REAL      smin, smax
      REAL      rtmp, tmp
C Offset and scale factor for Image Scale command
      INTEGER   ixmin, ixmax, iymin, iymax
      REAL      scoff, scfac
C
      REAL      xdeth, xdetl, ydeth, ydetl
      REAL      xmap0, ymap0, xtmp, ytmp
      INTEGER   imx, imy, ixm0, ixmn, iym0, iymn, ntimes
C Data size for FITS images written out.
      INTEGER   isize, i, j
C
      CHARACTER cdisp*256
      INTEGER   idisp, ldisp
C
C Number of options
      INTEGER   ier, ihh, ihl, iscomp, inew
      INTEGER   itmp, itype
      INTEGER   lbuf
      INTEGER   ltok, nmapi, nmapj
C Offset
      INTEGER   ixoff, iyoff
C Size of cross correlation region
      INTEGER   nccf
C
C Extension header number of events data.  =0 for no EVENTS data.
      INTEGER   ievent
C A map has been defined?
      LOGICAL   qmap
C Bounds have been defined?
      LOGICAL   qbound
C Is valid plot device open?
      LOGICAL   qplot
C Some process failed
      LOGICAL   qfail
C Has plot device just been opened?
      LOGICAL   qnew
C
      INTEGER   NCODE
      PARAMETER (NCODE=42)
C
      CHARACTER*24 cmess(NCODE)
      CHARACTER*20 codes(NCODE)
      DATA codes/'?', 'AP','BA', 'BO', 'BP', 'CO', 'CPD', 'CR',
     &     'CU', 'DA', 'DB', 'DS', 'ER', 'EX', 'HE', 'IH',
     &     'IN','IP', 'IS', 'MB', 'OG', 'OS', 'P', 'Q', 'RC', 'RE',
     &     'RP', 'SE','SF', 'SH', 'SM', 'ST', 'TM', 'U', 'V',
     &     'WF', 'WI', 'XC', 'XG', 'XH', 'XT', 'Z2'/
C
      DATA cmess/'print this list,        ',
     &     'Angular Profile,        ', 'BAckground,             ',
     &     'set BOunds for analysis,', 'BoxProject,             ',
     &     'COntour plot,           ', 'Change Plot Device,     ',
     &     'CRoss correlate,        ', 'CUrsor,                 ',
     &     'read DAta file,         ', 'Display Bounds,         ',
     &     'Display Surface,        ', 'ERase graphs,           ',
     &     'EXit,                   ',
     &     'ask for HElp,           ', 'Image Histogram,        ',
     &     'INfo,                   ', 'Image Projection,       ',
     &     'Image Scale,            ', 'Mark Bad,               ',
     &     'Overlay Grid,           ', 'Overplot Sources,       ',
     &     'Plot image,             ', 'Quit,                   ',
     &     'Row/Col projection,     ', 'REsponse,               ',
     &     'Radial Profile,         ', 'SEt parameter value,    ',
     &     'Source Finding,         ', 'SHow parameter value    ',
     &     'SMooth map,             ', 'compute STatistics,     ',
     &     'Type Map,               ', 'Use map                 ',
     &     'show Version number,    ', 'Write FITS image,       ',
     &     'Write Integer FITS,     ', 'Xtract Counts (in box), ',
     &     'Xtract Grating spectra, ', 'Xtract Spectrum,        ',
     &     'Xtract Timing file,     ', 'Z**2 test.              '/

      DATA cvers/'2020-07-06'/
      DATA cimdev/'/null'/
      DATA ixoff,iyoff/0,0/,limdev/5/
      DATA scoff,scfac/0.0,1.0/
C
      chlib = 'lextrct'
      CALL PTEND('$XANTOP','doc',chlib)
C
C Get default PGPLOT plot device
      CALL TRLOG('PGPLOT_TYPE',11,ctok,ltok)
      qfail = .TRUE.
      IF ( ltok.GT.0 ) THEN
         cimdev = ctok
         limdev = ltok
C Make sure device is valid
         CALL PDIMAG(cimdev, qfail)
      END IF
      IF ( qfail ) THEN
         CALL PTBUF(' Failed to open plot device.  Use cpd to set.', -1)
      ELSE
         CALL PTBUF('Current plot device is '//cimdev(:limdev), -1)
      END IF
      CALL PTBUF(' ', 1)
C
C Input LUN
      lun = 1
C
C Initialize variables
C
      ievent = 0
      Mxrow = MXMAP
      Mxcol = MXMAP
      qmap = .FALSE.
      qbound = .FALSE.
C World Coordinate System variables
      Iwcs = 0
      Dxpix0 = NO
      Dypix0 = NO
      Dxref = NO
      Dyref = NO
      Dxdel = NO
      Dydel = NO
      Drota = NO
C Default to displaying the data array.
      idisp = 1
C No data
      Ldata = 0
C No background file read, no subtracted image.
      Lback = 0
      Ibsub = 0
C No response file, no flat field corrected image.
      Lresp = 0
      Iflat = 0
C
C Print initial list of options
C
      CALL PRCODE(codes,cmess,NCODE,qmap,qbound,ievent,Iwcs)
C
 100  CONTINUE
C
C Prompt for action code and check to see whether it can be obeyed
C
      cbuf = ' '
      CALL GTBUF('LEXTRCT>',ier)
      IF ( ier.GT.0 ) GOTO 100
      IF ( ier.LT.0 ) THEN
         cbuf(1:1) = 'Q'
         lbuf = 1
      ELSE
         CALL GTCHAR(cbuf,lbuf)
         CALL UPC(cbuf)
      END IF
 120  CONTINUE
      IF ( .NOT.QVALID(cbuf,qmap,qbound,ievent,Iwcs) ) THEN
         CALL PTBUF('   Invalid code.', -1)
         GOTO 100
      END IF
C
C '?' print out a list of possible commands
C
      IF ( cbuf(1:1).EQ.'?' ) THEN
         CALL PRCODE(codes,cmess,NCODE,qmap,qbound,ievent,Iwcs)
C
C 'AP' compute angular profile.
C
      ELSE IF ( cbuf(1:2).EQ.'AP' ) THEN
         CALL APROF(Rmap, Tdata, Back, Tback, Mxrow, Nrow, Ncol,
     &            cmd, MXCMD)
C
C 'BA' Background
C
      ELSE IF ( cbuf(1:2).EQ.'BA' ) THEN
         CALL RDMAP(ievent, Back, Mxrow, Mxcol, Nrow, Ncol,
     &         Cback, Lback, Tback, ier)
         IF ( ier.EQ.0 ) THEN
C Invalidate the background subtracted map.
            Ibsub = 0
C
            WRITE(cbuf,121) Nrow,Ncol
  121       FORMAT('        Map dimensions :',I6, I6)
            lbuf = LENACT(cbuf)
            CALL PTBUF(cbuf,lbuf)
            CALL ISCALE(Back, Mxrow, Nrow, Ncol, 0, 0.0, 0.0,
     &         rbmin, rbmax, imx, imy)
            CALL POS_MAP_DET(imx, imy, xtmp, ytmp)
            WRITE(cbuf,131) rbmin
  131       FORMAT('    Background minimum :',F11.1)
            lbuf = LENACT(cbuf)
            CALL PTBUF(cbuf,lbuf)
            WRITE(cbuf,141) rbmax, xtmp, ytmp
  141       FORMAT('    Background maximum :',F11.1,
     &             ' at (',F8.1,F8.1,')')
            lbuf = LENACT(cbuf)
            CALL PTBUF(cbuf,lbuf)
         END IF
C
C 'BP' Box project (in rotated box coordinates).
C
      ELSE IF ( cbuf(1:2).EQ.'BP' ) THEN
         CALL PICKAR(idisp, cdisp, ldisp, ier)
         CALL PDSEL('LI',qnew,qplot)
         ncmd = 1
         cmd(ncmd)='LAB F '//cdisp(:ldisp)
         CALL RCPROJ(Rdat(1,idisp),Mxrow,Nrow,Ncol,1,qplot,
     &      MXCMD,ncmd,cmd)
         qnew = .TRUE.
C
C 'B' ask user to define an analysis bounds.
C
      ELSE IF ( cbuf(1:1).EQ.'B' ) THEN
         CALL PDSEL('IM',qnew,qplot)
         CALL PLSCAL(qnew)
         CALL BOUDEF(qplot,qbound,qfail)
         IF ( qfail ) GOTO 100
         IF ( .NOT.qbound ) GOTO 100
         CALL BOUSTR(cbuf)
         CALL PTBUF(' '//cbuf(:LENACT(cbuf)), -1)
C
C 'CO' compute contour map of current box prompting user for contour levels
C
      ELSE IF ( cbuf(1:2).EQ.'CO' ) THEN
         CALL PDSEL('IM',qnew,qplot)
         IF ( .NOT.qplot ) GOTO 100
         CALL GTBUF('Enter contour levels:', ier)
         IF ( ier.ne.0 ) GOTO 100
         CALL GTREST(clevs, llevs)
         ncmd = 0
         cmd(ncmd+1) = 'COL OFF 1..999'
         cmd(ncmd+2) = 'CON LEV '//clevs
         ncmd = ncmd + 3
         WRITE(cmd(ncmd), 211) smin,smax
         CALL PIMAGE(Rmap,Mxrow,Nrow,Ncol,cmd,MXCMD,ncmd)
         qnew = .FALSE.
C
C 'CP' change plotting device for image, set and initialize the plot device.
C
      ELSE IF ( cbuf(1:2).EQ.'CP' ) THEN
C
C Get the new device and verify it.
C
         CALL GTPEEK(cpeek,lpeek)
         IF ( cpeek.EQ.' ' ) THEN
            cprom = 'Image device (' // cimdev(1:LENACT(cimdev)) // '):'
            CALL GTBUF(cprom,ier)
            IF ( ier.NE.0 ) GOTO 100
         END IF
         CALL GTCHAR(ctmp,ltmp)
         IF ( ltmp.EQ.0 ) GOTO 100
         cimdev = ctmp
         limdev = ltmp
C
C See if user entered a valid device name.
C
         CALL PDIMAG(cimdev, qfail)
         IF ( qfail ) THEN
            CALL PTBUF(' Failed to open image device.', -1)
            GOTO 100
         END IF
         qnew = .TRUE.
C
C 'CR' CRoss correlate source and background image
C
      ELSE IF ( cbuf(1:2).EQ.'CR' ) THEN
         cmd(1)='LAB f '//cdata(:ldata)//'**'//cback(:lback)
         ncmd=1
         nccf=10
         CALL CROSSC(Mxrow,Nrow,Ncol,Rmap,Back,nccf,Resp,
     &    MXCMD, cmd, ncmd)
         idisp=3
C
C 'CU' read CUrsor position
C
      ELSE IF ( cbuf(1:2).EQ.'CU' ) THEN
         CALL ASKPOS(qplot,qfail)
         WRITE(cbuf,151) Xdet,Ydet
  151    FORMAT(' Det =',F9.2, F9.2)
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
         CALL POS_DT_RM(Xdet, Ydet, xmap0, ymap0)
         WRITE(cbuf, 161) xmap0, ymap0
  161    FORMAT(' Map =', F7.0, 1X, F7.0)
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
         IF ( Iwcs.NE.0 ) THEN
C If transformation matrix is defined then print out RA and DEC
            CALL POS_DT_RA(Xdet, Ydet, dra, ddec)
            CALL POS_SKY(itmp)
            IF ( itmp.NE.0 ) THEN
C Sky coordinates
               CALL POS_DEG_HMS(dra,ddec,irah,iram,rras,
     &               ctmp,idecm,rdecs)
               WRITE(cbuf,171) irah, iram,rras, ctmp(:3), idecm, rdecs
 171           FORMAT(' ',I4,I3,1X,F6.3,', ',A3,I3,1X,F5.2,'  (')
               lbuf = LENACT(cbuf)
               CALL CRAMDF(dra,10, 5, cbuf, lbuf)
               cbuf(lbuf+1:lbuf+1) = ','
               lbuf = lbuf + 2
               CALL CRAMDF(ddec, 9, 5, cbuf, lbuf)
               cbuf(lbuf+1:lbuf+1) = ')'
               lbuf = lbuf + 1
            ELSE
               cbuf = ' Corresponding to: '
               lbuf = LENACT(cbuf)
               CALL CRAMDF(dra, 10, -4, cbuf, lbuf)
               cbuf(lbuf+1:lbuf+1) = ','
               lbuf = lbuf + 2
               CALL CRAMDF(ddec, 9, -4, cbuf, lbuf)
            END IF
            CALL PTBUF(cbuf,lbuf)
         END IF
C
C 'DA' read DAta (FITS) file.
C
      ELSE IF ( cbuf(1:2).EQ.'DA' ) THEN
         CALL LEXOPN(lun, 1, Cdata, Ldata, inew, ier)
         IF ( ier.NE.0 ) THEN
            WRITE(cbuf, 181) ier
  181       FORMAT(' Unable to open file.  Error=',I5)
            lbuf = LENACT(cbuf)
            CALL PTBUF(cbuf,lbuf)
            GOTO 100
         END IF
C
C Initilize various things
         CALL LIMSB(0)
         CALL LIMSS(0, 0, 0, 1)
         CALL DIXLBL('X (raw)')
         CALL DIYLBL('Y (raw)')
         CALL DITLBL(' ')
C
C Read Fits header
         CALL RFHEAD(lun, ievent, Nrow, Ncol, ier)
         IF ( ier.NE.0 ) GOTO 100
C
C Not reading same file, then reset offset.
         IF ( inew.NE.0 ) THEN
            IF ( ievent.GT.0 ) THEN
               CALL RFCQPR(ixmin, ixmax, iymin, iymax)
               ixoff = ixmin - 1
               iyoff = iymin - 1
            ELSE
               ixoff = 0
               iyoff = 0
            END IF
         END IF
C
         CALL DIFLBL(Cdata)
C
         IF ( inew.NE.0 .OR. Ifac.EQ.0 ) THEN
C Get offset and scaling.
            itmp=MAX(1,NINT(SQRT(Nrow*Ncol/(REAL(MXROW)*REAL(MXCOL)))))
C Make sure we don't overflow the Mxrow/Mxcol limits.  Since integer
C divide truncates, we add one to effectively round up.
            IF ( (Nrow-ixoff)/itmp.GT.Mxrow ) itmp=1+(Nrow-ixoff)/Mxrow
            IF ( (Ncol-iyoff)/itmp.GT.Mxcol ) itmp=1+(Ncol-iyoff)/Mxcol
         ELSE
            itmp = Ifac
         END IF
 190     CONTINUE
         WRITE(ctmp, 191) ixoff, iyoff, itmp
 191     FORMAT('Map offset and compression (',2I7,I5,'): ')
         CALL GTBUF(ctmp,ier)
         IF ( ier.NE.0 )  GOTO 100
         CALL GTINT(ixoff,ier)
         CALL GTINT(iyoff,ier)
         CALL GTINT(itmp,ier)
         Ifac = MAX(1,itmp)
C Set up transformation given the offset and compression.
         xdmap1 = ixoff + ifac/2. + 0.5
         ydmap1 = iyoff + ifac/2. + 0.5
C
         IF ( ievent.GT.0 ) THEN
C---
 220        CONTINUE
            duts = 0.0D0
            dute = 0.0D0
            WRITE(cprom,221) duts, dute
 221        FORMAT('Start, Stop time (',F10.0,',',F10.0,'):')
            CALL GTBUF(cprom,ier)
            IF ( ier.LT.0 ) GOTO 190
            CALL GTDBLE(duts, ier)
            CALL GTDBLE(dute, ier)
            IF ( duts.NE.0.0 .AND. dute.NE.0.0 ) THEN
               CALl GTISUT(duts, dute)
            END IF
C---
            CALL LIMQS(itmp, ihl, ihh, iscomp)
            WRITE(cprom,351) ihl, ihh
 351        FORMAT('Histogram limits (',I4,',',I4,'):')
            CALL GTBUF(cprom,ier)
            IF ( ier.NE.0 ) GOTO 220
            CALL GTINT(ihl,ier)
            CALL GTINT(ihh,ier)
            CALL GTINT(iscomp,ier)
C Load the compression
            CALL LIMSS(0, 0, 0, iscomp)
            IF ( ihl.GT.0 .AND. ihh.GT.0 ) THEN
               CALL LIMSS(1, ihl, ihh, iscomp)
            END IF
         END IF
C
         CALL RFDATA(lun, ievent, Tdata,
     &      .TRUE.,Rmap,Mxrow,Mxcol,Nrow,Ncol,
     &      0,rtmp,1,itmp, .FALSE.,rtmp,1,itmp)
C Invalidate the background subtracted map.
         Ibsub = 0
         qmap = .TRUE.
         WRITE(cbuf, 121) Nrow,Ncol
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
         CALL ISCALE(Rmap, Mxrow, Nrow, Ncol, 0, 0.0, 0.0,
     &      rdmin, rdmax, imx, imy)
         CALL POS_MAP_DET(imx, imy, Xdet, Ydet)
         WRITE(cbuf,361) rdmin
  361    FORMAT('      Map data minimum :',F11.1)
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
         WRITE(cbuf,371) rdmax, Xdet, Ydet
  371    FORMAT('      Map data maximum :',F11.1,' at (',F8.1,F8.1,')')
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
C Reset the default region to be the total image.
         cbuf = 'T N'
         CALL LDBUF1(cbuf, ier)
         IF ( ier.NE.0 ) THEN
            WRITE(*,*) 'LEXTRCT--LDBUF1 returned an error of', ier
         END IF
         CALL GTBUF(' ', ier)
C Set first arg to be .FALSE. to avoid spurious drawing of region.
         CALL BOUDEF(.FALSE., qbound, qfail)
C
         smin = rdmin
         smax = rdmax
         Ibsub = 0
         GOTO 100
C
C 'DB' Display the current bounds on the graphics device.
C
      ELSE IF ( cbuf(1:2).EQ.'DB' ) THEN
         CALL BOUSTR(cbuf)
         CALL PTBUF(cbuf(:LENACT(cbuf)), -1)
         CALL PDSEL('IM',qnew,qplot)
         CALL PLSCAL(qnew)
         IF ( qplot ) CALL BOUDIS(1)
C
C 'DS' Display Surface
C
      ELSE IF ( cbuf(1:2).EQ.'DS' ) THEN
         CALL GTPEEK(cpeek,lpeek)
 250     CONTINUE
         IF ( lpeek.EQ.0 ) THEN
            WRITE(cprom,251) smin, smax
 251        FORMAT(' Display min and max (',1PG11.3,',',G11.3,'):')
            CALL GTBUF(cprom,ier)
            IF ( ier.NE.0 ) GOTO 100
         END IF
         CALL GTREAL(smin,ier)
         CALL GTREAL(smax,ier)
         IF ( ABS(smax-smin).LT.1.0E-34 ) THEN
            CALL PTBUF(' **Error** MIN=MAX', -1)
            lpeek = 0
            GOTO 250
         END IF
         CALL PLOTS(-1.0,1.0,-1.0,1.0)
         CALL PLTSE(Rmap, Mxrow, Mxcol, 1, Nrow, 1, Ncol, smin, smax)
C
C 'EX' or 'Q' exit the program
C
      ELSE IF ( cbuf(1:2).EQ.'EX' .OR. cbuf(1:1).EQ.'Q' ) THEN
C
C If plot device open, close it and exit.
C
         IF ( qplot ) CALL PGEND
         GOTO 999
C
C 'E' erase the current plot on the image device
C
      ELSE IF ( cbuf(1:1).EQ.'E' ) THEN
         CALL PDSEL('IM',qnew,qplot)
         IF ( .NOT.qplot ) GOTO 100
         CALL PGADVANCE
         qnew = .TRUE.
C
C 'HE' enter EXOSAT help library
C
      ELSE IF ( cbuf(1:2).EQ.'HE' ) THEN
         CALL GTCHAR(ctopic, ltmp)
C         CALL GTHELP(chlib, ctopic)
         CALL GETLUN(itmp)
         CALL GTXHLP(itmp,chlib, ctopic)
         CALL FRELUN(itmp)
C
C 'IH' construct and display the Image Histogram
C
      ELSE IF ( cbuf(1:2).EQ.'IH' ) THEN
         CALL PICKAR(idisp, cdisp, ldisp, ier)
         CALL PDSEL('LI',qnew,qplot)
         ncmd = 1
         cmd(ncmd)='LAB F '//cdisp(:ldisp)
         CALL IHISTO(Rdat(1,idisp),Mxrow,Nrow,Ncol,smin,smax,
     &      cmd,MXCMD,ncmd)
C
C 'IN' Print Information about data that has been read in.
C
      ELSE IF ( cbuf(1:2).EQ.'IN' ) THEN
         IF ( Ldata.EQ.0 ) THEN
            CALL PTBUF(' Name of data file: (none)', -1)
         ELSE
            CALL PTBUF(' Name of data file: '//Cdata(:Ldata), -1)
         END IF
         IF ( Lback.EQ.0 ) THEN
            CALL PTBUF(' Name of back file: (none)', -1)
         ELSE
            CALL PTBUF(' Name of back file: '//Cback(:Lback), -1)
         END IF
         IF ( Lresp.EQ.0 ) THEN
            CALL PTBUF(' Name of resp file: (none)', -1)
         ELSE
            CALL PTBUF(' Name of resp file: '//Cresp(:Lresp), -1)
         END IF
C
C 'IP' project image onto sides of current box
C
      ELSE IF ( cbuf(1:2).EQ.'IP' ) THEN
         CALL PDSEL('IM',qnew,qplot)
         CALL PLSCAL(qnew)
         CALL IPROJ(Rdat(1,idisp),Mxrow,Nrow,Ncol,qplot)
C
C 'IS' Image Scale
C
      ELSE IF ( cbuf(1:2).EQ.'IS' ) THEN
         itype = 1
  280    CONTINUE
         CALL GTPEEK(cpeek,lpeek)
         IF ( lpeek.GT.0 ) THEN
            CALL UPC(cpeek)
            IF ( cpeek(1:2).EQ.'LI' ) THEN
               itype = 1
               CALL GTCHAR(ctmp, ltmp)
               GOTO 280
            ELSE IF ( cpeek(1:2).EQ.'LO' ) THEN
               itype = 3
               CALL GTCHAR(ctmp, ltmp)
               GOTO 280
            ELSE IF ( cpeek(1:1).EQ.'R' ) THEN
               itype = 4
               CALL GTCHAR(ctmp, ltmp)
               GOTO 280
            ELSE IF ( cpeek(1:1).EQ.'S' ) THEN
               itype = 2
               CALL GTCHAR(ctmp, ltmp)
               GOTO 280
            END IF
         END IF
         IF ( lpeek.EQ.0 ) THEN
            WRITE(cprom,291) scoff, scfac
  291       FORMAT(' Map offset (',1PG11.3,'), scale (',G11.3,'):')
            CALL GTBUF(cprom,ier)
            IF ( ier.NE.0 ) GOTO 100
         END IF
         CALL GTPEEK(cpeek,lpeek)
         IF ( lpeek.GT.0 ) THEN
            CALL UPC(cpeek)
            IF ( cpeek(1:2).EQ.'MI' ) THEN
C IScale MIn:  IScale uses unbinned rate, hence we divide by Ifac**2
               IF ( idisp.EQ.2 ) THEN
                  scoff = rbmin/(Ifac*Ifac)
               ELSE
                  scoff = rdmin/(Ifac*Ifac)
               END IF
               scfac = 1.0
               CALL GTCHAR(ctmp,ltmp)
            ELSE IF ( cpeek(1:2).EQ.'MA' ) THEN
               IF ( idisp.EQ.2 ) THEN
                  scoff = rbmax
               ELSE
                  scoff = rdmax
               END IF
               scfac = 1.0
               CALL GTCHAR(ctmp,ltmp)
            ELSE
               CALL GTREAL(scoff,ier)
            END IF
            CALL GTREAL(scfac,ier)
         END IF
         CALL ISCALE(Rdat(1,idisp), Mxrow, Nrow, Ncol, itype,
     &         scoff*Ifac*Ifac, scfac, rdmin, rdmax, imx, imy)
         CALL POS_MAP_DET(imx, imy, Xdet, Ydet)
         IF ( idisp.EQ.2 ) THEN
            WRITE(cbuf,361) rbmin
         ELSE
            WRITE(cbuf,361) rdmin
         END IF
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
         IF ( idisp.EQ.2 ) THEN
            WRITE(cbuf,371) rbmax, Xdet, Ydet
         ELSE
            WRITE(cbuf,371) rdmax, Xdet, Ydet
         END IF
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
         smin = rdmin
         smax = rdmax
C
C 'MB' Mark Bad
C
      ELSE IF ( cbuf(1:2).EQ.'MB' ) THEN
         CALL MBAD(Rdat(1,idisp), Mxrow, Nrow, Ncol, qplot, itmp)
         IF ( itmp.NE.0 ) THEN
            CALL ISCALE(Rdat(1,idisp),Mxrow,Nrow,Ncol, 0, 0.0,0.0,
     &         rdmin, rdmax, imx, imy)
            CALL POS_MAP_DET(imx, imy, Xdet, Ydet)
            WRITE(cbuf,361) rdmin
            lbuf = LENACT(cbuf)
            CALL PTBUF(cbuf,lbuf)
            WRITE(cbuf,371) rdmax, Xdet, Ydet
            lbuf = LENACT(cbuf)
            CALL PTBUF(cbuf,lbuf)
         END IF
C
C 'OG' Overlay an RA and dec grid
C
      ELSE IF ( cbuf(1:2).EQ.'OG' ) THEN
         CALL PDSEL('IM',qnew,qplot)
         IF ( .NOT.qplot ) GOTO 100
         CALL PLSCAL(qnew)
         CALL OGRID
C
C 'OS' Overplot Source list
C
      ELSE IF ( cbuf(1:2).EQ.'OS' ) THEN
         CALL SORPLOT(Qplot)
C
C 'P' Plot Image
C
      ELSE IF ( cbuf(1:1).EQ.'P' .OR. cbuf(1:2).EQ.'DI' ) THEN
         CALL PICKAR(idisp, cdisp, ldisp, ier)
         IF ( ier.NE.0 ) GOTO 100
C
         CALL GTPEEK(cpeek, lpeek)
 200     CONTINUE
         IF ( lpeek.EQ.0 ) THEN
            WRITE(cprom,201) smin, smax
 201        FORMAT(' Display min and max (',1PG11.3,',',G11.3,'):')
            CALL GTBUF(cprom,ier)
            IF ( ier.NE.0 ) GOTO 100
         END IF
         CALL GTREAL(smin,ier)
         CALL GTREAL(smax,ier)
         IF ( ABS(smax-smin).LT.1.0E-34 ) THEN
            CALL PTBUF(' **Error** MIN=MAX', -1)
            lpeek = 0
            GOTO 200
         END IF
C
         ncmd = 0
         cmd(ncmd+1) = 'COL OFF 1..999'
         cmd(ncmd+2) = 'LAB F '//cdisp(:ldisp)
         ncmd = ncmd + 2
C
         CALL PDSEL('IM',qnew,qplot)
         IF ( .NOT.qplot ) GOTO 100
         ncmd = ncmd + 1
         WRITE(cmd(ncmd), 211) smin,smax
 211     FORMAT('IMAGE MIN ',1PG10.3,' MAX ',G10.3)
         CALL PIMAGE(Rdat(1,idisp),Mxrow,Nrow,Ncol,cmd,MXCMD,ncmd)
         qnew = .FALSE.
C
C 'RC' Row/Column projection.
C
      ELSE IF ( cbuf(1:2).EQ.'RC' ) THEN
         CALL PICKAR(idisp, cdisp, ldisp, ier)
         CALL PDSEL('LI',qnew,qplot)
         ncmd = 1
         cmd(ncmd)='LAB F '//cdisp(:ldisp)
         CALL RCPROJ(Rdat(1,idisp),Mxrow,Nrow,Ncol,0,qplot,
     &      MXCMD,ncmd,cmd)
         qnew = .TRUE.
C
C 'RE' REsponse File
C
      ELSE IF ( cbuf(1:2).EQ.'RE' ) THEN
  260    CONTINUE
         CALL RDMAP(ievent, Resp, Mxrow, Mxcol, Nrow, Ncol,
     &         Cresp, Lresp, Tresp, ier)
         IF ( ier.NE.0 ) GOTO 100
         WRITE(cbuf, 121) Nrow,Ncol
         lbuf = LENACT(cbuf)
         CALL PTBUF(cbuf,lbuf)
C
         CALL GTPEEK(ctmp,ltmp)
         IF ( ltmp.LE.0 ) THEN
            cprom = 'Do you wish to apply correction now ('
            lprom = LENACT(cprom)
            IF ( iflat.LE.0 ) THEN
               cprom(lprom+1:lprom+1) = 'N'
            ELSE
               cprom(lprom+1:lprom+1) = 'Y'
            END IF
            cprom(lprom+2:lprom+4) = ')?'
            CALL GTBUF(cprom, ier)
            IF ( ier.NE.0 ) GOTO 260
         END IF
         CALL GTCHAR(ctmp, ltmp)
         CALL UPC(ctmp(1:1))
C
         IF ( ctmp(1:1).EQ.'Y' ) THEN
            CALL FLAT(Rmap, Mxrow, Nrow, Ncol, Resp, Rdmin, Rdmax)
            Iflat = 1
            WRITE(*,*) 'Map data minimum/maximum:',rdmin,rdmax
            smin = rdmin
            smax = rdmax
            idisp = 1
         ELSE
            Iflat = -1
            WRITE(*,*) 'Response minimum/maximum:',rdmin,rdmax
         END IF
C
C 'RP' compute a radial profile.
C
      ELSE IF ( cbuf(1:2).EQ.'RP' .OR. cbuf(1:1).EQ.'A' ) THEN
         CALL RPROF(Rmap, Tdata, Back, Tback, Mxrow, Nrow, Ncol,
     &            cmd, MXCMD)
C
C 'SE' set parameter value
C
      ELSE IF ( cbuf(1:2).EQ.'SE' ) THEN
         nmapi = Mxrow
         nmapj = Mxcol
         CALL SETSHO(1)
         IF ( nmapi.NE.Mxrow .OR. nmapj.NE.Mxcol ) qmap = .FALSE.
C
C 'SF' access source-finding routines
C
      ELSE IF ( cbuf(1:2).EQ.'SF' ) THEN
         CALL GTFWAT(tmp)
         IF ( tmp.LE.0.0 ) THEN
            WRITE(*,*) 'SFIND--Please SET PSF SIGMA to be non-zero '
            GOTO 100
         END IF
         CALL PDSEL('IM',qnew,qplot)
         CALL PLSCAL(qnew)
         ncmd = 0
         cmd(ncmd+1) = 'COL OFF 1..999'
         cmd(ncmd+2) = 'LAB F '//cdata
         ncmd = ncmd + 2
         CALL SFIND(qplot, smin, smax, MXCMD, cmd, ncmd)
C
C 'SH' show parameter value
C
      ELSE IF ( cbuf(1:2).EQ.'SH' ) THEN
         CALL SETSHO(0)
C
C 'SM' do a nearest neighbour triangular smooth on the contents of the current
C box
C
      ELSE IF ( cbuf(1:2).EQ.'SM' ) THEN
C Check for optional argument for number of times to smooth.
         ntimes = 1
         CALL GTPEEK(ctmp, ltmp)
         IF ( ISNUM(ctmp, ltmp).NE.0 ) THEN
            CALL GTCHAR(ctmp, ltmp)
            itmp = FPNUM(ctmp, ltmp, ier)
            IF ( itmp.LE.0 ) GOTO 100
            ntimes = itmp
            WRITE(*,*) 'Doing',ntimes,' smooths.'
         END IF
         CALL BOUQOB(xdetl,ydetl,xdeth,ydeth)
         CALL POS_DT_IM(xdetl,ydetl,ixm0,iym0)
         ixm0 = MIN(MAX(1,ixm0),Nrow)
         iym0 = MIN(MAX(1,iym0),Ncol)
         CALL POS_DT_IM(xdeth,ydeth,ixmn,iymn)
         ixmn = MIN(MAX(1,ixmn),Nrow)
         iymn = MIN(MAX(1,iymn),Ncol)
         nmapi = ixmn - ixm0 + 1
         nmapj = iymn - iym0 + 1
         CALL SMOOTH(Rdat(1,idisp), Mxrow, Nrow, Ncol,
     :      ixm0,iym0,nmapi,nmapj,ntimes)
C
C 'ST' calculate statistics on current box
C
      ELSE IF ( cbuf(1:2).EQ.'ST' ) THEN
         CALL PTBUF(' File      = '//Cdata(:ldata), -1)
         CALL STATS(Rmap,Mxrow,Nrow,Ncol)
         IF ( Lback.GT.0 ) THEN
            CALL PTBUF(' ', 1)
            CALL PTBUF(' Background= '//Cback(:lback), -1)
            CALL STATS(Back,Mxrow,Nrow,Ncol)
         END IF
         IF ( Lresp.GT.0 ) THEN
            CALL PTBUF(' ', 1)
            CALL PTBUF(' Response  = '//Cresp(:lresp), -1)
            CALL STATS(Resp,Mxrow,Nrow,Ncol)
         END IF
C
C 'TM' Type map
C
      ELSE IF ( cbuf(1:2).EQ.'TM' ) THEN
         CALL PICKAR(idisp, cdisp, ldisp, ier)
         CALL TMAP(Rdat(1,idisp),Mxrow,Nrow,Ncol,cdisp,ldisp,qplot)
      ELSE IF ( cbuf(1:1).EQ.'U' ) THEN
C Use array
         CALL PICKAR(idisp, cdisp, ldisp, ier)
         WRITE(*,*) 'idisp=',idisp
C
C 'V' type current version number
C
      ELSE IF ( cbuf(1:1).EQ.'V' ) THEN
         CALL TTWRT(' LEXTRCT ver.: '//cvers,-1)
         CALL FTVERS(rtmp)
         WRITE(ctmp,311) rtmp
  311    FORMAT(' FITSIO  ver.: ',F5.3)
         CALL TTWRT(ctmp, -1)
         CALL PLTVER(ctok, ltok)
         ctmp = ' PLT     ver.: '//ctok(:ltok)
         CALL TTWRT(ctmp, -1)
         CALL PGQINF('VERSION',ctok,ltok)
         ctmp = ' PGPLOT  ver.: '//ctok(:ltok)
         CALL TTWRT(ctmp, -1)
C
C 'WF' write the present map to a FITS file.
C
      ELSE IF ( cbuf(1:2).EQ.'WF' .OR. cbuf(1:2).EQ.'WI' ) THEN
         IF ( cbuf(1:2).EQ.'WI' ) THEN
C Write integer*2 data
            isize = 2
            DO i=1, ncol
               DO j=1, nrow
                  itmp = j + (i-1)*MXROW
                  i2scrat(itmp) = Rmap(itmp)
               END DO
            END DO
         ELSE
            isize = 4
         END IF
         chead(1) = 'VLEXTRCT'
         chead(1)(9:) = '= '''//cvers
         ltmp = 1+LENACT(chead(1))
         chead(1)(ltmp:ltmp) = ''''
         chead(1)(32:) = '/ Version of LEXTRCT'
         nhead = 1
C
         IF ( idisp.EQ.1 ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 781) Tdata
  781       FORMAT('LIVETIME=',7X,1PE14.7,' / [s] Live time')
         ELSE IF ( idisp.EQ.2 ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 781) Tback
         ELSE IF ( idisp.EQ.3 ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 781) TResp
         ELSE
C not defined.
         END IF
C
         IF ( ctype1.NE.' ' ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 801) 1,'RA--'//ctype1
  801       FORMAT('CTYPE',I1,'  = ''',A8,'''',10X,' / Projection')
            nhead = nhead + 1
            IF ( ctype2.NE.' ' ) THEN
               WRITE(chead(nhead), 801) 2,'DEC-'//ctype2
            ELSE
               WRITE(chead(nhead), 801) 2,'DEC-'//ctype1
            END IF
         END IF
C
         IF ( imat.EQ.0 ) THEN
            IF ( Drota.NE.NO ) THEN
               nhead = nhead + 1
               WRITE(chead(nhead), 806) 1,Drota
  806          FORMAT('CROTA',I1,'  = ',6X,1PE14.7,' / Rotation')
            END IF
            IF ( Dxdel.NE.NO ) THEN
               nhead = nhead + 1
               WRITE(chead(nhead), 831) 1,Dxdel*Ifac
  831          FORMAT('CDELT',I1,'  = ',6X,1PE14.7,' / Pixel size')
               nhead = nhead + 1
               chead(nhead)='CUNIT1  = ''deg''                /'
            END IF
            IF ( Dydel.NE.NO ) THEN
               nhead = nhead + 1
               WRITE(chead(nhead), 831) 2,Dydel*Ifac
               nhead = nhead + 1
               chead(nhead)='CUNIT2  = ''deg''                /'
            END IF
         ELSE
            IF ( Dcd11.NE.NO ) THEN
               nhead = nhead + 1
               WRITE(chead(nhead), 841) 1,1,Dcd11*Ifac
  841          FORMAT('CD',I1,'_',I1,'   = ',6X,1PE14.7,' /')
            END IF
            IF ( Dcd12.NE.NO ) THEN
               nhead = nhead + 1
               WRITE(chead(nhead), 841) 1,2,Dcd12*Ifac
            END IF
            IF ( Dcd21.NE.NO ) THEN
               nhead = nhead + 1
               WRITE(chead(nhead), 841) 2,1,Dcd21*Ifac
            END IF
            IF ( Dcd22.NE.NO ) THEN
               nhead = nhead + 1
               WRITE(chead(nhead), 841) 2,2,Dcd22*Ifac
            END IF
         END IF
C
         IF ( Dxref.NE.NO ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 811) 1,Dxref
  811       FORMAT('CRVAL',I1,'  = ',6X,1PE14.7,' / Reference value')
         END IF
         IF ( Dyref.NE.NO ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 811) 2,Dyref
         END IF
C
         CALL POS_DT_RM(SNGL(Dxpix0), SNGL(Dypix0), xmap0, ymap0)
         IF ( Dxpix0.NE.NO ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 821) 1,xmap0
  821       FORMAT('CRPIX',I1,'  = ',6X,1PE14.7,' / Reference pixel')
         END IF
         IF ( Dypix0.NE.NO ) THEN
            nhead = nhead + 1
            WRITE(chead(nhead), 821) 2,ymap0
         END IF
C
         CALL cphead(lun, ievent, MXHEAD, nhead, chead)
         IF ( isize.EQ.2 ) THEN
            CALL WI2MAP(i2scrat,Mxrow,Nrow,Ncol, chead,nhead, ier)
         ELSE
            CALL WR4MAP(rdat(1,idisp),Mxrow,Nrow,Ncol,chead,nhead,ier)
         END IF
C
C XC Xtract counts (in box)
C
      ELSE IF ( cbuf(1:2).EQ.'XC' ) THEN
         CALL XCNT(lun, ievent, Tdata)
C
C XG Xtract Grating
C
      ELSE IF ( cbuf(1:2).EQ.'XG' ) THEN
         CALL PDSEL('LI',qnew,qplot)
         ncmd = 1
         cmd(ncmd)='LAB F '//cdata
         CALL XGRATE(lun, ievent, qplot, mxcmd, ncmd, cmd)
         qnew = .TRUE.
C
C XH Xtract spectral histogram
C
      ELSE IF ( cbuf(1:2).EQ.'XH' ) THEN
         CALL PDSEL('LI',qnew,qplot)
         ncmd = 0
         CALL XHISTO(lun,ievent,cmd,MXCMD,ncmd)
         qnew = .TRUE.
C
C XT Xtract Timing
C
      ELSE IF ( cbuf(1:2).EQ.'XT' ) THEN
         CALL PDSEL('LI',qnew,qplot)
         ncmd = 1
         cmd(ncmd)='LAB F '//cdata
         CALL XTMR(lun,ievent,cmd,MXCMD,ncmd)
         qnew = .TRUE.
C
C Z2 Zsquared test
C
      ELSE IF ( cbuf(1:2).EQ.'Z2' ) THEN
         CALL PDSEL('LI',qnew,qplot)
         ncmd = 1
         cmd(ncmd)='LAB F '//cdata
         CALL z2test(Lun, Ievent, Qplot, Mxcmd, Ncmd, Cmd)
C
      ELSE
         IF ( lbuf.GT.0 ) CALL PTBUF(' Bad command='//cbuf(:lbuf), -1)
C
      END IF
C
C loop round for next instruction
C
      GOTO 100
C
 999  CONTINUE
      lbuf=0
      CALL LOGGER(4, rtmp, itmp, cbuf, lbuf)
      CALL GTDEST
      CALL EDICOM('OFF',3)
      CALL DMPLUN
      END
