      SUBROUTINE OGRID
C---
C LEXTRCT routine to dispaly an RA, DEC grid on the image.  Grid lines
C plotted are actually contours of constant RA and DEC.
C---
C 1998-02-19 - Update to do either HMS or simple decimal coordinates [AFT].
C original version  tam/kaa
C---
C Machine min and max
      REAL      SYSMIN, SYSMAX
      PARAMETER (SYSMIN=-1.E32,SYSMAX=1.E32)
      INTEGER   MXLEV, MXGRID
      PARAMETER (MXLEV=15, MXGRID=19)
      REAL      FINTRP
      INTEGER   LENACT
C
      CHARACTER cbuf*32
      DOUBLE PRECISION dra, ddec
      REAL      ragrid(MXGRID,MXGRID), decgrid(MXGRID,MXGRID)
      REAL      vpos(MXGRID), vals(MXGRID)
      REAL      ralev(MXLEV), declev(MXLEV)
      REAL      pgtr(6)
      REAL      vport(4), wind(4)
      REAL      ca, d, decscl, decx, dmax, dmin
      REAL      range, rax, rmax, rmin, rascl, rota
      REAL      sa, scalex, scaley, schnew, schold
      REAL      tmp, tsec, x, xdet, xmar, y, ydet, ymar
      INTEGER   i, ici, icisav, iddx, iddy, idl, idh, idrx, idry
      INTEGER   idig, idms, ilwpar, ilwsav
      INTEGER   irh, irl, ihr, ideg, ilog, imin, isec, itmin, itsec
      INTEGER   j, lbuf, nd, ndec, nr
      REAL      rtd
      DATA  rtd/57.2957795/
C
C Character size
      schnew = 0.9
C Number of characters in DEC label
      ndec = 7
C      schnew = 1.7
C
C Idms=1 for degrees/hours, mins, sec; =0 for decimal degrees
      CALL POS_SKY(idms)
C
C Define constant arrays
C
      tmp = 1./(MXGRID-1.)
      DO i=1,MXGRID
         vpos(i) = (i-1)*tmp
      END DO
C
C Allow for future rotations
      rota=0.0
      ca=COS(rota/rtd)
      sa=SIN(rota/rtd)
      pgtr(1) =-tmp
      pgtr(2) = tmp*ca
      pgtr(3) =-tmp*sa
      pgtr(4) =-tmp
      pgtr(5) = tmp*sa
      pgtr(6) = tmp*ca
C
C Save current viewport and window.
      CALL PGQVP(0,vport(1),vport(3),vport(2),vport(4))
      CALL PGQWIN(wind(1),wind(2),wind(3),wind(4))
      CALL PGQCI(icisav)
      CALL PGQCH(schold)
      CALL PGQLW(ilwsav)
C
      CALL SSGCLW(ici, ilwpar)
      CALL PGSCI(ici)
      CALL PGSCH(schnew)
      CALL PGSLW(ilwpar)
C
      dmax = SYSMIN
      dmin = SYSMAX
      rmax = SYSMIN
      rmin = SYSMAX
C
C Get coordinates of grid on screen with 64 pixel resolution.
C
      scalex = (wind(2)-wind(1))/(MXGRID-1.)
      scaley = (wind(4)-wind(3))/(MXGRID-1.)
      DO i = 1, MXGRID
         xdet = wind(1) + scalex*(i-1.)
         DO j = 1, MXGRID
            ydet = wind(3) + scaley*(j-1.)
            CALL POS_DT_RA(xdet,ydet,dra,ddec)
            IF ( idms.GT.0 ) THEN
C Convert to minutes.
               rax = 60.*24.*SNGL(dra)/360.
               decx = 60.*SNGL(ddec)
            ELSE
               rax = SNGL(dra)
               decx = SNGL(ddec)
            END IF
C
            ragrid(i,j) = rax
            decgrid(i,j) = decx
            rmax = MAX(rmax, ragrid(i,j))
            rmin = MIN(rmin, ragrid(i,j))
            dmax = MAX(dmax, decx)
            dmin = MIN(dmin, decx)
         END DO
      END DO
C
      IF ( idms.GT.0 ) THEN
         range = 24.0*60.0
         IF ( (rmax-rmin).GT.range/2.0 ) THEN
C Looks like RA wrapped around.  Apply a fixup.
            rmax = SYSMIN
            rmin = SYSMAX
            DO i = 1, MXGRID
               DO j = 1, MXGRID
                  IF ( ragrid(i,j).GT.range/2.0 ) THEN
                     ragrid(i,j) = ragrid(i,j) - range
                  END IF
                  rmax = MAX(rmax, ragrid(i,j))
                  rmin = MIN(rmin, ragrid(i,j))
               END DO
            END DO
         END IF
      END IF
C
C Set distance between dec grid lines as one fifth of the distance across
C the current image. Note that accuracy of numbers on grid is limited to
C one arcminute
C
      IF ( idms.GT.0 ) THEN
         ndec = 7
         IF (      (dmax-dmin).LT.  0.2) THEN
            decscl =   2./60.
            ndec = 10
         ELSE IF ( (dmax-dmin).LT.  0.5) THEN
            decscl =   5./60.
            ndec = 10
         ELSE IF ( (dmax-dmin).LT.  1.0) THEN
            decscl =  10./60.
            ndec = 10
         ELSE IF ( (dmax-dmin).LT.  3. ) THEN
            decscl =  30./60.
            ndec = 10
         ELSE IF ( (dmax-dmin).LT.  6. ) THEN
            decscl =  1.
         ELSE IF ( (dmax-dmin).LT. 16. ) THEN
            decscl =  2.
         ELSE IF ( (dmax-dmin).LT. 40. ) THEN
            decscl =  5.
         ELSE IF ( (dmax-dmin).LT. 80. ) THEN
            decscl = 10.
         ELSE IF ( (dmax-dmin).LT.160. ) THEN
            decscl = 20.
         ELSE IF ( (dmax-dmin).LT.360. ) THEN
            decscl = 60.
         ELSE IF ( (dmax-dmin).LT.720. ) THEN
            decscl = 120.
         ELSE IF ( (dmax-dmin).LT.2400. ) THEN
            decscl = 300.
         ELSE
            decscl = 900.
         END IF
      ELSE
         ilog = INT(LOG10((dmax-dmin)/5.)+100.)-100
         idig = NINT((dmax-dmin)/(4.5*10.**ilog))
         IF ( idig.EQ.3 ) idig = 2
         IF ( idig.GE.4) idig = 5
         decscl = idig*10.**ilog
      END IF
C
C Similarly for the RA grid lines though here need COS(dec) factor
C division by 15 is minutes in RA to arc-minutes conversion.  Numbers on
C grid have an accuracy of 0.1 minutes in RA.
C
      IF ( idms.GT.0 ) THEN
C         rascl = decscl/COS(decgrid(5,5)*3.1415927/(180*60))/15.
         rascl = (rmax-rmin)/5.
         IF      ( rascl.LT.0.002 ) THEN
            rascl = 1./600.
         ELSE IF ( rascl.LT.0.003 ) THEN
            rascl = 1./300.
         ELSE IF ( rascl.LT.0.01 ) THEN
            rascl = 1./120.
         ELSE IF ( rascl.LT.0.02 ) THEN
            rascl = 1./60.
         ELSE IF ( rascl.LT.0.04 ) THEN
            rascl = 2./60.
         ELSE IF ( rascl.LT.0.10 ) THEN
            rascl = 5./60.
         ELSE IF ( rascl.LT.0.15 ) THEN
            rascl = 0.10
         ELSE IF ( rascl.LT.0.30 ) THEN
            rascl = 0.20
         ELSE IF ( rascl.LT.0.60 ) THEN
            rascl = 0.50
         ELSE
            rascl = NINT(rascl)
         END IF
      ELSE
         ilog = INT(LOG10((rmax-rmin)/5.)+100.)-100
         idig = NINT((rmax-rmin)/(4.5*10.**ilog))
         IF ( idig.EQ.3 ) idig = 2
         IF ( idig.GE.4) idig = 5
         rascl = idig*10.**ilog
      END IF
C
C Find number of contour levels.  Note, INT rounds towards zero which
C is incorrect for negative numbers, hence a constant offset of 6000 (>60*90)
C is added to force number to be positive and then offset is subtracted.
C
      irl = INT(6000+rmin/rascl)-6000
      irh = INT(6000+rmax/rascl)-6000
      nr = MIN(MXLEV, irh - irl + 1)
      idl = INT(6000+dmin/decscl)-6000
      idh = INT(6000+dmax/decscl)-6000
      nd = MIN(MXLEV, idh - idl + 1)
C
C Set contour levels
C
      DO i = 1, nr
         ralev(i) = rascl*(irl+i-1)
      END DO
      DO i=1, nd
         declev(i) = decscl*(idl+i-1)
      END DO
C
C Plot coordinate grids.
C
      xmar = 0.130*schnew*ndec/7.0
      ymar = 0.050*schnew
      CALL PGSVP(vport(1),vport(3)-xmar*(vport(3)-vport(1)),
     :           vport(2),vport(4)-ymar*(vport(4)-vport(2)))
      CALL PGSWIN(0.0,1.0-xmar,0.0,1.0-ymar)
      CALL PGCONT(ragrid,MXGRID,MXGRID,1,MXGRID,1,MXGRID,ralev,nr,pgtr)
      CALL PGSLS(3)
      CALL PGCONT(decgrid,MXGRID,MXGRID,1,MXGRID,1,MXGRID,
     &      declev,nd,pgtr)
      CALL PGSLS(1)
C
C Plot coordinates.
C
      IF ( ABS(ragrid(1,1)-ragrid(1,MXGRID)).GT.
     &     ABS(ragrid(1,1)-ragrid(MXGRID,1)) ) THEN
         idrx = 0
         idry = 1
         iddx = 1
         iddy = 0
      ELSE
         idrx = 1
         idry = 0
         iddx = 0
         iddy = 1
      END IF
C
C RA.
C
      DO i = 1, MXGRID
         IF ( idrx.NE.0 ) THEN
            vals(i) = ragrid(i,MXGRID)
         ELSE
            vals(i) = ragrid(MXGRID,i)
         END IF
      END DO
C
      DO i = 1, nr
         d = FINTRP(ralev(i),vals,vpos,MXGRID)
C Skip if too close to edge of plotting window
         IF ( 0.05.LE.d .AND. d.LE.1.0-xmar ) THEN
            IF ( idrx.EQ.0 ) THEN
C Right side
               x = 1.0 - 0.6*xmar
               y = d - 0.15*ymar
            ELSE
C Top
               x = d
               y = 1.0 - 0.7*ymar
            END IF
            IF ( idms.NE.0 ) THEN
               ihr = ralev(i)/60.
               rmin = ralev(i) - 60.*ihr
               IF ( rascl.LT.0.01 ) THEN
                  itmin = rmin
                  tsec = (rmin-itmin)*60.
                  WRITE (cbuf,211) ihr, itmin, tsec
  211             FORMAT(I2,'\uh\d',I2,'\um\d',F4.1,'\us\d')
               ELSE IF ( rascl.LT.0.09 ) THEN
                  itmin = rmin
                  itsec = NINT((rmin-itmin)*60.)
                  WRITE (cbuf,221) ihr, itmin, itsec
  221             FORMAT(I2,'\uh\d',I2,'\um\d',I2,'\us\d')
               ELSE
                  WRITE (cbuf,231) ihr, rmin
  231             FORMAT(I2,'\uh\d',F4.1,'\um\d')
               END IF
               lbuf = LENACT(cbuf)
            ELSE
               lbuf = 0
               CALL CRAMF(ralev(i),cbuf,lbuf)
            END IF
            CALL PGPTXT(x,y,0.0,0.5,cbuf(:lbuf))
         END IF
      END DO
C
C Dec.
C
      DO i = 1, MXGRID
C         vals(i) = decgrid(1+(i-1)*iddx,1+(i-1)*iddy)
         IF ( iddx.NE.0 ) THEN
            vals(i) = decgrid(i,MXGRID)
         ELSE
            vals(i) = decgrid(MXGRID,i)
         END IF
      END DO
C
      DO i = 1, nd
         d = FINTRP(declev(i),vals,vpos,MXGRID)
         IF ( 0.01.LE.d .AND. d.LE.0.95 ) THEN
            decx = declev(i)
            IF ( iddx.EQ.0 ) THEN
C Right side
               x = 1.0 - 0.52*xmar
               y = d*iddy - 0.15*ymar
            ELSE
C Top
               x = d*iddx
               y = 1.0 - 0.7*ymar
            END IF
            IF ( idms.NE.0 ) THEN
               IF ( decscl.LT.1.0 ) THEN
                  ideg = decx/60.
                  imin = ABS(decx-60.*ideg) + .01
                  isec = 60*(ABS(decx-60.*ideg)-imin) + .01
                  WRITE(cbuf,261) ideg, imin, isec
  261             FORMAT(I3,'\(718)',I2,'\(716)',I2,'\(717)')
                  lbuf = LENACT(cbuf)
                  IF ( -1.0.LT.ddec .AND. ddec.LT.0.0 ) THEN
C Handle negative 0.
                     cbuf(2:2)='-'
                  END IF
               ELSE
                  ideg = decx/60.
                  imin = ABS(decx-60.*ideg) + .01
                  WRITE (cbuf,271) ideg, imin
  271             FORMAT(I3,'\(718)',I2,'\(716)')
                  lbuf = LENACT(cbuf)
                  IF ( -1.0.LT.ddec .AND. ddec.LT.0.0 ) THEN
                     cbuf(2:2)='-'
                  END IF
               END IF
            ELSE
               lbuf = 0
               CALL CRAMF(decx,cbuf,lbuf)
            END IF
            CALL PGPTXT(x,y,0.0,0.5,cbuf(:lbuf))
         END IF
      END DO
C
C Flush plot to display and then restore coordinate system.
C
      CALL PGUPDT
      CALL PGSLW(ilwsav)
      CALL PGSCH(schold)
      CALL PGSCI(icisav)
      CALL PGSVP(vport(1),vport(3),vport(2),vport(4))
      CALL PGSWIN(wind(1),wind(2),wind(3),wind(4))
      RETURN
C
      END
