C****************************************************************************
C File containing LEXTRCT routines for handling coordinate transformations.
C all routines have names POS_????
C
C 1997-Apr-08 - Bug fix [AFT]
C 1993-Sep-02 - Use reals for world coordinates [AFT]
C 4/5/85      - version(1)  [kaa]
C----------------------------------------------------------------------------
C
      SUBROUTINE POSDET(Xpos, Ypos)
      REAL     Xpos, Ypos
C---
C Returns the current Xdet, Ydet location.
C---
C
      INCLUDE 'posdef.inc'
C
      Xpos = Xdet
      Ypos = Ydet
      RETURN
C
      END
C
C----------------------------------------------------------------------------
C
      SUBROUTINE POS_DT_IM(Xdpos, Ydpos, Ixmap, Iymap)
      REAL      Xdpos, Ydpos
      INTEGER   Ixmap, Iymap
C---
C Converts detector coordinates (Xdpos,Ydpos) into map array integer
C coordinates (Ixm,Iym) using (Xdmap1,Ydmap1) as detector coordinate
C corresponding to map coordinate (1,1) and IFAC as compression factor
C from detector to map.
C WARNING:  This routine does not check if the map position actually
C lie within the data array indices.
C---
C Xdpos   I    X position in detector coordinates
C Ydpos   I    Y position in detector coordinates
C Ixmap     O  X position in map coordinates
C Iymap     O  Y position in map coordinates
C
C Global:
C  Xdmap1,Ydmap1  map (1,1) in detector coords
C  Ifac           det to map compression factor
C---
      INCLUDE 'posdef.inc'
C---
      Ixmap = NINT(1. + (Xdpos-Xdmap1)/Ifac)
      Iymap = NINT(1. + (Ydpos-Ydmap1)/Ifac)
C
      END
C
C----------------------------------------------------------------------------
C
      SUBROUTINE POS_DT_RM(Xdpos, Ydpos, Rxmap, Rymap)
      REAL      Xdpos, Ydpos
      REAL      Rxmap, Rymap
C---
C Converts detector coordinates (Xdpos,Ydpos) into map array real
C coordinates (Ixm,Iym) using (Xdmap1,Ydmap1) as detector coordinate
C corresponding to map coordinate (1,1) and IFAC as compression factor
C from detector to map.
C---
C Xdpos   I    X position in detector coordinates
C Ydpos   I    Y position in detector coordinates
C Ixmap     O  X position in map coordinates
C Iymap     O  Y position in map coordinates
C
C Global:
C  Xdmap1,Ydmap1  map (1,1) in detector coords
C  Ifac           det to map compression factor
C---
      INCLUDE 'posdef.inc'
C---
      Rxmap = 1. + (Xdpos-Xdmap1)/Ifac
      Rymap = 1. + (Ydpos-Ydmap1)/Ifac
C
      END
C
C ---------------------------------------------------------------------------
C
      SUBROUTINE POS_MAP_DET(Ixm, Iym, Xdpos, Ydpos)
      INTEGER   Ixm, Iym
      REAL      Xdpos, Ydpos
C---
C Converts map coordinates (Ixm,Iym) into detector coordinates (Xdpos,Ydpos)
C using (Xdmap1,Ydmap1) as detector coordinate corresponding to map coordinate
C (1,1) and Ifac as compression factor from detector to map.
C---
C Ixm     I
C Iym     I
C Xdpos     O
C Ydpos     O
C
C Global:
C  Xdmap1,Ydmap1  map (1,1) in detector coords
C  Ifac           det to map compression factor
C---
      INCLUDE 'posdef.inc'
C
C Transform the center of the map pixel
      Xdpos = Xdmap1 + Ifac*(Ixm - 1)
      Ydpos = Ydmap1 + Ifac*(Iym - 1)
C
      END
C
C --------------------------------------------------------------------------
C
      SUBROUTINE POS_RA_DT(Dra, Ddec, Xdpos, Ydpos)
      DOUBLE PRECISION Dra, Ddec
      REAL      Xdpos, Ydpos
C---
C Converts RA and DEC in radians into detector pixels, Xdpos, Ydpos, using
C FITS coordinate conventions.
C---
C Dra     I    RA in degrees
C Ddec    I    Dec in degrees
C Xdpos     O
C Ydpos     O
C---
      INCLUDE 'posdef.inc'
C
      DOUBLE PRECISION dxpix, dypix
      INTEGER   istat
C---
      istat = 0
      IF ( Imat.NE.0 ) THEN
         CALL afxypx(Dra,Ddec,dxref,dyref,dxpix0,dypix0,
     &     dcd11,dcd12,dcd21,dcd22,ctype1,dxpix,dypix,istat)
      ELSE
         CALL ftxypx(Dra,Ddec,dxref,dyref,dxpix0,dypix0,
     &           dxdel,dydel,drota,ctype1,dxpix,dypix,istat)
C If ftxypx doesn't recognize coordindates, it returns a 504, and we
C do a simple linear transformation.
         IF ( istat.EQ.504 ) THEN
            dxpix=(Dra -dxref)/dxdel+dxpix0
            dypix=(Ddec-dyref)/dydel+dypix0
         END IF
      END IF
      Xdpos = dxpix
      Ydpos = dypix
      RETURN
      END
C
C ----------------------------------------------------------------------------
C
      SUBROUTINE POS_DT_RA(Xdpos, Ydpos, Ra, Dec)
      REAL      Xdpos, Ydpos
      DOUBLE PRECISION Ra, Dec
C---
C Converts detector pixel position, Xdpos, Ydpos into RA and DEC in radians
C using the FITS coordinate conventions.
C---
C Xdpos   I
C Ydpos   I
C Ra        O  RA in decimal degrees
C Dec       O  Dec in decimal degrees
C---
      INCLUDE 'posdef.inc'
C
      CHARACTER ctmp*32
      DOUBLE PRECISION dxpix, dypix
      INTEGER   istat
C---
      dxpix = Xdpos
      dypix = Ydpos
      istat = 0
      IF ( Imat.NE.0 ) THEN
         CALL afwldp(dxpix,dypix,dxref,dyref,dxpix0,dypix0,
     &           dcd11,dcd12,dcd21,dcd22,ctype1,Ra,Dec,istat)
      ELSE
         CALL ftwldp(dxpix,dypix,dxref,dyref,dxpix0,dypix0,
     &               dxdel,dydel,drota,ctype1,Ra,Dec,istat)
C If ftwldp doesn't recognize coordindates, it returns a 504, and we
C do a simple linear transformation.
         IF ( istat.EQ.504 ) THEN
            Ra =(dxpix-dxpix0)*dxdel+dxref
            Dec=(dypix-dypix0)*dydel+dyref
         END IF
      END IF
      RETURN
      END
C
C ----------------------------------------------------------------------------
C
      SUBROUTINE POS_DEG_HMS(Ra, Dec, Irah, Iram, Rras,
     &      Cdecd, Idecm, Decs)
      DOUBLE PRECISION Ra, Dec
      INTEGER   Irah, Iram, Idecm
      CHARACTER Cdecd*3
      REAL      Rras, Decs
C---
C Converts RA and DEC in degrees into hours, minutes, seconds (RAH, RAM, RAS)
C and degrees, minutes, seconds (Idecd, Idecm, Idecs) format.
C Note:  Dec degrees is a character array to handle the -0 case.
C---
C Ra      I    RA in degrees
C Dec     I    Dec in degrees
C Rah       O  RA hours
C Ram       O  RA minutes
C Ras       O  RA seconds
C Cdecd     O  Dec degrees
C Idecm     O  Dec minutes
C Decs      O  Dec seconds
C---
      REAL*8    r, d
      INTEGER   idecd, ltmp
C
C Convert to hours (RA) and degrees (DEC)
C
      r = Ra*12.D0/180.D0
C
C Put RA within 0 - 24
C
      IF ( r.LT.0 ) r = 24.D0 + r
      r = MOD(r,24.D0)
C
C Convert to HMS
C
      Irah = r
      r = 60.D0*(r-Irah)
      Iram = r
      r = 60.D0*(r-Iram)
      Rras = SNGL(r)
C
C Convert to DMS
C
C Make positive so truncation works correctly.
      d=ABS(Dec)
      idecd = d
      ltmp=0
      Cdecd=' '
      CALL CRAMIF(idecd, 3, Cdecd, ltmp)
      IF ( Dec.LT.0.0 ) THEN
         IF ( Cdecd(2:2).EQ.' ' ) THEN
             Cdecd(2:2)='-'
         ELSE
             Cdecd(1:1)='-'
         END IF
      END IF
      d = 60.D0*(d-Idecd)
      Idecm = d
      d = 60D0*(d-Idecm)
      Decs = d
C
      END
C
C -----------------------------------------------------------------------------
C
      SUBROUTINE POS_HMS_DEG(Irah, Iram, Ras, Isn, Idecd, Idecm, Decs,
     &      Ra, Dec)
      INTEGER   Irah, Iram, Isn, Idecd, Idecm
      REAL      Ras, Decs
      DOUBLE PRECISION Ra, Dec
C---
C Converts ra and dec from h, m, s (RAH, RAM, RAS) and d, m, s (DECD, DECM,
C DECS) format to radians (RA, DEC)
C---
C Irah    I    RA hours
C Iram    I    RA mins
C Rras    I    RA seconds
C Isn     I    +1/-1 (sign of DEC)
C Idecd   I    Dec degrees
C Idecn   I    Dec minutes
C Decs    I    Dec seconds
C Ra        O  RA in degrees
C Dec       O  Dec in degrees
C---
      REAL*8    r, d
C
C Convert to degrees
C
      r = DFLOAT(Irah)*15.D0 + DFLOAT(Iram)/4.D0 + DBLE(Ras)/240.D0
C
      IF ( Isn.GT.0 ) THEN
         d = DFLOAT(Idecd) + DFLOAT(Idecm)/60.D0 + DBLE(Decs)/3600.D0
      ELSE
         d =DFLOAT(ABS(Idecd))+DFLOAT(ABS(Idecm))/60.D0+
     &      DBLE(ABS(Decs))/3600.D0
         d = -d
      END IF
C
      Ra = r
      Dec = d
C

      END
C
C -----------------------------------------------------------------------------
C
      SUBROUTINE POS_ON_MAP(Rdx, Rdy)
      REAL      Rdx, Rdy
C---
C Ensure that the detector position (Rdx, Rdy) lies on the current map.
C---
C Rdx     I/O
C Rdy     I/O
C---
      INCLUDE 'mapdef.inc'
      INTEGER   imx, imy
C---
      CALL POS_DT_IM(Rdx, Rdy, imx, imy)
      IF ( imx.LE.0 .OR. Mxrow.LT.imx .OR.
     &     imy.LE.0 .OR. Mxcol.LT.imy ) THEN
         imx = MIN(MAX(1,imx),Mxrow)
         imy = MIN(MAX(1,imy),Mxcol)
         CALL POS_MAP_DET(imx, imy, Rdx, Rdy)
      END IF
      END
C -----------------------------------------------------------------------------
C
      SUBROUTINE POS_SKY(isky)
      INTEGER   isky
C---
C Check to see if WCS converter can convert to sky coordinates.
C---
C Isky      O
C---
      INCLUDE 'posdef.inc'
      DOUBLE PRECISION dra, ddec
      INTEGER istat
C---
C If ftwldp fails, it does a simple linear transformation but returns
C an istat=504.
      istat = 0
      CALL ftwldp(1.0D0,1.0D0,dxref,dyref,dxpix0,dypix0,
     &            dxdel,dydel,drota,ctype1,dra,ddec,istat)
      IF ( istat.EQ.504 ) THEN
         isky=0
      ELSE
         isky=1
      END IF
C
      END
C
C -----------------------------------------------------------------------------
C
      SUBROUTINE POS_HMS(Ihms)
      INTEGER   Ihms
C---
C If we are using RA/Dec system, then allow Ihms=1, otherwise Ihms=0
C---
C IHms      O
C---
      INCLUDE 'posdef.inc'
      CHARACTER ctmp*32
C---
      ctmp=ctype1
      CALL UPC(ctmp)
      IF ( ctmp(1:4).EQ.'-TAN' ) THEN
         Ihms=1
      ELSE
         Ihms=0
      END IF
C
      END
C
C -----------------------------------------------------------------------------
