      SUBROUTINE PSFSIG(Xdpos, Ydpos, Rsig)
      REAL      Xdpos, Ydpos, Rsig
C Entry GTCASA
      REAL      Ca1, Sa1
C Entry GTFWCN/STFWCN
      REAL      Xcen1, Ycen1
C Entry GTFWAT/STFWAT
      REAL      Fwatc1
C Entry GTFWQU/STFWQU
      REAL      Fwqua1
      REAL      Roll1
C Entry PSFCXO
      REAL      Xpos1, Ypos1, Sigx, Sigy, Ang
C---
C LEXTRCT function to return the full width half maximum of the point
C spread function at the detectory position Xdpos, Ydpos.  Sigma is returned
C in detector units.
C---
C Xdpos   I    X detector position
C Ydpos   I    Y detector position
C---
C 1997-Apr-15 - Updated to remove LE specific code [AFT]
C---
      INTEGER   MXR
      PARAMETER (MXR=13)
C
      REAL      sx(MXR), sy(MXR)
      REAL      phi, rad, rsq, xt, yt
      REAL      ca, Fxcen, Fycen, Fwatcn, Fwquad, roll, sa
      SAVE      ca, Fxcen, Fycen, Fwatcn, Fwquad, roll, sa
      REAL      rtd
      INTEGER   ir
C These are the average sigx, sigy within 0-1, 1-2, 2-3, etc.  The
C first bin is repeated to avoid extrapolation for r<0.5.  Units are
C PSFLIB pixels or 0.25 of an ACIS pixel.
      DATA sx/2.01,2.01,2.66,5.02,8.11,12.2,18.6,23.6,30.4,
     &       36.9,44.2,51.9,64.2/
      DATA sy/1.71,1.71,1.78,2.18,3.01,4.56,8.36,11.0,15.5,
     &       19.7,25.0,30.9,41.2/
      DATA Fxcen, Fycen, Fwatcn, Fwquad/0.0,0.0,0.0,0.0/
      DATA ca, roll, sa/1.0,0.0,0.0/
      DATA  rtd/57.2957795/
C---
C rsq is the offset distance squared measured in pixels.
      rsq = (Xdpos-Fxcen)**2 + (Ydpos-Fycen)**2
C
      Rsig = Fwatcn + Fwquad*rsq
C---
      RETURN
C*********
      ENTRY GTCASA(Ca1, Sa1)
C Get Cos/Sin of Roll angle
      Ca1 = ca
      Sa1 = sa
      RETURN
C*********
      ENTRY GTFWCN(Xcen1, Ycen1)
C Center for FWHM calculations.
      Xcen1 = Fxcen
      Ycen1 = Fycen
      RETURN
C*********
      ENTRY GTFWAT(Fwatc1)
C The Sigma at the center.
      fwatc1 = Fwatcn
      RETURN
C*********
      ENTRY GTFWQU(Fwqua1)
C The quadratic part of Sigma.
      Fwqua1 = Fwquad
      RETURN
C*********
      ENTRY PSFCXO(Xpos1, Ypos1, Sigx, Sigy, Ang)
C---
C Xpos1,Ypos1 I    The position of interest
C Sigx,SIgx     O  Sigma-x/Sigma-y of the ellipse
C Ang           O  The angle of the ellipse
C---
      xt =  (Xpos1-fxcen)*ca+(Ypos1-fycen)*sa
      yt = -(Xpos1-fxcen)*sa+(Ypos1-fycen)*ca
      phi = rtd*ATAN2(yt, xt)
C At -69.9 degrees image is vertical
      Ang = -roll-90.0+(phi+69.5)/2.
      IF ( Ang.LT.0.0 ) Ang = Ang + 360.0
C R is measured in arcmin, the first bin is from 0-1, then 1-2, etc.
      rad = 0.492*SQRT(xt*xt+yt*yt)/60.
      ir  = MIN(MXR-1,1+NINT(rad))
      Sigx = 1.00*(sx(ir)+(sx(ir+1)-sx(ir))*(rad-(ir-1.5)))/4.
      Sigy = 1.00*(sy(ir)+(sy(ir+1)-sy(ir))*(rad-(ir-1.5)))/4.
      RETURN
C*********
      ENTRY PSGROL(Roll1)
C Roll angle
      Roll1 = roll
      RETURN
C*********
      ENTRY PSSROL(Roll1)
C Roll angle.  Increasing roll angle, rotates the FoV clockwise on the sky.
      roll = Roll1
      ca = COS(-roll/rtd)
      sa = SIN(-roll/rtd)
      RETURN
C*********
      ENTRY STFWCN(Xcen1, Ycen1)
C Center for Sigma calculations.
      Fxcen = Xcen1
      Fycen = Ycen1
      RETURN
C*********
      ENTRY STFWAT(Fwatc1)
C The Sigma at the center.
      Fwatcn =fwatc1
      RETURN
C*********
      ENTRY STFWQU(Fwqua1)
C The quadratic part of Sigma.
      Fwquad = Fwqua1
      RETURN
      END
C*********
      SUBROUTINE PSFSET(Ifac, Xpos, Ypos, Mxpsf, Npsf, Xpect)
      INTEGER   Ifac, Npsf, Mxpsf
      REAL      Xpos, Ypos
      REAL      Xpect(-Mxpsf:Mxpsf,-Mxpsf:Mxpsf)
C Entry PSGELL
      REAL      Xpos1, Ypos1, Sigx1, Sigy1, Ang1
C Entry PSGFIL/PSPFIL
      CHARACTER Cpsf1*(*)
      INTEGER   Lpsf1
C Entry PSGSLI
      INTEGER   Isliceo
C Entry PSQMO/PSSMO
      INTEGER   Ipsfmi
C Entry PSQMOC/PSSMOC
      CHARACTER cpmod*(*)
C Entry PSQPAR
      INTEGER   Npar
      REAL      Pars(*)
C Entry PSSPAR
      CHARACTER cpar*(*)
      INTEGER   lpar
C---
C This subroutine sets up the xpect array describing the psf normalized so
C that it sums to 1 (so we are estimating total no. of counts not amplitude).
C---
C Ifac
C Xpos
C Ypos
C Mxpsf   I    The actual size of the Xpect array
C Npsf    I/O  Requested/actual size of Xpect
C Xpect     O  The PSF image.
C---
      INTEGER   MXDAT
      PARAMETER (MXDAT=512)
      REAL      FPNUM
C
      CHARACTER cpsf*256
      SAVE      cpsf
      CHARACTER ctmp*32
      DOUBLE PRECISION dtot
      REAL      rdat(MXDAT,MXDAT)
      SAVE      rdat
      REAL      rbuf(MXDAT)
      REAL      ecc, p4, theta
      SAVE      ecc, p4, theta
      REAL      ang, rnorm, rnul, rsig, rsum, scale, sigx, sigy
      REAL      ca, r2, sa, tmp, xcen, xp, ycen, yp
      REAL      over, space, xpc, ypc
      SAVE      over, space, xpc, ypc
      INTEGER   naxes(11)
      SAVE      naxes
      INTEGER   i, ibegin, ibits, ic5, ic6, ien, ier, igcnt, igroup
      INTEGER   ioff, ip, ipcnt, islice, istat, itmp
      INTEGER   j, jp, kp, ltmp, naxis
      INTEGER   ipsfm, lpsf, lslice, lunpsf, iwarn
      SAVE      ipsfm, lpsf, lslice, lunpsf, iwarn
      LOGICAL   qbad, qsimp, qxtend
C
      REAL      rtd
      REAL      rlast
      SAVE      rlast
      DATA cpsf,lpsf,lslice,lunpsf,ipsfm/' ',0,0,0,1/
      DATA  rtd/57.2957795/
      DATA rlast/0.0/
      DATA   ecc, p4, theta/0.1, 4.0, 0.0/
C
  100 CONTINUE
      IF ( Npsf.GT.Mxpsf ) THEN
         IF ( iwarn.LE.0 ) THEN
            WRITE(*,*) '***WARNING--Npsf=',Npsf,', Mxpsf=',Mxpsf
         END IF
         iwarn = 1
         Npsf=MIN(Npsf, Mxpsf)
      END IF
      IF ( lpsf.LE.0 ) THEN
         IF ( ipsfm.EQ.1 ) THEN
C Default here Unless ipsfm.EQ.2
C Fill the Xpect array assuming the PSF to be a circular gaussian.
            CALL PSFSIG(Xpos, Ypos, rsig)
C Calculate sig in map units
            rsig = rsig/Ifac
            sigx = rsig
            sigy = rsig
            rnorm = 1.0/(6.28318531*sigx*sigy)
            rsum = 0.0
            DO j = -Npsf,Npsf
               DO i = -Npsf,Npsf
                  r2 = (i/sigx)**2 + (j/sigy)**2
                  Xpect(i,j) = rnorm*EXP(-r2/2.)
                  rsum = rsum + Xpect(i,j)
               END DO
            END DO
C
         ELSE IF ( ABS(ipsfm).EQ.2 ) THEN
            IF ( ipsfm.LT.0 ) THEN
C Fill the Xpect array assuming the PSF to be the CXO ellipitical gaussian.
               CALL PSFCXO(Xpos, Ypos, sigx, sigy, ang)
            ELSE
               CALL PSFSIG(Xpos, Ypos, rsig)
               sigx=rsig
               sigy=rsig*(1.-ecc)
               ang=theta
            END IF
            sigx = sigx/Ifac
            sigy = sigy/Ifac
            ca = COS(ang/rtd)
            sa = SIN(ang/rtd)
            rnorm = 1.0/(6.28318531*sigx*sigy)
            rsum = 0.0
            DO j = -Npsf,Npsf
               DO i = -Npsf,Npsf
                  xp = ca*i + sa*j
                  yp =-sa*i + ca*j
                  r2 = (xp/sigx)**2 + (yp/sigy)**2
                  IF ( r2.LT.144.0 ) THEN
                     Xpect(i,j) = rnorm*EXP(-r2/2.)
                     rsum = rsum + Xpect(i,j)
                  ELSE
                     Xpect(i,j) = 0.0
                  END IF
               END DO
            END DO
         ELSE IF ( ipsfm.EQ.3 ) THEN
C Fill the Xpect array assuming a Moffat function
            CALL PSFSIG(Xpos, Ypos, rsig)
C Calculate sig in map units (Note factor of 2 for Moffat)
            rsig = 2.0*rsig/Ifac
            IF ( rsig.NE.rlast ) THEN
               rlast = rsig
               rnorm = (P4-1)/(3.14159*rsig*rsig)
               rsum = 0.0
               DO j = -Npsf,Npsf
                  DO i = -Npsf,Npsf
                     r2 = (i/rsig)**2 + (j/rsig)**2
                     Xpect(i,j) = rnorm/(1.0+r2)**p4
                     rsum = rsum + Xpect(i,j)
                  END DO
               END DO
            END IF
C
         END IF
C
C If psf_sig .lt. 1 pixel, then the discrete sum does not match integral,
C so we need to renormalize.
         IF ( rsum.GT.1.02 ) THEN
            rnorm = 1./rsum
            DO j = -Npsf,Npsf
               DO i = -Npsf,Npsf
                  Xpect(i,j) = rnorm*Xpect(i,j)
               END DO
            END DO
         END IF
      ELSE
         IF ( lunpsf.LE.0 ) THEN
            CALL GETLUN(lunpsf)
            istat = 0
            CALL FTOPEN(lunpsf,cpsf,0,itmp,istat)
            IF ( istat.NE.0 ) THEN
               WRITE(*,*) 'PSFSET--Unable to open ',cpsf(:lpsf)
               CALL FRELUN(lunpsf)
               lunpsf = 0
               lpsf = 0
               GOTO 100
            END IF
            istat = 0
            CALL FTGHPR(lunpsf,11,qsimp,ibits,naxis,naxes,ipcnt,igcnt,
     &         qxtend, istat)
            IF ( naxes(1).GT.MXDAT .OR. naxes(2).GT.MXDAT ) THEN
               WRITE(*,*) 'PSFSET--PSF size of ',naxes(1),naxes(2),
     &            'exceeds ',MXDAT,MXDAT,' closing file.'
               CALL FRELUN(lunpsf)
               lunpsf = 0
               lpsf = 0
               GOTO 100
            END IF
C Get X,Y location of the center of the PSF
            xpc = naxes(1)/2
            ypc = naxes(2)/2
            CALL FTGKYE(lunpsf, 'CRPIX1', xpc, ctmp, istat)
            CALL FTGKYE(lunpsf, 'CRPIX1', ypc, ctmp, istat)
C Compute pixel oversample rate, and spacing between images
            CALL FTGKYE(lunpsf, 'CD1_1P', tmp, ctmp, istat)
            over = 1.0/tmp
            CALL FTGKYE(lunpsf, 'CD5_5P', tmp, ctmp, istat)
            space = 1.0/tmp
         END IF
         CALL GTFWCN(xcen, ycen)
         CALL GTCASA(ca, sa)
C ic5, and ic6 are the (Fortran) array indices for columns 5 and 6.
         ic5=NINT((naxes(5)+1)/2+( (Xpos-xcen)*ca+(Ypos-ycen)*sa)*space)
         ic6=NINT((naxes(6)+1)/2+(-(Xpos-xcen)*sa+(Ypos-ycen)*ca)*space)
         ic5 = MIN(MAX(1,ic5),naxes(5))
         ic6 = MIN(MAX(1,ic6),naxes(6))
         ien = 2
C      write(*,*) ien,ic5,ic6
         islice = ien + (ic5-1)*naxes(4) + (ic6-1)*naxes(4)*naxes(5)
C To speed up, we read the current PSF into a SAVE'd buffer array rdat
C and avoid FITSIO overhead if correct array is already in memory.
         IF ( islice.NE.lslice ) THEN
C This is the azimuth and elevation angles in arcmins.
C         write(*,*) 'az=',( (Xpos-xcen)*ca+(Ypos-ycen)*sa)/121.95
C         write(*,*) 'el=',(-(Xpos-xcen)*sa+(Ypos-ycen)*ca)/121.95
C         write(*,*) 'islice=',islice
            ibegin = 1 + (islice-1)*naxes(1)*naxes(2)
            igroup = 0
            rnul = 0.
            dtot = 0.0d0
            istat = 0
            DO j=1, naxes(2)
               ioff = ibegin + (j-1)*naxes(1)
               CALL FTGPVE(lunpsf,igroup,ioff,naxes(1),rnul,
     &            rbuf,qbad,istat)
               DO i=1, naxes(1)
                  dtot = dtot + rbuf(i)
                  rdat(i,j) = rbuf(i)
               END DO
            END DO
            IF ( istat.NE.0 ) THEN
               WRITE(*,*) 'Failed to read islice=',islice,
     &            ', istat=',istat
            END IF
C Documentation says PSF is normalized to 1, but it lies.
            rnorm = 1./dtot
            DO j=1, naxes(2)
               DO i=1, naxes(1)
                  rdat(i,j) = rnorm*rdat(i,j)
               END DO
            END DO
            lslice = islice
C From here to END IF can be recomputed each time.  However, since nothing
C changes it is 2.2x faster assume the Xpect array has not been modified.
            DO j = -Npsf, Npsf
               DO i = -Npsf, Npsf
                  Xpect(i,j) = 0.0
               END DO
            END DO
            scale = 1./(over*Ifac)
            DO j=1, naxes(2)
               DO i=1, naxes(1)
                  ip=NINT(Mxpsf+scale*( (i-xpc)*ca-(j-ypc)*sa))-Mxpsf
                  jp=NINT(Mxpsf+scale*( (i-xpc)*sa+(j-ypc)*ca))-Mxpsf
                  IF ( -Npsf.LE.ip .AND. ip.LE.Npsf .OR.
     &                 -Npsf.LE.jp .AND. jp.LE.Npsf ) THEN
                     Xpect(ip,jp) = Xpect(ip,jp) + rdat(i,j)
                  END IF
               END DO
            END DO
         END IF
      END IF
C
      RETURN
C*********
      ENTRY PSGELL(Xpos1, Ypos1, Sigx1, Sigy1, Ang1)
C Get ellipical approximation to PSF.
C Sigx1, Sigy1 are in detector units
      IF ( ipsfm.LT.0 ) THEN
         CALL PSFCXO(Xpos1, Ypos1, Sigx1, Sigy1, Ang1)
      ELSE
         CALL PSFSIG(Xpos1, Ypos1, rsig)
         IF ( ipsfm.EQ.3 ) THEN
            Sigx1=2.1*rsig
            Sigy1=1.9*rsig
            Ang1=0.0
         ELSE IF ( ipsfm.EQ.2 ) THEN
            Sigx1=rsig
            Sigy1=rsig*(1.-ecc)
            Ang1=theta
         ELSE
            Sigx1=1.05*rsig
            Sigy1=0.95*rsig
            Ang1=0.0
         END IF
      END IF
      RETURN
C*********
      ENTRY PSGFIL(Cpsf1, Lpsf1)
C PSF in external file
      Cpsf1 = cpsf
      Lpsf1 = lpsf
      RETURN
C*********
      ENTRY PSGSLI(isliceo)
C Last slice
      IF ( lpsf.GT.0 .AND. lslice.GT.0 ) THEN
         isliceo = lslice
      ELSE
         isliceo = 0
      END IF
      RETURN
C*********
      ENTRY PSQMO(Ipsfmi)
      Ipsfmi = ipsfm
      RETURN
C*********
      ENTRY PSQMOC(cpmod)
      IF ( ipsfm.EQ.-2 ) THEN
         cpmod = 'CHan'
      ELSE IF ( ipsfm.EQ.1 ) THEN
         cpmod = 'Cgau'
      ELSE IF ( ipsfm.EQ.2 ) THEN
         cpmod = 'Egau'
      ELSE
         cpmod = 'Moff'
      END IF
      RETURN
C*********
      ENTRY PSQPAR(Npar, Pars)
      Npar=0
      IF ( ipsfm.EQ.2 ) THEN
         Npar=2
         Pars(1)=ecc
         Pars(2)=theta
      ELSE IF ( ipsfm.EQ.3 ) THEN
         Npar=1
         Pars(1)=p4
      END IF
      RETURN
C*********
      ENTRY PSSFIL(Cpsf1, Lpsf1)
C PSF in external file
      cpsf = Cpsf1
      lpsf = Lpsf1
      IF ( lunpsf.GT.0 ) THEN
         istat = 0
         CALL FTCLOS(lunpsf, istat)
         CALL FRELUN(lunpsf)
         lunpsf = 0
      END IF
      lslice = 0
      RETURN
C*********
      ENTRY PSSMO(Ipsfmi)
      ipsfm = Ipsfmi
      RETURN
C*********
      ENTRY PSSMOC(cpmod)
      ctmp = cpmod(:2)
      CALL UPC(ctmp(:2))
      IF ( ctmp(:2).EQ.'CH' ) THEN
         ipsfm = -2
      ELSE IF ( ctmp(:1).EQ.'C' ) THEN
         ipsfm = 1
      ELSE IF ( ctmp(:1).EQ.'E' ) THEN
         ipsfm = 2
      ELSE IF ( ctmp(:1).EQ.'M' ) THEN
         ipsfm = 3
      ELSE
         WRITE(*,*) 'Possible choices are Cgau, Egau, CHandra, Moff.'
      END IF
      RETURN
C*********
      ENTRY PSSPAR(cpar, lpar)
C PSF parameters
      kp=0
      IF ( ipsfm.EQ.2 ) THEN
         CALL ALF(cpar, lpar, kp, ctmp, ltmp)
         IF ( ltmp.GT.0 ) THEN
            tmp=FPNUM(ctmp, ltmp, ier)
            IF ( 0.0.LE.tmp .AND. tmp.LT.1.0 ) ecc=tmp
         END IF
         CALL ALF(cpar, lpar, kp, ctmp, ltmp)
         IF ( ltmp.GT.0 ) THEN
            tmp=FPNUM(ctmp, ltmp, ier)
            IF ( -90.0.LE.tmp .AND. tmp.LT.90.0 ) theta=tmp
         END IF
      ELSE IF ( ipsfm.EQ.3 ) THEN
         CALL ALF(cpar, lpar, kp, ctmp, ltmp)
         IF ( ltmp.GT.0 ) THEN
            tmp=FPNUM(ctmp, ltmp, ier)
            IF ( 0.0.LE.tmp .AND. tmp.LT.20.0 ) p4=tmp
         END IF
      END IF
      RETURN
C
      END
