      REAL FUNCTION rmean(x0, y0, d)
      REAL   x0, y0, d
C---
C Assume a square pixel with a side of length of d.  Assume center
C of pixel is located at (x0, y0).  Return in rmean the mean
C value of r averaged over the pixel assuming all points in the
C pixel are weighted equally.  This expression was produced with
C Mathematica.
C---
C x0      I    X position of center
C y0      I    Y position of center
C d       I    size of pixel
C----
C 1999-Jan-20 - Fix so that roundoff cannot cause a LOG(0) - [AFT]
C 1997-May-30 - New routine - [AFT/RFE/Mathematica]
C---
      REAL    tmp1, tmp2, xl, xr, yb, yt
C---
      xl = x0 - d/2.
      xr = x0 + d/2.
      yb = y0 - d/2.
      yt = y0 + d/2.
C
      rmean = -2*xr*yb*SQRT(xr*xr + yb*yb)
     a        -2*xl*yt*SQRT(xl*xl + yt*yt)
     a        +2*xl*yb*SQRT(xl*xl + yb*yb)
     a        +2*xr*yt*SQRT(xr*xr + yt*yt)
C Clearly if yb.EQ.0.0 then SQRT(xl*xl)=ABS(xl) and if xl.LT.0.0
C the argument to the LOG is zero.  Less obvious, on some systems
C roundoff error will cause SQRT(xl*xl+yb*yb)=ABS(xl).  I've seen
C this happen on a Solaris when xl=-265 and yb=0.1.  The same values
C did not cause a problem using g77 on an Intel based OpenStep system.
C This can only happen if yb is small, hence the yb**3 factor will
C make value of the term relatively small, hence it is same to neglect.
      tmp1 = xl + SQRT(xl*xl + yb*yb)
      tmp2 = xr + SQRT(xr*xr + yb*yb)
      IF( tmp1.GT.0.0 .AND. tmp2.GT.0.0 ) THEN
         rmean = rmean + yb*yb*yb*(
     a         LOG(tmp1)
     a        -LOG(tmp2))
      END IF
      tmp1 = yt + SQRT(xr*xr + yt*yt)
      tmp2 = yb + SQRT(xr*xr + yb*yb)
      IF( tmp1.GT.0.0 .AND. tmp2.GT.0.0 ) THEN
         rmean = rmean + xr*xr*xr*(
     a         LOG(tmp1)
     a        -LOG(tmp2))
      END IF
      tmp1 = xr + SQRT(xr*xr + yt*yt)
      tmp2 = xl + SQRT(xl*xl + yt*yt)
      IF( tmp1.GT.0.0 .AND. tmp2.GT.0.0 ) THEN
         rmean = rmean + yt*yt*yt*(
     a         LOG(tmp1)
     a        -LOG(tmp2))
      END IF
      tmp1 = yb + SQRT(xl*xl + yb*yb)
      tmp2 = yt + SQRT(xl*xl + yt*yt)
      IF( tmp1.GT.0.0 .AND. tmp2.GT.0.0 ) THEN
         rmean = rmean + xl*xl*xl*(
     a         LOG(tmp1)
     a        -LOG(tmp2))
      END IF
C
      rmean = rmean/(6.0*d**2)
C
      RETURN
      END
