      SUBROUTINE SFITN(Qplot, Mxrow, Aray, Nrow, Ncol, Xmin, Xmax,
     &   Cfree, Nsour, Xsour, Ysour,
     &   Mxpar,Npars,Pval,Plim,Icomp,Nterm,Nfit,Xchi,Ier)
      LOGICAL   Qplot
      INTEGER   Mxrow, Nrow, Ncol
      INTEGER   Nsour, Mxpar, Npars, Icomp(*), Nterm, Nfit, Ier
      CHARACTER Cfree*(*)
      REAL      Aray(*), Xmin(2), Xmax(2), Xsour(*), Ysour(*)
      REAL      Pval(Mxpar), Plim(3,Mxpar), Xchi
C Entry SFITNS/SFITNQ
      INTEGER   ipstat
C---
C Fit N sources with positions in the Xsour, Ysour arrays to gaussians.
C This code assumes that the model parameters do not change
C significantly across the field being fit.  Thus, except for position
C and norms, all other parameters default to being tied to the value
C for the first source.  Thus all widths are linked.  This helps prevent
C one source from moving over and modeling the wings of a bright nearby
C source.
C---
C Qplot   I
C Mxrow   I
C Aray    I
C Nrow    I
C Ncol    I
C Xmin    I
C Xmax    I
C Cfree   I    any of N norm, P position, W width
C Nsour   I
C Xsour   I
C Ysour   I
C Mxpar   I
C Npars   I    The number of parameters per comp already set.
C Pval    I/O
C Plim    I/O
C Icomp     O
C Nterm     O
C Nfit      O  The number of points included in the fit
C Xchi      O  The final Chi^2
C Ier       O  =0 fits were OK, <>0 had a problem
C---
C 2002-Mar-27 - [AFT]
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INTEGER   MXLOC
      PARAMETER (MXLOC=120)
      REAL      WEIGHT
C
      CHARACTER cmod*256
      CHARACTER ctmp*32
      CHARACTER ctyp(7,3)
      DOUBLE PRECISION dsum
      REAL      pslim(3,MXLOC)
      REAL      xt(2)
      REAL      ang, r2, rsig, tmp, wtmp, sigx, sigy, xcen, ycen
      INTEGER   ibeg, ichat, icmd, ihit, niter
      INTEGER   i, idoun, iegood, iery, igroup, insen, ipass, ipsfm
      INTEGER   isgood, isour
      INTEGER   istat, ixmap, iymap, iyi, iyl, iyoff, iy0
      INTEGER   k, lmod, ndim, nidim
C The number of parameters for the stellar component.
      INTEGER   nparc
C The default fit statistic (ML)
      INTEGER   idstat
      SAVE      idstat
      DATA      idstat/1/
      DATA ctyp/'P', 'P', 'W', 'N', ' ', ' ', ' ',
     &          'P', 'W', 'P', 'W', 'W', 'N', ' ',
     &          'P', 'P', 'W', 'W', 'N', ' ', ' '/
C
      CALL PSQMO(ipsfm)
      Ier = 0
      igroup = 1
      iy0 = 0
C
      IF ( Qplot) THEN
         CALL PGSFS(2)
         CALL PGSCI(2)
         CALL PGRECT(xmin(1),xmax(1),xmin(2),xmax(2))
      END IF
C
C Count the number of points in the region and compute some other statistics.
      Isgood=0
      Iegood=0
      Nfit = 0
      dsum = 0.0d0
      CALL PLTXCC(Aray, 1, igroup, xt, nidim, iyoff)
      DO 190 I=1,Nrow*Ncol
         CALL PLTXCC(Aray, i, igroup, xt, ndim, iyoff)
         IF ( xt(1).EQ.NO ) GOTO 190
         IF ( xt(1).LT.Xmin(1) .OR. xt(1).GT.Xmax(1) ) GOTO 190
         IF ( nidim.GT.1 .AND.
     &        xt(2).LT.Xmin(2) .OR. xt(2).GT.Xmax(2) ) GOTO 190
         iyi = iy0 + iyoff
         wtmp = WEIGHT(Aray(iyi), Mxrow, 0)
         IF ( wtmp.LE.0. ) GOTO 190
         IF ( Isgood.EQ.0 ) Isgood=I
         Iegood=I
         Nfit=Nfit+1
         dsum = dsum + aray(iyi)
  190 CONTINUE
      IF ( Nfit.LE.0 ) THEN
         CALL PTBUF('SFITN--No points in allowed range.',-1)
         Ier = 1
         RETURN
      END IF
C
C We will use the sigma at the center of the box
      xcen = (Xmin(1)+Xmax(1))/2.
      ycen = (Xmin(2)+Xmax(2))/2.
      CALL PSFSIG(xcen, ycen, rsig)
C---
C Define the model
      cmod = 'MOD CO /'
      lmod = 8
      Nterm =1
      Plim(1,Nterm) = 0.0
      DO isour=1,nsour
         IF ( ipsfm.EQ.1 ) THEN
C NCG
            nparc = 4
            iyl = 2
            cmod = cmod(:lmod-1)//'NCG /'
            lmod = lmod + 4
            Plim(1,Nterm+1) = -1.0
            Plim(1,Nterm+2) = -1.0
            Plim(1,Nterm+3) = -1.0
            IF ( index(Cfree, 'N').NE.0 ) THEN
               Plim(1,Nterm+4) = 0.0
            ELSE
               Plim(1,Nterm+4) = -1.0
            END IF
            IF ( Npars.NE.nparc ) THEN
C X
               Pval(Nterm+1) = xsour(isour)
C Y
               Pval(Nterm+2) = ysour(isour)
C SIG
               Pval(Nterm+3) = rsig
               IF ( isour.NE.1 ) THEN
                  Plim(1,Nterm+3) = -(1+3)
               END IF
C Norm.  We base the initial guess on how far above background the
C center is.
               CALL POS_DT_IM(xsour(isour), ysour(isour), ixmap, iymap)
               tmp = MAX(0.1, aray(ixmap+Mxrow*(iymap-1)) - dsum/Nfit)
               Pval(Nterm+4) = tmp*6.28*rsig*rsig
               dsum = dsum - pval(Nterm+4)
            END IF
            Nterm = Nterm + nparc
         ELSE IF ( ABS(ipsfm).EQ.2 ) THEN
C NEG
            CALL PSGELL(xcen, ycen, sigx, sigy, ang)
            nparc = 6
            iyl = 3
            cmod = cmod(:lmod-1)//'NEG /'
            lmod = lmod + 4
            Plim(1,Nterm+1) = -1.0
            Plim(1,Nterm+2) = -1.0
            Plim(1,Nterm+3) = -1.0
            Plim(1,Nterm+4) = -1.0
            Plim(1,Nterm+5) = -1.0
            IF ( index(Cfree, 'N').NE.0 ) THEN
               Plim(1,Nterm+6) = 0.0
            ELSE
               Plim(1,Nterm+6) = -1.0
            END IF
            IF ( Npars.NE.nparc ) THEN
C X
               Pval(Nterm+1) = xsour(isour)
C SX
               Pval(Nterm+2) = sigx
C Y
               Pval(Nterm+3) = ysour(isour)
C SY
               Pval(Nterm+4) = sigy
C Theta
               Pval(Nterm+5) = ang
               IF ( isour.NE.1 ) THEN
C Tie width parameters to central source.
                  Plim(1,Nterm+2) = -(1+2)
                  Plim(1,Nterm+4) = -(1+4)
                  Plim(1,Nterm+5) = -(1+5)
               END IF
C Norm.  We base the initial guess on how far above background the
C center is.
               CALL POS_DT_IM(xsour(isour), ysour(isour), ixmap, iymap)
               tmp = MAX(0.1, aray(ixmap+Mxrow*(iymap-1)) - dsum/Nfit)
               Pval(Nterm+6) = tmp*6.28*sigx*sigy
               dsum = dsum - Pval(Nterm+6)
            END IF
            Nterm = Nterm + nparc
         ELSE IF ( ipsfm.EQ.3 ) THEN
C nmoffat normalized moffat function (COD file)
            nparc = 5
            iyl = 2
            cmod = cmod(:lmod-1)//'nmoffat /'
            lmod = lmod + 8
            Plim(1,Nterm+1) = -1.0
            Plim(1,Nterm+2) = -1.0
            Plim(1,Nterm+3) = -1.0
            Plim(1,Nterm+4) = -1.0
            IF ( index(Cfree, 'N').NE.0 ) THEN
               Plim(1,Nterm+5) = 0.0
            ELSE
               Plim(1,Nterm+5) = -1.0
            END IF
            IF ( Npars.NE.nparc ) THEN
C X
               Pval(Nterm+1) = xsour(isour)
C Y
               Pval(Nterm+2) = ysour(isour)
C
C Width (value is closer to FWHM)
               Pval(Nterm+3) = rsig
C Index
               Pval(Nterm+4) = 4.0
               IF ( isour.NE.1 ) THEN
                  Plim(1,Nterm+3) = -(1+3)
                  Plim(1,Nterm+4) = -(1+4)
               END IF
C Norm.  We base the initial guess on how far above background the
C center is.  Note, 4.19=4*pi/(p4-1) where P4=4.
               CALL POS_DT_IM(xsour(isour), ysour(isour), ixmap, iymap)
               tmp = MAX(0.1, aray(ixmap+Mxrow*(iymap-1)) - dsum/Nfit)
               Pval(Nterm+5) = tmp*4.19*rsig*rsig
               dsum = dsum - Pval(Nterm+5)
            END IF
            Nterm = Nterm + nparc
         END IF
      END DO
C For likelihood, const must be .GE.0.0
      IF ( Npars.NE.nparc )  Pval(1) = MAX(0.0,REAL(dsum/Nfit))
C Now define the model.  Since Model can thaw parameters, make sure
C this does not happen by passing in bogus pslim array.
      icmd = -1
      CALL MODEL(cmod, xmin, xmax, MXPAR, ctmp, 0, icmd,
     &    Icomp, Pval, Pslim, Nterm)
      IF ( nterm.LE.0 ) THEN
         CALL PTBUF('SFITN--No model defined.',-1)
         Ier = 1
         RETURN
      END IF
C
C Make sure parameters are legal
      CALL FITLIM(Icomp,Pval,Plim,Nterm,ihit)
C---
      iery  = 0
      ichat = -1
      niter = 50
      istat = idstat
      idoun = 0
      ipass=0
      insen=0
  200 CONTINUE
C      IF ( (xsour(1)-746.9)**2+(ysour(1)-1059.9)**2.LT.10.0 ) THEN
C         write(*,*) '! ipass=',ipass
C         ichat=1
C         do i=1,nterm
C            write(*,*) pval(i),(plim(k,i),k=1,3),' ! p', i
C         end do
C      END IF
      CALL CURFIT(Aray, igroup, iy0, iery, Mxrow, isgood, iegood,
     &  Xmin,Xmax,ichat,niter,istat,Icomp,idoun,Nterm,
     &  0.0, Pval,Plim,Xchi)
      IF ( xchi.LT.0.0 ) THEN
         IF ( insen.NE.0 ) THEN
C Cannot deal with being insensitive to two parameters.
            Ier = 2
            RETURN
         END IF
         insen=-NINT(Xchi)
C Freeze the insensitive parameter.
         Plim(1,insen)=-1.
         GOTO 200
      END IF
C
      IF ( ipass.EQ.0 ) THEN
C For this section we only free the primary source parameters, first
C the position and then the rest (currently only width)
         IF ( index(Cfree, 'P').NE.0 ) THEN
            IF ( Plim(1,2).LT.0 .OR. Plim(1,1+iyl).LT.0 ) THEN
               Plim(1,2)    =0.0
               Plim(1,1+iyl)=0.0
C Set ipass back to zero to ensure other parameters are thawed.
               ipass=0
               GOTO 200
            END IF
         END IF
C Thaw all parameters that user marked as free.
         DO i=1,nparc
            IF ( index(Cfree, ctyp(i,ABS(ipsfm))(1:1)).NE.0 ) THEN
               IF (Plim(1,i+1).LT.0 .AND. i+1.NE.insen ) THEN
                  ipass=1
                  Plim(1,i+1)=0.0
               END IF
            END IF
         END DO
         IF ( ipass.EQ.1 ) THEN
C All parameters are free, no need to return to this block.
            ipass=1
            GOTO 200
         END IF
      END IF
C
      IF ( index(Cfree, 'P').NE.0 ) THEN
C If positions are allowed to vary, we thaw the positions of nearby
C sources, i.e., only the ones that are likely to have an impact on
C the central source.
         ipass=1
         ibeg = 1
         DO i=1,nsour
            r2 = (xsour(1)-xsour(i))**2+(ysour(1)-ysour(i))**2
            IF ( r2.LT.9.*rsig*rsig ) THEN
               IF ( Plim(1,ibeg+1).LT.0.0 ) THEN
                  ipass=2
                  Plim(1,ibeg+1) = 0.0
                  Plim(1,ibeg+iyl) = 0.0
               END IF
            END IF
            ibeg = ibeg + nparc
         END DO
         IF ( ipass.EQ.2 ) THEN
            ipass=1
            GOTO 200
         END IF
      END IF
C
C Negative norms will cause a problem, so get rid of them.
      ipass=2
      ibeg = 1
      DO i=1,nsour
         IF ( Pval(ibeg+nparc).LE.0.0 ) THEN
            IF ( i.EQ.1 ) THEN
               WRITE(*,221) xsour(1),ysour(1)
  221          FORMAT('SFITN--Negative norm at',2F10.1)
               Ier = 1
               RETURN
            END IF
            CALL RMCOMP(ibeg, nparc, Pval, Plim, Nterm)
            nsour=nsour-1
            ipass=3
         END IF
         ibeg=ibeg+nparc
      END DO
      IF ( ipass.EQ.3 ) THEN
         ipass=2
         GOTO 200
      END IF
C
C Check to see if the position has moved a large amount.
      ipass=3
      ibeg = 1
      DO i=1,nsour
         IF ( Pval(ibeg+nparc).LE.0.0 .OR.
     &        ABS(Pval(ibeg+1)-xsour(i)).GT.7.0*rsig .OR.
     &        ABS(Pval(ibeg+iyl)-ysour(i)).GT.7.0*rsig ) THEN
            IF ( i.EQ.1 ) THEN
              WRITE(*,231) Xsour(i),Ysour(i),Pval(ibeg+1),Pval(ibeg+iyl)
  231          FORMAT('SFITN--Large shift from',2F10.1,' to',2F10.1)
C Large change for the main source, we give up.
               Ier = 1
               RETURN
            END IF
C If nearby source has moved, then we remove it and try again.
            CALL RMCOMP(ibeg, nparc, Pval, Plim, Nterm)
            nsour=nsour-1
            ipass=4
         END IF
         ibeg=ibeg+nparc
      END DO
      IF ( ipass.EQ.4 ) THEN
         ipass=3
         GOTO 200
      END IF
C
  900 CONTINUE
      IF ( Qplot ) THEN
         CALL PGSCI(3)
         CALL PGCIRC(pval(2), pval(1+iyl), 2.0*rsig)
      END IF
C---
      Ier = 0
      RETURN
C***
      ENTRY SFITNQ(ipstat)
      ipstat=idstat
      RETURN
C***
      ENTRY SFITNS(ipstat)
      idstat=ipstat
      RETURN

      END
C*********
      SUBROUTINE CPPARM(Pval, Plim, Nterm, Psval, Pslim)
      INTEGER   Nterm
      REAL      Pval(*), Plim(3,*), Psval(*), Pslim(3,*)
C---
C Save parameters.
C---
C Pval    I
C Plim    I
C Nterm   I
C Psval     O
C Pslim     O
C---
      INTEGER   i
C---
      DO i=1,Nterm
         Psval(i)   = Pval(i)
         Pslim(1,i) = Plim(1,i)
         Pslim(2,i) = Plim(2,i)
         Pslim(3,i) = Plim(3,i)
      END DO
C---
      RETURN
      END
C*********
      SUBROUTINE RMCOMP(Ibeg, Nparc, Pval, Plim, Nterm)
      INTEGER   Ibeg, Nparc, Nterm
      REAL      Pval(*), Plim(3,*)
C---
C Remove a component from the model parameters/limits.
C---
C Ibeg    I
C Nparc   I
C Pval    I/O
C Plim    I/O
C Nterm   I/O
C---
      INTEGER   i
C---
      DO i=Ibeg+1, Nterm-Nparc
         Pval(i)   = Pval(i+Nparc)
         Plim(1,i) = Plim(1,i+Nparc)
         Plim(2,i) = Plim(2,i+Nparc)
         Plim(3,i) = Plim(3,i+Nparc)
      END DO
      Nterm = Nterm - Nparc
C---
      RETURN
      END
