      SUBROUTINE SFMVUE(Rmap,Mxrow,Nrow,Ncol,Back,Lback,Qpoint,
     &      Mxray,Mxpsf,Brad,Srad,Sfparm,Box,Sorc,Sdev,Bgpix,Excess,
     &      Nmx,Nmy,Xpect,Npsf)
      INTEGER   Mxrow, Nrow, Ncol, Lback, Mxray, Mxpsf
      INTEGER   Nmx, Nmy, Npsf
      REAL      Rmap(Mxrow, *), Back(Mxrow, *), Brad, Srad, Sfparm(2)
      LOGICAL   Qpoint
      REAL      Sorc(Mxray,Mxray), Sdev(Mxray,Mxray)
      REAL      Bgpix(Mxray,Mxray), Excess(Mxray,Mxray)
      REAL      Xpect(-Mxpsf:Mxpsf,-Mxpsf:Mxpsf)
      REAL      Box(4)
C---
C Find sources using the minimum variance method.
C
C Assume that the number of counts at a position is C(i,j) can be modeled
C by S*P(i,j)+B where S is the total number of counts from the source
C P(i,j) is the PSF and B is the background in a single map pixel.
C This routine condiders the variance
C
C var = <(S*P+B-C)**2>
C
C and minimizes the variance by setting dvar/dS=0 and dvar/dB=0.  Basically
C this is doing a least squares fit to a straight line.
C The error comes from var(S) = (dS/dC)**2*var(C), where var(C)=C.
C dS/dC(i)=(N*P(i)-p1sum)/det
C dB/dC(i)=(p2sum-p(i)p1sum)/det
C Since we assume that var(Rmap) = Rmap this *requires* that Rmap be positive.
C---
C Rmap    I
C Mxrow   I
C Nrow    I
C Ncol    I
C Qpoint  I    Is one point being interrogated?
C Mxray   I    Maximum size of Sorc, Sdev, and Bgpix
C Mxpsf   I    Maximum size of Xpect
C Brad    I    Radius of background region (in units of PSF_sig)
C Srad    I    Radius of source region (in units of PSF_sig)
C Sfparm  I    Source finding parameters (PFrac, PThres)
C Box       O  Maximum size of region searched
C Sorc      O  output array of source estimators
C Sdev      O  output array of sd of above
C Bgpix     O  output array of bg estimators
C Excess    O  Number of sigma above background
C Nmx,Mny   O  Actual size of Sorc, Sdev and Bgpix
C Xpect     O  The last PSF computed
C Npsf      O  Actual size of Xpect
C---
C 2000-09-01 - New variance and general update [AFT]
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      INCLUDE 'posdef.inc'
C
      CHARACTER cout*132
      DOUBLE PRECISION dra, ddec
      REAL      p1sum, p2sum, pbsum, c1sum, pcsum, p2csum
      REAL      best, btmp, bvar, cthres, csumi, csumo, den
      REAL      psig, psumi, pthres
      REAL      sest, rsig, srat, svar
      REAL      xpos, xsum, xtmp, ypos, ysum, ytmp
      INTEGER   i1, ib2, icnt, icnti, icnto, id2, imap
      INTEGER   imxh, imxl, imyh, imyl, is2, israd
      INTEGER   jmap, j1, l, l1, lhi, llo, m, m1, mhi, mlo
C
C Find corners of analysis box in map coordinates
      CALL POS_DT_IM(Box(1),Box(2),imxl,imyl)
      imxl = MIN(MAX(1,imxl),Nrow)
      imyl = MIN(MAX(1,imyl),Ncol)
      CALL POS_DT_IM(Box(3),Box(4),imxh,imyh)
      imxh = MIN(MAX(1,imxh),Nrow)
      imyh = MIN(MAX(1,imyh),Ncol)
C
      DO imap=imxl, imxh
         DO jmap=imyl, imyh
            IF( Rmap(imap,jmap).NE.NO .AND. Rmap(imap,jmap).LT.0.0) THEN
               CALL PTBUF(
     & 'sfmvue--Data contains negative values, cannot use sfmvue.',-1)
               Npsf = -1
               RETURN
            END IF
         END DO
      END DO
C
      CALL GTFWAT(psig)
C
C Start loop over search box.
      DO imap=imxl, imxh
         DO 400 jmap=imyl, imyh
            i1 = imap - imxl + 1
            j1 = jmap - imyl + 1
            IF ( Rmap(imap,jmap).EQ.NO ) THEN
               IF ( .NOT.Qpoint ) THEN
                  Sorc(i1,j1) = NO
                  Sdev(i1,j1) = NO
                  Bgpix(i1,j1) = NO
                  Excess(i1,j1) = NO
               ELSE
                  CALL PTBUF('Sorry, no valid data at center.',-1)
               END IF
               GOTO 400
            END IF
C
C Find position of i,j in terms of detector coordinates
            CALL POS_MAP_DET(imap,jmap,xpos,ypos)
C
C Find local size of detection cell (Npsf is in map units).
            CALL PSFSIG(xpos, ypos, rsig)
            israd = MAX(1, NINT(Srad*rsig/Ifac))
            is2 = MAX(1,NINT((srad*rsig)**2/(Ifac*Ifac)))
            Npsf = MAX(israd+1,MAX(1, NINT(brad*rsig/Ifac)))
            ib2 = MAX(2,NINT((brad*rsig)**2/(Ifac*Ifac)))
C
C Set up array giving the weightings due to the PSF.
            CALL PSFSET(Ifac, xpos, ypos, Mxpsf, Npsf, Xpect)
C
C Initialize sums
            p1sum = 0.0
            p2sum = 0.0
            pbsum = 0.0
            c1sum = 0.0
C            c2sum = 0.0
            pcsum = 0.0
            p2csum = 0.0
            icnt = 0
            csumi = 0.0
            psumi = 0.0
            csumo = 0.0
            icnti = 0
            icnto = 0
C
C Loop over detection cell, calculating sums.
            llo = -Npsf - (MIN(imap-Npsf,1)-1)
            lhi =  Npsf - (MAX(imap+Npsf,Nrow)-Nrow)
            cthres = Rmap(imap,jmap) + 3.*SQRT(MAX(1.0,Rmap(imap,jmap)))
            pthres = Sfparm(2)*Xpect(0,0)
            DO l=llo, lhi
               l1 = imap + l
               mlo = -Npsf - (MIN(jmap-Npsf,1)-1)
               mhi =  Npsf - (MAX(jmap+Npsf,Ncol)-Ncol)
               DO m=mlo, mhi
                  m1 = jmap + m
                  id2 = l*l + m*m
                  IF ( Rmap(l1,m1).NE.NO .AND. id2.LE.ib2 .AND.
     &         (Xpect(l,m).GE.pthres .OR. Rmap(l1,m1).LE.cthres) ) THEN
                     IF ( id2.LE.is2 ) THEN
                        icnti = icnti + 1
                        csumi = csumi + Rmap(l1,m1)
                        psumi = psumi + Xpect(l,m)
                     ELSE
                        icnto = icnto + 1
                        csumo = csumo + Rmap(l1,m1)
                     END IF
                     IF(lback.GT.0) pbsum=pbsum+Xpect(l,m)*Back(l1,m1)
                     p1sum = p1sum + Xpect(l,m)
                     p2sum = p2sum + Xpect(l,m)**2
                     c1sum = c1sum + Rmap(l1,m1)
C                     c2sum = c2sum + Rmap(l1,m1)*Rmap(l1,m1)
                     pcsum = pcsum + Xpect(l,m)*Rmap(l1,m1)
                     p2csum= p2csum+ Xpect(l,m)*Xpect(l,m)*Rmap(l1,m1)
                     icnt = icnt + 1
                  END IF
               END DO
            END DO
C
C If less than the requested fraction of the bins were included then give up.
            IF ( icnt.LT.Sfparm(1)*(2*Npsf+1)*(2*Npsf+1) ) THEN
               IF ( .NOT.Qpoint ) THEN
                  Sorc(i1,j1) = NO
                  Sdev(i1,j1) = NO
                  Bgpix(i1,j1) = NO
                  Excess(i1,j1) = NO
               ELSE
                  CALL PTBUF('Sorry, not enough data points in map.',-1)
               END IF
               GOTO 400
            END IF
C
            IF ( lback.GT.0 ) THEN
C Force background to match what's in the Back array.
               sest = (pcsum-pbsum)/p2sum
               svar = pcsum/p2sum
               best = Back(imap,jmap)
               bvar = 0.0
            ELSE
               den = icnt*p2sum - p1sum**2
               sest = (icnt*pcsum-c1sum*p1sum)/den
C Martin's variance, if svar is approximately zero, round off error
C can make number negative.
               svar = MAX(1.e-10,(icnt*icnt*p2csum-2.*icnt*p1sum*pcsum+
     &          p1sum*p1sum*c1sum)/(den*den))
               best = (p2sum*c1sum-p1sum*pcsum)/den
               bvar = (c1sum*p2sum*p2sum-2.*p1sum*p2sum*pcsum+
     &          p1sum*p1sum*p2csum)/(den*den)
            END IF
            IF ( Qpoint ) THEN
C
               xsum = 0.0
               ysum = 0.0
               llo = -israd - (MIN(imap-israd,1)-1)
               lhi =  israd - (MAX(imap+israd,Nrow)-Nrow)
               DO l=llo, lhi
                  l1 = imap + l
                  mlo = -israd - (MIN(jmap-israd,1)-1)
                  mhi =  israd - (MAX(jmap+israd,Ncol)-Ncol)
                  DO m=mlo, mhi
                     m1 = jmap + m
                     id2 = l*l+m*m
C      write(*,*) Xpect(l,m), Rmap(l1,m1)
                     IF ( Rmap(l1,m1).NE.NO .AND. id2.LE.is2 ) THEN
                        xsum = xsum + l*Rmap(l1,m1)
                        ysum = ysum + m*Rmap(l1,m1)
                     END IF
                  END DO
               END DO
               xtmp = xpos + Ifac*xsum/csumi
               ytmp = ypos + Ifac*ysum/csumi
C               srat = (csumi - best*icnti)/psumi
               srat = (csumi - csumo*icnti/real(icnto))/psumi
C
C If only one point asked for calculate upper limit and dump out all data
               CALL PTBUF(' ', 1)
               CALL POS_DT_RA(xtmp,ytmp,dra,ddec)
               WRITE(cout,231) xpos, ypos, sest/SQRT(svar)
 231           FORMAT(' Center  : ',F8.1,1X,F8.1,'; SNR = ',F7.2)
               CALL PTBUF(cout, -1)
               WRITE(cout,241) xtmp, ytmp, srat, dra, ddec
 241           FORMAT(' Position: ',F8.1,1X,F8.1,'; counts=',F10.1,
     &            '; World=',F10.5,',',F9.5)
               CALL PTBUF(cout, -1)
            WRITE(cout,251) sest,SQRT(svar),best/(Ifac*Ifac),SQRT(bvar)
 251           FORMAT(' Sest= ',F10.1,', sig(S)=',F8.1,
     &            ', bgdp=',F9.3,', sig(B)=',F9.3)
               CALL PTBUF(cout, -1)
               WRITE(cout,261) rsig, israd*Ifac, Npsf*Ifac, psumi
 261           FORMAT(' PSF_sig = ',F7.2,', Srad=',I5,
     &            ' dpix, Brad=',I5,' dpix, psumi=',F7.3)
               CALL PTBUF(cout, -1)
               CALL PTBUF(' ', -1)
            ELSE
C
C Place data in arrays
               Sorc(i1,j1) = sest
C Note, svar is calculated assuming integer counts.  If image does not
C contain counts, then svar can be negative.
               Sdev(i1,j1) = SQRT(svar)
               Bgpix(i1,j1) = best/(Ifac*Ifac)
C Excess is the number of sigma above background.  This effectively
C uses the lextrct stat command on inner and outer circular regions.
C The flaw is this method is affected by single counts near the edge
C of the region and is not centrally peaked.
C               btmp = icnti*csumo/icnto
C               Excess(i1,j1) = (csumi-back)/SQRT(MAX(1.0,back))
C Excess source counts in the inner region divided by SQRT(back).
C back is the estimated background counts in the inner region and is
C estimated by the total counts in region (inner plus outer) minus
C source counts.
               btmp = icnti*(csumi+csumo-sest)/(icnti+icnto)
               Excess(i1,j1) = psumi*sest/SQRT(MAX(1.0,btmp))
            END IF
C
  400    CONTINUE
      END DO
C
      RETURN
      END
