      SUBROUTINE SORPLOT(Qplot)
      LOGICAL   Qplot
C---
C Subroutine to plot out source positions read in from an external file
C this either reads in a .SOR file which is produced by the MVUE source
C finding routines, or a .POS file which is a free-format tagged field
C list.  The tags can be:
C  Box:          ! to denote corners of box
C  DRA, DDEC:    ! Decimal RA/Dec
C  RA, DEC:      ! RA and Dec in 'h m s d m s'
C  Pos:          ! Position in detector coordinates.
C---
C Qplot   I    Is plotting device active?
C---
      REAL      NO
      PARAMETER (NO=-1.2E-34)
      REAL      FPNUM
      INTEGER   INBOU, ISNUM, LENACT
C Number of possible labels for SKY coordinates, and total to consider.
      INTEGER   NSKY, NCOORDS
      PARAMETER (NSKY=3, NCOORDS=5)
C
      CHARACTER cxnam(NCOORDS)*8, cynam(NCOORDS)*8
      CHARACTER cbuf*132
      CHARACTER cfile*256, ctmp*64
      CHARACTER ccol*16, cnam*16, cnul*16
C
      DOUBLE PRECISION ddec, dnul, dra
      REAL      ang, fjust, prad, rdecs, rnul, rras
      REAL      sch, schold, sigx, sigy, xsigma, xtmp, ytmp
      INTEGER   naxes(99)
      INTEGER   i, ibeg, ibits, iblk, ici, icisav, idcol, idecd, idecm
      INTEGER   ier, iext, ifits, igcnt, ilw, iname, incol
      INTEGER   ipcnt, iccol, iecol, ira, irah, iram
      INTEGER   ircol, isig, isky, isn, istat, isxcol, isycol
      INTEGER   ithcol, itmp, iworld
      INTEGER   kp, lbuf, lun, lfile, lnam, ltmp, naxis, nrows
      LOGICAL   qbad, qsimp, qxtend
      DATA cxnam/ 'RA','RAJ2000','RA(ICRS)','X','X_IMAGE'/
      DATA cynam/'DEC','DEJ2000','DE(ICRS)','Y','Y_IMAGE'/
C
   11 FORMAT (A)
C
      CALL POS_SKY(isky)
  100 CONTINUE
      CALL GTPEEK(ctmp,ltmp)
      IF ( ltmp.LE.0 ) THEN
         CALL GTBUF('Position file, [size], [ci]:', ier)
         IF ( ier.NE.0 ) GOTO 990
      END IF
      lfile = 0
      CALL GTCHAR(cfile, lfile)
      IF ( lfile.EQ.0 ) GOTO 990
      CALL GTCHAR(cbuf, lbuf)
      prad = 10.0
      iname = 0
      isig = 0
      cnam = ' '
      IF ( lbuf.GT.0 ) THEN
         itmp = INDEX(cbuf,'_') - 1
         IF ( itmp.LE.0 ) itmp = lbuf
         IF ( ISNUM(cbuf,itmp).NE.0 ) THEN
            CALL UPC(cbuf(:lbuf))
            isig = INDEX(cbuf,'_SIG')
            IF ( isig.EQ.0 ) THEN
               CALL RGGTDI(cbuf, lbuf, prad, ier)
            ELSE
               prad = 1.0
               xsigma = FPNUM(cbuf, isig-1, ier)
            END IF
         ELSE
            cnam = cbuf
            lnam = lbuf
            iname = 1
         END IF
      END IF
      CALL PGSFS(2)
C
      CALL PGQCI(icisav)
      ici = icisav
      CALL GTINT(ici, ier)
      CALL PGSCI(ici)
C
      ang = 0.0
      fjust = 0.0
      CALL GTREAL(ang, ier)
      CALL GTREAL(fjust, ier)
      sch = 0.6
      CALL GTREAL(sch, ier)
      CALL PGQCH(schold)
      CALL PGSCH(sch)
C
      CALL GETLUN(lun)
      istat = 0
      CALL FTOPEN(Lun, cfile, 0, iblk, istat)
      CALL FTGHPR(lun,99,qsimp,ibits,naxis,naxes,ipcnt,igcnt,qxtend,
     &            istat)
      IF ( istat.EQ.0 ) THEN
         ifits = 1
      ELSE
         istat = 0
         CALL FTCLOS(lun, istat)
         ifits = 0
         CALL OPENWR(lun,cfile,'OLD',' ',' ',0,1,ier)
         IF ( ier.NE.0 ) THEN
            WRITE (*,*) 'Failed to open file ', cfile
            WRITE (*,*) 'IOSTAT from OPEN is', ier
            CALL FRELUN(lun)
            GOTO 100
         END IF
      END IF
C
      IF ( ifits.NE.0 ) THEN
C FITS file.  Go to 2nd HDU.
         istat = 0
         CALL FTMAHD(lun, 2, iext, istat)
         CALL FTGKYJ(lun, 'NAXIS2', nrows, ctmp, istat)
C Check to see if SIG or SIGX,SIGy,THETA are in the file.
         iccol=0
         iecol=0
         istat = 0
         CALL FTGCNN(lun, .FALSE., 'SIGX', ccol, isxcol, istat)
         CALL FTGCNN(lun, .FALSE., 'SIGY', ccol, isycol, istat)
         CALL FTGCNN(lun, .FALSE., 'THETA',ccol, ithcol, istat)
C Only if all three columns are defined
         IF ( istat.EQ.0 ) THEN
C must be elliptical
            iecol=1
         ELSE
C otherwise try circular
            istat=0
            CALL FTGCNN(lun, .FALSE., 'SIG', ccol, iccol, istat)
            IF ( istat.NE.0 ) THEN
               istat=0
               CALL FTGCNN(lun, .FALSE., 'PSF_SIG', ccol, iccol, istat)
            END IF
         END IF
C
         IF ( isky.EQ.0 ) THEN
C Don't consider RA/Dec if we cannot deal with sky coordinates
            ibeg=NSKY+1
         ELSE
            ibeg=1
         END IF
         DO i=ibeg,NCOORDS
            istat = 0
            CALL FTGCNN(lun, .FALSE., cxnam(i), ccol, ircol, istat)
            CALL FTGCNN(lun, .FALSE., cynam(i), ccol, idcol, istat)
            IF ( istat.EQ.0 .OR. istat.EQ.237) GOTO 290
         END DO
         IF ( istat.NE.0 ) THEN
            WRITE(*,*) 'SORPLOT--Failed to find needed columns.'
            CALL FTCLOS(lun, istat)
            CALL FRELUN(lun)
            GOTO 100
         END IF
C---
  290    CONTINUE
         IF ( i.LE.3 ) THEN
C Got RA/Dec
            ira = 1
         ELSE
C Only found X/Y
            ira = 0
         END IF
C
         incol = 0
         IF ( iname.NE.0 ) THEN
            istat = 0
            CALL FTGCNN(lun, .FALSE., cnam, ccol, incol, istat)
            IF ( istat.NE.0 ) THEN
               CALL UPC(cnam)
               IF ( cnam.EQ.'#ROW') THEN
                  iname = -1
               ELSE
                  WRITE(*,*) 'SORPLOT--No column name=',cnam(:lnam)
                  iname = 0
               END IF
            END IF
         END IF
         dnul = NO
         rnul = NO
         cnul = ' '
         DO i=1,nrows
            istat = 0
            IF ( ira.NE.0 ) THEN
               CALL FTGCVD(lun,ircol,i,1,1,dnul,dra, qbad,istat)
               CALL FTGCVD(lun,idcol,i,1,1,dnul,ddec,qbad,istat)
               CALL POS_RA_DT(dra,ddec,xtmp,ytmp)
            ELSE
               CALL FTGCVE(lun,ircol,i,1,1,rnul,xtmp,qbad,istat)
               CALL FTGCVE(lun,idcol,i,1,1,rnul,ytmp,qbad,istat)
            END IF
            IF ( iname.NE.0 ) THEN
               IF ( INBOU(xtmp,ytmp).NE.0 ) THEN
                  IF ( iname.GT.0 ) THEN
                     CALL FTGCVS(lun,incol,i,1,1,cnul,ctmp,qbad,istat)
                     ltmp = LENACT(ctmp)
                  ELSE
                     ltmp = 0
                     ctmp = ' '
                     CALL CRAMI(i, ctmp, ltmp)
                  END IF
                  cbuf=' '//ctmp
                  CALL PGPTXT(xtmp, ytmp, ang, fjust, cbuf(:ltmp+1))
               END IF
            ELSE
               IF ( isig.EQ.0 ) THEN
                  CALL PGCIRC(xtmp, ytmp, prad)
               ELSE
                  IF ( iccol.NE.0 ) THEN
                     CALL FTGCVE(lun,iccol,i,1,1,rnul,prad,qbad,istat)
                     sigx=prad
                     sigy=prad
                     ang=0.0
                  ELSE IF ( iecol.NE.0 ) THEN
                     istat=0
                     CALL FTGCVE(lun,isxcol,i,1,1,rnul,sigx,qbad,istat)
                     CALL FTGCVE(lun,isycol,i,1,1,rnul,sigy,qbad,istat)
                     CALL FTGCVE(lun,ithcol,i,1,1,rnul,ang ,qbad,istat)
                  ELSE
                     CALL PSGELL(xtmp, ytmp, sigx, sigy, ang)
                  END IF
                  CALL PLELIP(xtmp, ytmp, xsigma*sigx, xsigma*sigy, ang)
               END IF
            END IF
         END DO
      ELSE
C
C Read and plot positions
C
         CALL SSGCLW(ici, ilw)
         CALL PGSCI(ici)
         DO 390 i=1, 9999
            READ(lun,11,END=900) cbuf
            lbuf = LENACT(cbuf)
            iworld = 0
            kp = 0
  310       CONTINUE
            IF ( kp.GE.lbuf ) GOTO 390
            CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
            IF ( ltmp.LE.0 ) GOTO 310
            CALL UPC(ctmp(:ltmp))
            IF ( ctmp(1:1).EQ.'W' ) THEN
               iworld = 1
               GOTO 310
            ELSE IF ( ctmp(1:1).EQ.'P' ) THEN
               iworld = 0
               GOTO 310
            END IF
C We skip leading non-numeric tokens.
            IF ( ISNUM(ctmp,ltmp).EQ.0 ) GOTO 310
C
            IF ( iworld.EQ.0 ) THEN
C Pixels are easy.
               xtmp = FPNUM(ctmp, ltmp, ier)
               CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
               ytmp = FPNUM(ctmp, ltmp, ier)
            ELSE
C RA
               IF ( INDEX(ctmp(:ltmp),'.').GT.0 ) THEN
C Decimal degrees
                  dra = FPNUM(ctmp, ltmp, ier)
               ELSE
C Hours, mins, sec
                  isn = +1
                  irah = FPNUM(ctmp, ltmp, ier)
                  CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
                  iram = FPNUM(ctmp, ltmp, ier)
                  CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
                  rras = FPNUM(ctmp, ltmp, ier)
                  CALL POS_HMS_DEG(irah,iram,rras,isn,idecd,idecm,rdecs,
     &             dra,ddec)
               END IF
C Dec
               CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
               IF ( INDEX(ctmp(:ltmp),'.').GT.0 ) THEN
C Decimal degrees
                  ddec = FPNUM(ctmp, ltmp, ier)
               ELSE
C Degrees, mins, sec
                  IF ( ctmp(1:1).EQ.'-' ) THEN
                     isn=-1
                  ELSE
                     isn=+1
                  END IF
                  idecd = FPNUM(ctmp, ltmp, ier)
                  CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
                  idecm = FPNUM(ctmp, ltmp, ier)
                  CALL ALF(cbuf, lbuf, kp, ctmp, ltmp)
                  rdecs = FPNUM(ctmp, ltmp, ier)
                  CALL GTREAL(rdecs,ier)
                  CALL POS_HMS_DEG(irah,iram,rras,isn,idecd,idecm,rdecs,
     &             dra,ddec)
               END IF
               CALL POS_RA_DT(dra,ddec,xtmp,ytmp)
            END IF
            IF ( isig.NE.0 ) THEN
               CALL PSFSIG(xtmp, ytmp, prad)
               prad = xsigma*prad
            END IF
            CALL PGCIRC(xtmp, ytmp, prad)
  390    CONTINUE
      END IF
C
C Close input file
C
  900 CONTINUE
      IF ( ifits.NE.0 ) THEN
         CALL FTCLOS(lun, istat)
      ELSE
         CLOSE(UNIT=lun)
      END IF
      CALL FRELUN(lun)
C Restore original values
      CALL PGSCI(icisav)
      CALL PGSCH(schold)
C
  990 CONTINUE
C
      RETURN
      END
