# XANADU programming stuff (for csh/tcsh users)
# You must run qinitu before calling this routine.

# For each system, define FCOMPL, FCOPTS, CCOPTS, PGLIBS, and SYSLIB as needed.

if ( "$EXT" == "lnx" ) then
   setenv CCOPTS "-I${XANTOP}/xanlib/cfortran -Df2cFortran"
   if ( -z "$FCOMPL" ) then
      setenv FLOPTS " "
      if ( "`whereis gfortran | sed -e 's/.*://'`" != "" ) then
         setenv FCOMPL gfortran
         setenv FCOPTS "-fimplicit-none -fno-second-underscore -fno-backslash"
      else if ( "`whereis g95           | sed -e 's/.*://'`" != "" ) then
         setenv FCOMPL g95
         setenv FCOPTS "-Wimplicit-none -fno-second-underscore -fno-backslash"
      else if ( "`whereis g77      | sed -e 's/.*://'`" != "" ) then
         setenv FCOMPL g77
         setenv FCOPTS "-O -Wimplicit -fno-second-underscore -fno-backslash"
      else
# Fall back to f77 with f2c options
         setenv FCOMPL f77
         setenv FCOPTS "-O -u -b"
      endif
      if ( "$FCOMPL" == "g95" || "$FCOMPL" == "gfortran" ) then
         setenv FCOPTS "-O -fimplicit-none -fno-second-underscore -fno-backslash"
         if ( "`uname -s`" == "Darwin" ) then
# We have a Mac, assume using 32 bit HEASOFT...
            setenv CCOPTS "$CCOPTS -arch i386"
            setenv FCOPTS "$FCOPTS -m32"
            setenv FLOPTS "$FLOPTS -m32"
         endif
      endif
   endif
   setenv CCOMPL gcc
   setenv RANLIB ranlib
   if ( `arch` == "x86_64" ) then
      if ( -d /usr/lib/x86_64-linux-gnu ) then
# Ubuntu style
         set LTOP=/usr/lib/x86_64-linux-gnu
      else
# Red Hat style
         set LTOP=/usr/lib64
      endif
   else
      set LTOP=/usr/lib
   endif
   if ( -z "$LIBPNG" ) then
      if ( -f $LTOP/libpng.so ) then
         setenv LIBPNG "-lpng -lz"
      else if ( -f $LTOP/libpng12.so ) then
         setenv LIBPNG "-lpng12 -lz"
      else
         setenv LIBPNG ""
      endif
   endif
   if ( -d /usr/X11R6/lib ) then
# Old X11
      setenv PGLIBS "-L${XANSYS}/lib -lpgplot -L/usr/X11R6/lib -lX11 $LIBPNG"
   else
# New X11
      setenv PGLIBS "-L${XANSYS}/lib -lpgplot -L$LTOP -lX11 $LIBPNG"
   endif
   setenv SYSLIB " "

else if ( "$EXT" == "mac" ) then
   setenv FCOMPL g77
   setenv FCOPTS "-O -Wimplicit -fno-second-underscore -fno-backslash"
   setenv FLOPTS " "
   setenv CCOMPL gcc
   setenv CCOPTS "-I${XANTOP}/xanlib/cfortran -Df2cFortran"
   setenv RANLIB ranlib
   if ( -d /usr/X11R6 ) then
      setenv PGLIBS "-L${XANSYS}/lib -lpgplot -L/usr/X11R6/lib -lX11"
   else
      setenv PGLIBS "-L${XANSYS}/lib -lpgplot -lpng -lz"
   endif
   setenv SYSLIB "-lcc_dynamic"

else if ( "$EXT" == "sol" ) then
   setenv MAKE "/usr/ccs/bin/make"
   setenv FCOMPL "g77"
   setenv FCOPTS "-O -Wimplicit -fno-second-underscore -fno-backslash"
   setenv FLOPTS " "
   setenv CCOMPL "gcc"
   setenv CCOPTS "-g -O2 -DHAVE_DIRENT_H -DHAVE_STRING_H -INONE -LNONE -Dsolaris -Wl,-R/opt/SUNWspro/lib:/opt/SUNWspro/SC4.0/lib:/usr/ccs/lib:/usr/lib:${XANSYS}/lib -DEXT=sol -I$XANTOP/xanlib/cfortran -I$XANSYS/include"
   setenv RANLIB "ranlib"
   setenv PGLIBS "$XANSYS/lib/libpgplot.a -lX11"
   setenv SYSLIB "-lsocket -lnsl"

else if ( "$EXT" == "sun" ) then
   setenv MAKE "/usr/bin/make"
   setenv FCOMPL "f77"
   setenv FCOPTS "-O -f"
   setenv FLOPTS " "
   setenv CCOMPL "gcc"
   setenv CCOPTS "-g -O2 -DHAVE_DIRENT_H -DHAVE_STRING_H -INONE -LNONE -DEXT=sun -I$XANTOP/xanlib/cfortran -I$XANSYS/include"
   setenv RANLIB "ranlib"
   setenv PGLIBS "$XANSYS/lib/libpgplot.a -LNONE -LNONE -lX11"
   setenv SYSLIB "-L/usr/lang/SC1.0/cg87 -L/usr/lang/SC1.0 -ltermcap -ldl -lF77 -lm -lc -LNONE -lX11"

else
   echo Unknown system.  Using generic settings.
   setenv FCOMPL f77
   setenv CCOMPL cc
   setenv RANLIB ranlib

endif


# Add $XANTOP/tools to PATH
if ( "`echo $PATH | grep $XANTOP/tools`" == "" ) then
   setenv PATH $XANTOP/tools:$PATH
endif

# If there is a local initp file, then source it.
if ( -f $XANTOP/local/tools/initp.csh ) then
   source $XANTOP/local/tools/initp.csh
endif
