# XANADU programming stuff (for sh/bash/ksh users)
# You must run qinitu before calling this routine.

# For each system, define FCOMPL, FCOPTS, CCOPTS, PGLIBS, and SYSLIB as needed.

if    test "$EXT" = "lnx" ; then
   CCOPTS="-I${XANTOP}/xanlib/cfortran -Df2cFortran"
   if test -z "$FCOMPL" ; then
      FLOPTS=" "
      if    test "`whereis gfortran | sed -e 's/.*://'`" != "" ; then
         FCOMPL=gfortran
         FCOPTS="-fimplicit-none -fno-second-underscore -fno-backslash"
      elif test "`whereis g95       | sed -e 's/.*://'`" != "" ; then
         FCOMPL=g95
         FCOPTS="-Wimplicit-none -fno-second-underscore -fno-backslash"
      elif    test "`whereis g77    | sed -e 's/.*://'`" != "" ; then
         FCOMPL=g77
         FCOPTS="-O -Wimplicit -fno-second-underscore -fno-backslash"
      else
# Fall back to f77 with f2c options
         FCOMPL=f77
         FCOPTS="-O -u -b"
      fi
      if test "$FCOMPL" = "g95" -o "$FCOMPL" = "gfortran" ; then
         if test "`uname -s`" = "Darwin" ; then
# We have a Mac, assume using 32 bit HEASOFT...
            CCOPTS="$CCOPTS -arch i386"
            FCOPTS="$FCOPTS -m32"
            FLOPTS="$FLOPTS -m32"
         fi
      fi
   fi
   CCOMPL=gcc
   RANLIB=ranlib
   if test `arch` = "x86_64" ; then
      if test -d /usr/lib/x86_64-linux-gnu ; then
# Ubuntu style
         LTOP=/usr/lib/x86_64-linux-gnu
      else
# Red Hat style
         LTOP=/usr/lib64
      fi
   else
      LTOP=/usr/lib
   fi
   if test -z "$LIBPNG" ; then
      if test -f $LTOP/libpng.so ; then
         LIBPNG="-lpng -lz"
      elif test -f $LTOP/libpng12.so ; then
         LIBPNG="-lpng12 -lz"
      else
         LIBPNG=""
      fi
   fi
   if test -d /usr/X11R6/lib ; then
# Old X11
      PGLIBS="-L${XANSYS}/lib -lpgplot -L/usr/X11R6/lib -lX11 $LIBPNG"
   else
# New X11
      PGLIBS="-L${XANSYS}/lib -lpgplot -L$LTOP -lX11 $LIBPNG"
   fi
   SYSLIB=" "

elif    test "$EXT" = "mac" ; then
   FCOMPL=g77
   FCOPTS="-O -Wimplicit -fno-second-underscore -fno-backslash"
   FLOPTS=" "
   CCOMPL=gcc
   CCOPTS="-I${XANTOP}/xanlib/cfortran -Df2cFortran"
   RANLIB=ranlib
   if test -d /usr/X11R6 ; then
      PGLIBS="-L${XANSYS}/lib -lpgplot -L/usr/X11R6/lib -lX11"
   else
      PGLIBS="-L${XANSYS}/lib -lpgplot -lpng -lz"
   fi
   SYSLIB="-lcc_dynamic"

elif    test "$EXT" = "sol" ; then
   MAKE="/usr/ccs/bin/make"
   export MAKE
   FCOMPL="g77"
   FCOPTS="-O -Wimplicit -fno-second-underscore -fno-backslash"
   FLOPTS=" "
   CCOMPL="gcc"
   CCOPTS="-g -O2 -DHAVE_DIRENT_H -DHAVE_STRING_H -INONE -LNONE -Dsolaris -Wl,-R/opt/SUNWspro/lib:/opt/SUNWspro/SC4.0/lib:/usr/ccs/lib:/usr/lib:${XANSYS}/lib -DEXT=sol -I$XANTOP/xanlib/cfortran -I$XANSYS/include"
   RANLIB="ranlib"
   PGLIBS="$XANSYS/lib/libpgplot.a -lX11"
   SYSLIB="-lsocket -lnsl"

elif    test "$EXT" = "sun" ; then
   MAKE="/usr/bin/make"
   export MAKE
   FCOMPL="f77"
   FCOPTS="-O -f"
   FLOPTS=" "
   CCOMPL="gcc"
   CCOPTS="-g -O2 -DHAVE_DIRENT_H -DHAVE_STRING_H -INONE -LNONE -DEXT=sun -I$XANTOP/xanlib/cfortran -I$XANSYS/include"
   RANLIB="ranlib"
   PGLIBS="$XANSYS/lib/libpgplot.a -LNONE -LNONE -lX11"
   SYSLIB="-L/usr/lang/SC1.0/cg87 -L/usr/lang/SC1.0 -ltermcap -ldl -lF77 -lm -lc -LNONE -lX11"

else
   echo Unknown system.  Using generic settings.
   FCOMPL=f77
   CCOMPL=cc
   RANLIB=ranlib

fi

export FCOMPL FCOPTS CCOMPL CCOPTS FLOPTS PGLIBS SYSLIB RANLIB

# Add $XANTOP/tools to PATH
if test "`echo $PATH | grep $XANTOP/tools`" = "" ; then
   PATH=$XANTOP/tools:$PATH
fi

# If there is a local initp file, then . it.
if test -f $XANTOP/local/tools/initp.sh ; then
   . $XANTOP/local/tools/initp.sh
fi
