      SUBROUTINE wrftp(cout, chist, nhist, ccomm, ncomm,
     &         cstday, csttim, cenday, centim, csour, ctele,
     &         cinst, cfilt, chtype, Equinx, ranom, decnom, roll,
     &         cafile, texpos, area, bscale, cscale, nchan, idtype,
     &         i2chan, pha, qerror, errors, qsys, sysfrc,
     &        qqual, i2qual, qgroup, i2grp, nextra, cextra, ier, istat)

      CHARACTER*(*) cout, chist(*), ccomm(*)
      CHARACTER*(*) cstday, csttim, cenday, centim, csour
      CHARACTER*(*) ctele, cinst, cfilt, chtype, cafile(4)
      CHARACTER cextra(*)*80
      INTEGER   nhist, ncomm, nchan, idtype, nextra, ier, istat
      INTEGER*2 i2grp(*), i2qual(*), i2chan(*)
      REAL      pha(*), errors(*), sysfrc(*)
      REAL      Equinx, ranom, decnom, roll, texpos
      REAL      area, bscale, cscale
      LOGICAL   qgroup, qqual, qerror, qsys
C---
C Write out a FITS format PHA file.
C---
C cout      I    FITS filename
C chist(*)  I    History records
C nhist     I    # of history records
C ccomm(*)  I    Comment records
C ncomm     I    # of comment records
C cstday    I    Start day of observation
C csttim    I    Start time of observation
C cenday    I    End day of observation
C centim    I    End time of observation
C csour     I    Object observed
C ctele     I    Telescope/satellite name
C cinst     I    Instrument name
C cfilt     I    Instrument filter
C chtype    I    Channel type, typically PHA or PI
C Equinx    I    Equinox used for RA/Dec
C ranom     I    observation RA (decimal degrees)
C decnom    I    observation DEC (decimal degrees)
C roll      I    observation roll angle (decimal degrees)
C cafile(4) I    Associated files (back, corr, RMF, ARF)
C texpos    I    exposure time
C area      I    Total effective area
C bscale    I    backgroHund scaling factor
C cscale    I    correction scaling factor
C nchan     I    # of channels
C idtype    I    date type  1=counts, 2=counts/sec
C i2chan(*) I    channel numbers
C pha(*)    I    data in counts/sec
C qerror    I    statistical errors included
C errors(*) I    errors in counts/sec
C qsys      I    systematic errors included
C sysfrc(*) I    fractional systematics
C qqual     I    data has quality flags
C i2qual(*) I    channel quality
C qgroup    I    data should be grouped
C i2grp(*)  I    suggested grouping
C Nextra    I    Number of extra header records to write
C Cextra    I    The extra header records, must be 80 bytes FITS formated.
C ier         O  error
C                   1 = failed to open FITS file
C                   2 = failed to write primary header
C                   3 = failed to set up SPECTRUM extension
C                   4 = failed to write XSPEC keywords
C                   5 = failed to write extra extension keywords
C                   6 = failed to write SPECTRUM data
C                   7 = failed to close FITS file
C istat        O  FITSIO status
C---
      INTEGER   LENACT
C Local variables
      CHARACTER ctmp*256
      CHARACTER*20 ttype(6), tunit(6)
      CHARACTER*5  tform(6)
      INTEGER    lun, i, tfields, colnum
C
   11 FORMAT(A)

C ------------------------------------------------------------
C Open the output FITS file.
C ------------------------------------------------------------

      CALL getlun(lun)
  130 CONTINUE
      istat = 0
      CALL ftinit(lun, cout, 1, istat)
      IF (istat .NE. 0) THEN
         IF ( istat.EQ.105 ) THEN
            CALL EDICOM('SAV', 3)
            ctmp = COUT(1:LENACT(cout))//' exists, reuse it?'
            CALL PROMPT(ctmp, 0)
            READ(*,11,ERR=150,END=150) ctmp
            CALL EDICOM('RES', 3)
            IF( ctmp(1:1).EQ.'Y' .OR. ctmp(1:1).EQ.'y') THEN
               OPEN(UNIT=lun, FILE=cout, STATUS='UNKNOWN')
               CLOSE(UNIT=lun, STATUS='DELETE')
               GOTO 130
            END IF
         END IF
  150    CONTINUE
         ier = 1
         CALL frelun(lun)
         RETURN
      END IF

C ------------------------------------------------------------
C Write the primary header
C ------------------------------------------------------------

      CALL ftpdef(lun, 8, 0, 0, 0, 1, istat)

C Write the basic primary array keywords

      CALL ftphpr(lun, .TRUE., 8, 0, 0, 0, 1, .TRUE., istat)

C Write out the additional keywords about the creation of the
C FITS file.

      CALL ftpkys(lun, 'CONTENT', 'SPECTRUM',
     & 'spectrum file contains time intervals and event', istat)
      CALL ftpkys(lun, 'ORIGIN', 'NASA/MSFC',
     & 'origin of FITS file', istat)

C Write the TELESCOP and INSTRUME keywords.

      CALL ftpkys(lun,'TELESCOP', ctele,
     &            'Telescope (mission) name',istat)
      CALL ftpkys(lun,'INSTRUME', cinst,
     &            'Instrument name',istat)

C write the object name

      CALL ftpkys(lun, 'OBJECT', csour,
     &            'Name of observed object',istat)

C write the pointing data

      CALL ftpkye (lun,'RA-NOM', ranom, 5,
     &             'Right Ascension of target (deci. deg)',istat)

      CALL ftpkye (lun,'DEC-NOM', decnom, 5,
     &             'Declination of target (deci. deg)',istat)

      CALL ftpkyf(lun, 'EQUINOX', Equinx, 1,
     &            'Equinox of celestial coord system', istat)

      CALL ftpkye (lun,'DROLLANG', roll, 5,
     &             'Mean roll angle (deci. deg)',istat)

C Write the start and end times.

      CALL ftpkys(lun, 'DATE-OBS', cstday,
     &            'Time (yyyy-mm-ddThh:mm:ss) observation began',istat)
      CALL ftpkys(lun, 'DATE-END', cenday,
     &            'Time (yyyy-mm-ddThh:mm:ss) observation ended',istat)

C Write any history records to the FITS file as history.

      DO i = 1, nhist
         CALL ftphis(lun, chist(i), istat)
      END DO

      IF (istat .NE. 0) THEN
         ier = 2
         CALL ftclos(lun, istat)
         CALL frelun(lun)
         RETURN
      ENDIF

C ----------------------------------------------------------------
C Write the data extension with the SPECTRUM data
C ----------------------------------------------------------------

C Create the data extension

      CALL ftcrhd(lun, istat)

C Set up the header keywords for a binary extension

      tfields = 2
      ttype(1) = 'CHANNEL'
      IF ( nchan.LE.32767 ) THEN
         tform(1) = 'I'
      ELSE
         tform(1) = 'J'
      END IF
      tunit(1) = ' '
      IF (idtype .EQ. 1) THEN
         ttype(2) = 'COUNTS'
         tform(2) = 'E'
         tunit(2) = 'counts'
      ELSEIF (idtype .EQ. 2) THEN
         ttype(2) = 'RATE'
         tform(2) = 'E'
         tunit(2) = 'counts/sec'
      ENDIF
      IF (qerror) THEN
         tfields = tfields + 1
         ttype(tfields) = 'STAT_ERR'
         tform(tfields) = 'E'
         tunit(tfields) = 'counts/sec'
      ENDIF
      IF (qsys) THEN
         tfields = tfields + 1
         ttype(tfields) = 'SYS_ERR'
         tform(tfields) = 'E'
         tunit(tfields) = ' '
      ENDIF
      IF (qqual) THEN
         tfields = tfields + 1
         ttype(tfields) = 'QUALITY'
         tform(tfields) = 'I'
         tunit(tfields) = ' '
      ENDIF
      IF (qgroup) THEN
         tfields = tfields + 1
         ttype(tfields) = 'GROUPING'
         tform(tfields) = 'I'
         tunit(tfields) = ' '
      ENDIF

C Write the main header keywords.

      CALL ftphbn(lun, nchan, tfields, ttype, tform, tunit,
     &            'SPECTRUM', 0, istat)

C Write the additional keywords describing the data stored

      IF (idtype .EQ. 1) THEN
         CALL ftpkyl(lun, 'POISSERR', .TRUE.,
     &               'Poissonian errors to be assumed', istat)
      ENDIF
      IF (.NOT. qerror) THEN
         CALL ftpkyj(lun, 'STAT_ERR', 0,
     &               'no statistical error specified', istat)
      ENDIF
      IF (.NOT. qsys) THEN
         CALL ftpkyj(lun, 'SYS_ERR', 0,
     &               'no systematic error specified', istat)
      ENDIF
      IF (.NOT. qgroup) THEN
         CALL ftpkyj(lun, 'GROUPING', 0,
     &               'no grouping of the data has been defined',
     &               istat)
      ENDIF
      IF (.NOT. qqual) THEN
         CALL ftpkyj(lun, 'QUALITY', 0,
     &               'no data quality information specified', istat)
      ENDIF

      IF (istat .NE. 0) THEN
         ier = 3
         CALL ftclos(lun, istat)
         CALL frelun(lun)
         RETURN
      ENDIF

C --------------------------------------------------------------
C Write the XSPEC mandatory keywords
C --------------------------------------------------------------

C Write the TELESCOP and INSTRUME keywords.

      CALL ftpkys(lun,'TELESCOP', ctele,
     &            'Telescope (mission) name',istat)
      CALL ftpkys(lun,'INSTRUME', cinst,
     &            'Instrument name',istat)


C Write the filter keyword

      CALL ftpkys(lun,'FILTER', cfilt,
     &            'Instrument filter in use',istat)

C Write the exposure keyword

      CALL ftpkye (lun,'EXPOSURE', texpos, 5,
     &             'Exposure time',istat)

C Write the area scaling factor

      CALL ftpkye (lun,'AREASCAL', area, 5,
     &             'nominal effective area',istat)

C Write the background scaling factor

      CALL ftpkye (lun,'BACKSCAL', bscale, 5,
     &             'background scale factor',istat)

C Write the correlation scaling factor

      CALL ftpkye (lun,'CORRSCAL', cscale, 5,
     &             'Correction scale factor',istat)

C Write the background, correction, RMF and ARF filenames

      CALL ftpkys(lun,'BACKFILE', cafile(1),
     &            'background FITS file for this object',istat)
      CALL ftpkys(lun,'CORRFILE', cafile(3),
     &            'correction FITS file for this object',istat)
      CALL ftpkys(lun,'RESPFILE', cafile(2),
     &            'redistribution matrix (RMF)',istat)
      CALL ftpkys(lun,'ANCRFILE', cafile(4),
     &            'ancillary response matrix (ARF)',istat)

C Write any XSPEC filter keywords

C Write the channel type

      CALL ftpkys(lun,'CHANTYPE', chtype,
     &            'Channels assigned by detector electronics',istat)

C Write the number of detector channels

      CALL ftpkyj(lun, 'DETCHANS', nchan,
     *    'Total no. detector channels available', istat)

C Write the FITS format used

      CALL ftpkys(lun,'PHAVERSN', '1992a',
     &            'OGIP classification of FITS format style',istat)

      IF (istat .NE. 0) THEN
         ier = 4
         CALL ftclos(lun, istat)
         CALL frelun(lun)
         RETURN
      ENDIF

C ----------------------------------------------------------------
C Write out optional keywords - these are mostly repeats of those
C in the primary header.
C ----------------------------------------------------------------

      CALL ftpkys(lun, 'OBJECT', csour,
     &            'Name of observed object',istat)

      CALL ftpkys(lun, 'ORIGIN', 'NASA/MSFC',
     & 'origin of FITS file', istat)

      CALL ftpdat(lun, istat)

      CALL ftpkye (lun,'RA-NOM', ranom, 5,
     &             'Right Ascension of target (deci. deg)',istat)

      CALL ftpkye (lun,'DEC-NOM', decnom, 5,
     &             'Declination of target (deci. deg)',istat)

      CALL ftpkyf(lun, 'EQUINOX', Equinx, 1,
     &            'Equinox of celestial coord system', istat)

      CALL ftpkye (lun,'DROLLANG', roll, 5,
     &             'Mean roll angle (deci. deg)',istat)

      CALL ftpkys(lun, 'DATE-OBS', cstday,
     &            'Time (yyyy-mm-ddThh:mm:ss) observation began',istat)
      CALL ftpkys(lun, 'DATE-END', cenday,
     &            'Time (yyyy-mm-ddThh:mm:ss) observation ended',istat)

C Write any comment records using COMMENT keyword

      DO i = 1, ncomm
         CALL ftpcom(lun, ccomm(i), istat)
      ENDDO

C Write any history records using HISTORY keyword

      DO i = 1, nhist
         CALL ftphis(lun, chist(i), istat)
      ENDDO

      IF (istat .NE. 0) THEN
         ier = 5
         CALL ftclos(lun, istat)
         CALL frelun(lun)
         RETURN
      ENDIF

C --------------------------------------------------------------
C Write out the SPECTRUM data
C --------------------------------------------------------------

C define the binary table

      CALL ftbdef(lun, tfields, tform, 0, nchan, istat)

C write the elements into the table

      IF ( nchan.LE.32767 ) THEN
         CALL ftpcli(lun, 1, 1, 1, nchan, i2chan, istat)
      ELSE
         DO i=1,nchan
            CALL ftpclj(lun, 1, I, 1, 1, I, istat)
         END DO
      END IF

      CALL ftpcle(lun, 2, 1, 1, nchan, pha, istat)

      colnum = 2
      IF (qerror) THEN
         colnum = colnum + 1
         CALL ftpcle(lun, colnum, 1, 1, nchan, errors, istat)
      ENDIF
      IF (qsys) THEN
         colnum = colnum + 1
         CALL ftpcle(lun, colnum, 1, 1, nchan, sysfrc, istat)
      ENDIF
      IF (qqual) THEN
         colnum = colnum + 1
         CALL ftpcli(lun, colnum, 1, 1, nchan, i2qual, istat)
      ENDIF
      IF (qgroup) THEN
         colnum = colnum + 1
         CALL ftpcli(lun, colnum, 1, 1, nchan, i2grp, istat)
      ENDIF

      IF (istat .NE. 0) THEN
         ier = 6
         CALL ftclos(lun, istat)
         CALL frelun(lun)
         RETURN
      ENDIF

      DO i=1,nextra
         CALL FTPREC(lun, cextra(i), istat)
      END DO

C ---------------------------------------------------------------
C Close the FITS file
C ---------------------------------------------------------------

      CALL ftclos(lun, istat)
      CALL frelun(lun)

      IF (istat .NE. 0) THEN
         ier = 7
         RETURN
      ENDIF

      RETURN
      END
