$!----------------------------------------------------------------------
$! Use this file to create a PLT sharable user function UFNYSHR.
$!
$! If no argument is given this command file will use UFNYDEMO.F
$! and create both UFNYSHR.EXE and UFNYSHR.OLB.  All software that uses
$! a PLT user model can then be linked with this .OLB library.  To run
$! a program linked with the sharable, you must define the logical name
$! UFNUSHR to point to the directory containing the .EXE file.
$!
$! To create a new user function, invoke this command file with an
$! argument.  For example if your user function is in a file GAUS.F
$! then use:
$! $ @MKUFNY GAUS
$! This will only create the UFNYSHR.EXE file (continue to link with
$! the .OLB library created with UFNYDEMO.FOR).  To cause the new version
$! of UFNYSHR to be loaded at run time you must redefine the logical
$! to point to the directory containing the new UFNYSHR.EXE file.
$! If your current directory contains a option file (for example,
$! GAUS.OPT) then that file will be included in the link.  If you use
$! a common block in the sharable, then you must include the following
$! in an option file for each common block:
$! PSECT = comnam ,LCL,NOSHR
$! where comnam should be replaced with the Fortran name of the common
$! block.  Failing to do this will result in the fatal error at run
$! time that says "writable shareable images must be installed".
$!
$! Based on T.J. Pearson's routine BUILD.COM that creates a similar
$! sharable image library for the PGPLOT.
$!----------------------------------------------------------------------
$ ON WARNING THEN EXIT
$ file    := UFNYDEMO
$ outfile := UFNYSHR
$ IF P1.NES."" THEN file="''p1'"
$ IF P1.NES."" THEN outfile="''p1'"
$ WRITE SYS$OUTPUT "Creating ''file' user function for PLT."
$ XFOR 'FILE'
$!
$! Create a module containing transfer vectors into the shareable image.
$! This can only be done in MACRO.
$!
$ MACRO/OBJECT=UFNYTRN SYS$INPUT
	.TITLE Transfer Vectors for PLT user model
	.IDENT /1-1/
;
; This module contains all the transfer vectors into the shareable
; image UFNYSHR.  Only user-callable routines have transfer vectors.
; If you change this file and want it to be compatible with a previous
; version, you must add transfer vectors only at the end of the list.
; If you remove a transfer vector, replace it with a pointer to an
; error processing routine. 
;
	.PSECT $TRANS,EXE,NOWRT,PIC,SHR,GBL

        .MACRO TRANS UNAME,LNAME
	.TRANSFER    UNAME
 	.IF NB       LNAME
	.MASK        LNAME
	JMP          LNAME+2
        .IFF
	.MASK        UNAME
        JMP          UNAME+2
        .ENDC
        .ENDM        TRANS

	TRANS	UINFO
	TRANS	ULIMIT
	TRANS	UFNY
	TRANS	UDERIV
;
	.END
$!
$! Use the linker to create the shareable image, including the
$! transfer vector defined above.
$!
$ IF F$SEARCH("''file'.OPT").NES."" THEN optfile=",''file'/OPT"
$ LINK/NOUSER-
      /SHARE='outfile'.EXE-
      'FILE' 'optfile',SYS$INPUT/OPT
! The following defines the version number of the shareable image.
GSMATCH = LEQUAL,1,1
CLUSTER = $TRANS,,,UFNYTRN.OBJ
$!
$ DELETE/NOCONFIRM UFNYTRN.OBJ;*
$!
$ IF "''file'".NES."UFNYDEMO"
$ THEN
$      DELETE/NOCONFIRM 'file'.OBJ;*
$      WRITE SYS$OUTPUT "Don't forget to run SETUFNY."
$ ELSE
$!
$! Create a shareable image symbol table library.  This library
$! (UFNYSHR.OLB) should be linked with the user's program in order
$! to use the shareable image.
$!
$      LIBRARY/CREATE=(BLOCK:6,MOD:1,GLOB:8)/SHARE/LOG UFNYSHR.OLB UFNYSHR.EXE
$      PURGE/LOG UFNYSHR.OLB
$ ENDIF
